/*!
 * @feng3d/data-structures-and-algorithms - v0.1.0
 * Compiled Sun, 05 Jun 2022 09:12:02 UTC
 *
 * @feng3d/data-structures-and-algorithms is licensed under the MIT License.
 * http://www.opensource.org/licenses/mit-license
 */
/******************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */
/* global Reflect, Promise */

var extendStatics = function(d, b) {
    extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) { if (Object.prototype.hasOwnProperty.call(b, p)) { d[p] = b[p]; } } };
    return extendStatics(d, b);
};

function __extends(d, b) {
    if (typeof b !== "function" && b !== null)
        { throw new TypeError("Class extends value " + String(b) + " is not a constructor or null"); }
    extendStatics(d, b);
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
}

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        var arguments$1 = arguments;

        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments$1[i];
            for (var p in s) { if (Object.prototype.hasOwnProperty.call(s, p)) { t[p] = s[p]; } }
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

function __read(o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) { return o; }
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) { ar.push(r.value); }
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) { m.call(i); }
        }
        finally { if (e) { throw e.error; } }
    }
    return ar;
}

function __spreadArray(to, from, pack) {
    if (pack || arguments.length === 2) { for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) { ar = Array.prototype.slice.call(from, 0, i); }
            ar[i] = from[i];
        }
    } }
    return to.concat(ar || Array.prototype.slice.call(from));
}

/**
 * Gets the matrix's shape.
 *
 * @param m
 */
function shape(m) {
    var shapes = [];
    var dimension = m;
    while (dimension && Array.isArray(dimension)) {
        shapes.push(dimension.length);
        dimension = (dimension.length && __spreadArray([], __read(dimension))[0]) || null;
    }
    return shapes;
}
/**
 * Checks if matrix has a correct type.
 *
 * @throws Error
 */
function validateType(m) {
    if (!m
        || !Array.isArray(m)
        || !Array.isArray(m[0])) {
        throw new Error('Invalid matrix format');
    }
}
/**
 * Checks if matrix is two dimensional.
 *
 * @param m
 * @throws Error
 */
function validate2D(m) {
    validateType(m);
    var aShape = shape(m);
    if (aShape.length !== 2) {
        throw new Error('Matrix is not of 2D shape');
    }
}
/**
 * Validates that matrices are of the same shape.
 *
 * @param a
 * @param  b
 * @throws Error
 */
function validateSameShape(a, b) {
    validateType(a);
    validateType(b);
    var aShape = shape(a);
    var bShape = shape(b);
    if (aShape.length !== bShape.length) {
        throw new Error('Matrices have different dimensions');
    }
    while (aShape.length && bShape.length) {
        if (aShape.pop() !== bShape.pop()) {
            throw new Error('Matrices have different shapes');
        }
    }
}
/**
 * Generates the matrix of specific shape with specific values.
 *
 * @param mShape the shape of the matrix to generate
 * @param fill cell values of a generated matrix.
 */
function generate(mShape, fill) {
    /**
     * Generates the matrix recursively.
     *
     * @param recShape the shape of the matrix to generate
     * @param recIndices
     */
    function generateRecursively(recShape, recIndices) {
        if (recShape.length === 1) {
            return Array(recShape[0])
                .fill(null)
                .map(function (cellValue, cellIndex) { return fill(__spreadArray(__spreadArray([], __read(recIndices)), [cellIndex])); });
        }
        var m = [];
        for (var i = 0; i < recShape[0]; i += 1) {
            m.push(generateRecursively(recShape.slice(1), __spreadArray(__spreadArray([], __read(recIndices)), [i])));
        }
        return m;
    }
    return generateRecursively(mShape, []);
}
/**
 * Generates the matrix of zeros of specified shape.
 *
 * @param mShape shape of the matrix
 */
function zeros(mShape) {
    return generate(mShape, function () { return 0; });
}
/**
 * @param a
 * @param b
 * @throws Error
 */
function dot(a, b) {
    // Validate inputs.
    validate2D(a);
    validate2D(b);
    // Check dimensions.
    var aShape = shape(a);
    var bShape = shape(b);
    if (aShape[1] !== bShape[0]) {
        throw new Error('Matrices have incompatible shape for multiplication');
    }
    // Perform matrix multiplication.
    var outputShape = [aShape[0], bShape[1]];
    var c = zeros(outputShape);
    for (var bCol = 0; bCol < b[0].length; bCol += 1) {
        for (var aRow = 0; aRow < a.length; aRow += 1) {
            var cellSum = 0;
            for (var aCol = 0; aCol < a[aRow].length; aCol += 1) {
                cellSum += a[aRow][aCol] * b[aCol][bCol];
            }
            c[aRow][bCol] = cellSum;
        }
    }
    return c;
}
/**
 * Transposes the matrix.
 *
 * @param m
 * @throws Error
 */
function t(m) {
    validate2D(m);
    var mShape = shape(m);
    var transposed = zeros([mShape[1], mShape[0]]);
    for (var row = 0; row < m.length; row += 1) {
        for (var col = 0; col < m[0].length; col += 1) {
            transposed[col][row] = m[row][col];
        }
    }
    return transposed;
}
/**
 * Traverses the matrix.
 *
 * @param m
 * @param visit
 */
function walk(m, visit) {
    /**
     * Traverses the matrix recursively.
     *
     * @param recM
     * @param cellIndices
     */
    function recWalk(recM, cellIndices) {
        var recMShape = shape(recM);
        if (recMShape.length === 1) {
            for (var i = 0; i < recM.length; i += 1) {
                visit(__spreadArray(__spreadArray([], __read(cellIndices)), [i]), recM[i]);
            }
        }
        for (var i = 0; i < recM.length; i += 1) {
            recWalk(recM[i], __spreadArray(__spreadArray([], __read(cellIndices)), [i]));
        }
    }
    recWalk(m, []);
}
/**
 * Gets the matrix cell value at specific index.
 *
 * @param m Matrix that contains the cell that needs to be updated
 * @param cellIndices Array of cell indices
 */
function getCellAtIndex(m, cellIndices) {
    // We start from the row at specific index.
    var cell = m[cellIndices[0]];
    // Going deeper into the next dimensions but not to the last one to preserve
    // the pointer to the last dimension array.
    for (var dimIdx = 1; dimIdx < cellIndices.length - 1; dimIdx += 1) {
        cell = cell[cellIndices[dimIdx]];
    }
    // At this moment the cell variable points to the array at the last needed dimension.
    return cell[cellIndices[cellIndices.length - 1]];
}
/**
 * Update the matrix cell at specific index.
 *
 * @param m Matrix that contains the cell that needs to be updated
 * @param cellIndices Array of cell indices
 * @param cellValue New cell value
 */
function updateCellAtIndex(m, cellIndices, cellValue) {
    // We start from the row at specific index.
    var cell = m[cellIndices[0]];
    // Going deeper into the next dimensions but not to the last one to preserve
    // the pointer to the last dimension array.
    for (var dimIdx = 1; dimIdx < cellIndices.length - 1; dimIdx += 1) {
        cell = cell[cellIndices[dimIdx]];
    }
    // At this moment the cell variable points to the array at the last needed dimension.
    cell[cellIndices[cellIndices.length - 1]] = cellValue;
}
/**
 * Adds two matrices element-wise.
 *
 * @param a
 * @param b
 */
function add(a, b) {
    validateSameShape(a, b);
    var result = zeros(shape(a));
    walk(a, function (cellIndices, cellValue) {
        updateCellAtIndex(result, cellIndices, cellValue);
    });
    walk(b, function (cellIndices, cellValue) {
        var currentCellValue = getCellAtIndex(result, cellIndices);
        updateCellAtIndex(result, cellIndices, currentCellValue + cellValue);
    });
    return result;
}
/**
 * Multiplies two matrices element-wise.
 *
 * @param a
 * @param b
 */
function mul(a, b) {
    validateSameShape(a, b);
    var result = zeros(shape(a));
    walk(a, function (cellIndices, cellValue) {
        updateCellAtIndex(result, cellIndices, cellValue);
    });
    walk(b, function (cellIndices, cellValue) {
        var currentCellValue = getCellAtIndex(result, cellIndices);
        updateCellAtIndex(result, cellIndices, currentCellValue * cellValue);
    });
    return result;
}
/**
 * Subtract two matrices element-wise.
 *
 * @param a
 * @param b
 */
function sub(a, b) {
    validateSameShape(a, b);
    var result = zeros(shape(a));
    walk(a, function (cellIndices, cellValue) {
        updateCellAtIndex(result, cellIndices, cellValue);
    });
    walk(b, function (cellIndices, cellValue) {
        var currentCellValue = getCellAtIndex(result, cellIndices);
        updateCellAtIndex(result, cellIndices, currentCellValue - cellValue);
    });
    return result;
}

var Matrix = {
    __proto__: null,
    shape: shape,
    validateSameShape: validateSameShape,
    generate: generate,
    zeros: zeros,
    dot: dot,
    t: t,
    walk: walk,
    getCellAtIndex: getCellAtIndex,
    updateCellAtIndex: updateCellAtIndex,
    add: add,
    mul: mul,
    sub: sub
};

// Create alphabet array: ['a', 'b', 'c', ..., 'z'].
var englishAlphabet = 'abcdefghijklmnopqrstuvwxyz'.split('');
/**
 * Generates a cipher map out of the alphabet.
 * Example with a shift 3: {'a': 'd', 'b': 'e', 'c': 'f', ...}
 *
 * @param alphabet i.e. ['a', 'b', 'c', ... , 'z']
 * @param shift i.e. 3
 * @return i.e. {'a': 'd', 'b': 'e', 'c': 'f', ..., 'z': 'c'}
 */
var getCipherMap = function (alphabet, shift) {
    return alphabet
        .reduce(function (charsMap, currentChar, charIndex) {
        var charsMapClone = __assign({}, charsMap);
        // Making the shift to be cyclic (i.e. with a shift of 1 - 'z' would be mapped to 'a').
        var encryptedCharIndex = (charIndex + shift) % alphabet.length;
        // Support negative shifts for creating a map for decryption
        // (i.e. with shift -1 - 'a' would be mapped to 'z').
        if (encryptedCharIndex < 0) {
            encryptedCharIndex += alphabet.length;
        }
        charsMapClone[currentChar] = alphabet[encryptedCharIndex];
        return charsMapClone;
    }, {});
};
/**
 * 凯撒密码加密
 *
 * @param str
 * @param shift
 * @param alphabet
 */
var caesarCipherEncrypt = function (str, shift, alphabet) {
    if (alphabet === void 0) { alphabet = englishAlphabet; }
    // Create a cipher map:
    var cipherMap = getCipherMap(alphabet, shift);
    return str
        .toLowerCase()
        .split('')
        .map(function (char) { return cipherMap[char] || char; })
        .join('');
};
/**
 * 凯撒密码解密
 *
 * @param str
 * @param shift
 * @param alphabet
 */
var caesarCipherDecrypt = function (str, shift, alphabet) {
    if (alphabet === void 0) { alphabet = englishAlphabet; }
    // Create a cipher map:
    var cipherMap = getCipherMap(alphabet, -shift);
    return str
        .toLowerCase()
        .split('')
        .map(function (char) { return cipherMap[char] || char; })
        .join('');
};

// The code of an 'A' character (equals to 65).
var alphabetCodeShift = 'A'.codePointAt(0);
var englishAlphabetSize = 26;
/**
 * Generates key matrix from given keyString.
 *
 * @param keyString a string to build a key matrix (must be of matrixSize^2 length).
 */
function generateKeyMatrix(keyString) {
    var matrixSize = Math.sqrt(keyString.length);
    if (!Number.isInteger(matrixSize)) {
        throw new Error('Invalid key string length. The square root of the key string must be an integer');
    }
    var keyStringIndex = 0;
    return generate([matrixSize, matrixSize], 
    // Callback to get a value of each matrix cell.
    // The order the matrix is being filled in is from left to right, from top to bottom.
    function () {
        // A → 0, B → 1, ..., a → 32, b → 33, ...
        var charCodeShifted = (keyString.codePointAt(keyStringIndex)) % alphabetCodeShift;
        keyStringIndex += 1;
        return charCodeShifted;
    });
}
/**
 * Generates a message vector from a given message.
 *
 * @param message - the message to encrypt.
 */
function generateMessageVector(message) {
    return generate([message.length, 1], 
    // Callback to get a value of each matrix cell.
    // The order the matrix is being filled in is from left to right, from top to bottom.
    function (cellIndices) {
        var rowIndex = cellIndices[0];
        return message.codePointAt(rowIndex) % alphabetCodeShift;
    });
}
/**
 * Encrypts the given message using Hill Cipher.
 *
 * @param message plaintext
 * @param keyString
 */
function hillCipherEncrypt(message, keyString) {
    // The keyString and message can only contain letters.
    var onlyLettersRegExp = /^[a-zA-Z]+$/;
    if (!onlyLettersRegExp.test(message) || !onlyLettersRegExp.test(keyString)) {
        throw new Error('The message and key string can only contain letters');
    }
    var keyMatrix = generateKeyMatrix(keyString);
    var messageVector = generateMessageVector(message);
    // keyString.length must equal to square of message.length
    if (keyMatrix.length !== message.length) {
        throw new Error('Invalid key string length. The key length must be a square of message length');
    }
    var cipherVector = dot(keyMatrix, messageVector);
    var cipherString = '';
    for (var row = 0; row < cipherVector.length; row += 1) {
        var item = cipherVector[row];
        cipherString += String.fromCharCode((item % englishAlphabetSize) + alphabetCodeShift);
    }
    return cipherString;
}
// @TODO: Implement this method.
function hillCipherDecrypt() {
    throw new Error('This method is not implemented yet');
}

var DEFAULT_BASE$1 = 37;
var DEFAULT_MODULUS = 101;
var PolynomialHash = /** @class */ (function () {
    function PolynomialHash(_a) {
        var _b = _a === void 0 ? {} : _a, _c = _b.base, base = _c === void 0 ? DEFAULT_BASE$1 : _c, _d = _b.modulus, modulus = _d === void 0 ? DEFAULT_MODULUS : _d;
        this.base = base;
        this.modulus = modulus;
    }
    /**
     * Function that creates hash representation of the word.
     *
     * Time complexity: O(word.length).
     *
     * @param word String that needs to be hashed.
     */
    PolynomialHash.prototype.hash = function (word) {
        var _this = this;
        var charCodes = Array.from(word).map(function (char) { return _this.charToNumber(char); });
        var hash = 0;
        for (var charIndex = 0; charIndex < charCodes.length; charIndex += 1) {
            hash *= this.base;
            hash += charCodes[charIndex];
            hash %= this.modulus;
        }
        return hash;
    };
    /**
     * Function that creates hash representation of the word
     * based on previous word (shifted by one character left) hash value.
     *
     * Recalculates the hash representation of a word so that it isn't
     * necessary to traverse the whole word again.
     *
     * Time complexity: O(1).
     *
     * @param prevHash
     * @param prevWord
     * @param newWord
     */
    PolynomialHash.prototype.roll = function (prevHash, prevWord, newWord) {
        var hash = prevHash;
        var prevValue = this.charToNumber(prevWord[0]);
        var newValue = this.charToNumber(newWord[newWord.length - 1]);
        var prevValueMultiplier = 1;
        for (var i = 1; i < prevWord.length; i += 1) {
            prevValueMultiplier *= this.base;
            prevValueMultiplier %= this.modulus;
        }
        hash += this.modulus;
        hash -= (prevValue * prevValueMultiplier) % this.modulus;
        hash *= this.base;
        hash += newValue;
        hash %= this.modulus;
        return hash;
    };
    /**
     * Converts char to number.
     *
     * @param char
     */
    PolynomialHash.prototype.charToNumber = function (char) {
        var charCode = char.codePointAt(0);
        // Check if character has surrogate pair.
        var surrogate = char.codePointAt(1);
        if (surrogate !== undefined) {
            var surrogateShift = Math.pow(2, 16);
            charCode += surrogate * surrogateShift;
        }
        return charCode;
    };
    return PolynomialHash;
}());

var DEFAULT_BASE = 17;
var SimplePolynomialHash = /** @class */ (function () {
    /**
     * @param base Base number that is used to create the polynomial.
     */
    function SimplePolynomialHash(base) {
        if (base === void 0) { base = DEFAULT_BASE; }
        this.base = base;
    }
    /**
     * Function that creates hash representation of the word.
     *
     * Time complexity: O(word.length).
     *
     * @assumption: This version of the function  doesn't use modulo operator.
     * Thus it may produce number overflows by generating numbers that are
     * bigger than Number.MAX_SAFE_INTEGER. This function is mentioned here
     * for simplicity and LEARNING reasons.
     *
     * @param word String that needs to be hashed.
     */
    SimplePolynomialHash.prototype.hash = function (word) {
        var hash = 0;
        for (var charIndex = 0; charIndex < word.length; charIndex += 1) {
            hash += word.charCodeAt(charIndex) * (Math.pow(this.base, charIndex));
        }
        return hash;
    };
    /**
     * Function that creates hash representation of the word
     * based on previous word (shifted by one character left) hash value.
     *
     * Recalculates the hash representation of a word so that it isn't
     * necessary to traverse the whole word again.
     *
     * Time complexity: O(1).
     *
     * @assumption: This function doesn't use modulo operator and thus is not safe since
     * it may deal with numbers that are bigger than Number.MAX_SAFE_INTEGER. This
     * function is mentioned here for simplicity and LEARNING reasons.
     *
     * @param prevHash
     * @param prevWord
     * @param newWord
     */
    SimplePolynomialHash.prototype.roll = function (prevHash, prevWord, newWord) {
        var hash = prevHash;
        var prevValue = prevWord.charCodeAt(0);
        var newValue = newWord.charCodeAt(newWord.length - 1);
        hash -= prevValue;
        hash /= this.base;
        hash += newValue * (Math.pow(this.base, (newWord.length - 1)));
        return hash;
    };
    return SimplePolynomialHash;
}());

/**
 *
 */
var DIRECTIONS = { UP: -1, DOWN: 1 };
/**
 * Builds a fence with a specific number of rows.
 *
 * @param rowsNum
 */
function buildFence(rowsNum) {
    return Array(rowsNum).fill(null).map(function () { return []; });
}
/**
 * Get next direction to move (based on the current one) while traversing the fence.
 *
 * @returns The next direction to take
 */
function getNextDirection(_a) {
    var railCount = _a.railCount, currentRail = _a.currentRail, direction = _a.direction;
    switch (currentRail) {
        case 0:
            // Go down if we're on top of the fence.
            return DIRECTIONS.DOWN;
        case railCount - 1:
            // Go up if we're at the bottom of the fence.
            return DIRECTIONS.UP;
        default:
            // Continue with the same direction if we're in the middle of the fence.
            return direction;
    }
}
/**
 * @param targetRailIndex
 * @param letter
 */
function addCharToRail(targetRailIndex, letter) {
    /**
     * Given a rail, adds a char to it if it matches a targetIndex.
     *
     * @param rail
     * @param currentRail
     */
    function onEachRail(rail, currentRail) {
        return currentRail === targetRailIndex
            ? __spreadArray(__spreadArray([], __read(rail)), [letter]) : rail;
    }
    return onEachRail;
}
/**
 * Hangs the characters on the fence.
 */
function fillEncodeFence(_a) {
    var fence = _a.fence, currentRail = _a.currentRail, direction = _a.direction, chars = _a.chars;
    if (chars.length === 0) {
        // All chars have been placed on a fence.
        return fence;
    }
    var railCount = fence.length;
    // Getting the next character to place on a fence.
    var _b = __read(chars), letter = _b[0], nextChars = _b.slice(1);
    var nextDirection = getNextDirection({
        railCount: railCount,
        currentRail: currentRail,
        direction: direction,
    });
    return fillEncodeFence({
        fence: fence.map(addCharToRail(currentRail, letter)),
        currentRail: currentRail + nextDirection,
        direction: nextDirection,
        chars: nextChars,
    });
}
/**
 * @param params
 */
function fillDecodeFence(params) {
    var strLen = params.strLen, chars = params.chars, fence = params.fence, targetRail = params.targetRail, direction = params.direction, coords = params.coords;
    var railCount = fence.length;
    if (chars.length === 0) {
        return fence;
    }
    var _a = __read(coords, 2), currentRail = _a[0], currentColumn = _a[1];
    var shouldGoNextRail = currentColumn === strLen - 1;
    var nextDirection = shouldGoNextRail
        ? DIRECTIONS.DOWN
        : getNextDirection({ railCount: railCount, currentRail: currentRail, direction: direction });
    var nextRail = shouldGoNextRail ? targetRail + 1 : targetRail;
    var nextCoords = [
        shouldGoNextRail ? 0 : currentRail + nextDirection,
        shouldGoNextRail ? 0 : currentColumn + 1 ];
    var shouldAddChar = currentRail === targetRail;
    var _b = __read(chars), currentChar = _b[0], remainderChars = _b.slice(1);
    var nextString = shouldAddChar ? remainderChars : chars;
    var nextFence = shouldAddChar ? fence.map(addCharToRail(currentRail, currentChar)) : fence;
    return fillDecodeFence({
        strLen: strLen,
        chars: nextString,
        fence: nextFence,
        targetRail: nextRail,
        direction: nextDirection,
        coords: nextCoords,
    });
}
/**
 * @param params
 */
function decodeFence(params) {
    var strLen = params.strLen, fence = params.fence, currentRail = params.currentRail, direction = params.direction, code = params.code;
    if (code.length === strLen) {
        return code.join('');
    }
    var railCount = fence.length;
    var _a = __read(fence[currentRail]), currentChar = _a[0], nextRail = _a.slice(1);
    var nextDirection = getNextDirection({ railCount: railCount, currentRail: currentRail, direction: direction });
    return decodeFence({
        railCount: railCount,
        strLen: strLen,
        currentRail: currentRail + nextDirection,
        direction: nextDirection,
        code: __spreadArray(__spreadArray([], __read(code)), [currentChar]),
        fence: fence.map(function (rail, idx) { return (idx === currentRail ? nextRail : rail); }),
    });
}
/**
 * Encodes the message using Rail Fence Cipher.
 *
 * @param string The string to be encoded
 * @param railCount The number of rails in a fence
 * @returns Encoded string
 */
function encodeRailFenceCipher(string, railCount) {
    var fence = buildFence(railCount);
    var filledFence = fillEncodeFence({
        fence: fence,
        currentRail: 0,
        direction: DIRECTIONS.DOWN,
        chars: string.split(''),
    });
    return filledFence.flat().join('');
}
/**
 * Decodes the message using Rail Fence Cipher.
 *
 * @param string Encoded string
 * @param railCount The number of rows in a fence
 * @returns Decoded string.
 */
function decodeRailFenceCipher(string, railCount) {
    var strLen = string.length;
    var emptyFence = buildFence(railCount);
    var filledFence = fillDecodeFence({
        strLen: strLen,
        chars: string.split(''),
        fence: emptyFence,
        targetRail: 0,
        direction: DIRECTIONS.DOWN,
        coords: [0, 0],
    });
    return decodeFence({
        strLen: strLen,
        fence: filledFence,
        currentRail: 0,
        direction: DIRECTIONS.DOWN,
        code: [],
    });
}

/**
 * @param callbacks
 */
function initCallbacks$3(callbacks) {
    if (callbacks === void 0) { callbacks = {}; }
    var initiatedCallback = callbacks;
    var stubCallback = function () { };
    var allowTraversalCallback = (function () {
        var seen = {};
        return function (_a) {
            var nextVertex = _a.nextVertex;
            if (!seen[nextVertex.getKey()]) {
                seen[nextVertex.getKey()] = true;
                return true;
            }
            return false;
        };
    })();
    initiatedCallback.allowTraversal = callbacks.allowTraversal || allowTraversalCallback;
    initiatedCallback.enterVertex = callbacks.enterVertex || stubCallback;
    initiatedCallback.leaveVertex = callbacks.leaveVertex || stubCallback;
    return initiatedCallback;
}
/**
 * @param graph
 * @param currentVertex
 * @param previousVertex
 * @param callbacks
 */
function depthFirstSearchRecursive$1(graph, currentVertex, previousVertex, callbacks) {
    callbacks.enterVertex({ currentVertex: currentVertex, previousVertex: previousVertex });
    graph.getNeighbors(currentVertex).forEach(function (nextVertex) {
        if (callbacks.allowTraversal({ previousVertex: previousVertex, currentVertex: currentVertex, nextVertex: nextVertex })) {
            depthFirstSearchRecursive$1(graph, nextVertex, currentVertex, callbacks);
        }
    });
    callbacks.leaveVertex({ currentVertex: currentVertex, previousVertex: previousVertex });
}
/**
 * @param graph
 * @param startVertex
 * @param callbacks
 */
function depthFirstSearch(graph, startVertex, callbacks) {
    var previousVertex = null;
    depthFirstSearchRecursive$1(graph, startVertex, previousVertex, initCallbacks$3(callbacks));
}

/**
 * Helper class for visited vertex metadata.
 */
var VisitMetadata$1 = /** @class */ (function () {
    function VisitMetadata(_a) {
        var discoveryTime = _a.discoveryTime, lowDiscoveryTime = _a.lowDiscoveryTime;
        this.discoveryTime = discoveryTime;
        this.lowDiscoveryTime = lowDiscoveryTime;
        // We need this in order to check graph root node, whether it has two
        // disconnected children or not.
        this.independentChildrenCount = 0;
    }
    return VisitMetadata;
}());
/**
 * Tarjan's algorithm for finding articulation points in graph.
 *
 * @param graph
 */
function articulationPoints(graph) {
    // Set of vertices we've already visited during DFS.
    var visitedSet = {};
    // Set of articulation points.
    var articulationPointsSet = {};
    // Time needed to discover to the current vertex.
    var discoveryTime = 0;
    // Peek the start vertex for DFS traversal.
    var startVertex = graph.getAllVertices()[0];
    var dfsCallbacks = {
        enterVertex: function (_a) {
            var currentVertex = _a.currentVertex, previousVertex = _a.previousVertex;
            // Tick discovery time.
            discoveryTime += 1;
            // Put current vertex to visited set.
            visitedSet[currentVertex.getKey()] = new VisitMetadata$1({
                discoveryTime: discoveryTime,
                lowDiscoveryTime: discoveryTime,
            });
            if (previousVertex) {
                // Update children counter for previous vertex.
                visitedSet[previousVertex.getKey()].independentChildrenCount += 1;
            }
        },
        /**
         */
        leaveVertex: function (_a) {
            var currentVertex = _a.currentVertex, previousVertex = _a.previousVertex;
            if (previousVertex === null) {
                // Don't do anything for the root vertex if it is already current (not previous one)
                return;
            }
            // Update the low time with the smallest time of adjacent vertices.
            // Get minimum low discovery time from all neighbors.
            visitedSet[currentVertex.getKey()].lowDiscoveryTime = currentVertex.getNeighbors()
                .filter(function (earlyNeighbor) { return earlyNeighbor.getKey() !== previousVertex.getKey(); })
                .reduce(function (lowestDiscoveryTime, neighbor) {
                var neighborLowTime = visitedSet[neighbor.getKey()].lowDiscoveryTime;
                return neighborLowTime < lowestDiscoveryTime ? neighborLowTime : lowestDiscoveryTime;
            }, visitedSet[currentVertex.getKey()].lowDiscoveryTime);
            // Detect whether previous vertex is articulation point or not.
            // To do so we need to check two [OR] conditions:
            // 1. Is it a root vertex with at least two independent children.
            // 2. If its visited time is <= low time of adjacent vertex.
            if (previousVertex === startVertex) {
                // Check that root vertex has at least two independent children.
                if (visitedSet[previousVertex.getKey()].independentChildrenCount >= 2) {
                    articulationPointsSet[previousVertex.getKey()] = previousVertex;
                }
            }
            else {
                // Get current vertex low discovery time.
                var currentLowDiscoveryTime = visitedSet[currentVertex.getKey()].lowDiscoveryTime;
                // Compare current vertex low discovery time with parent discovery time. Check if there
                // are any short path (back edge) exists. If we can't get to current vertex other then
                // via parent then the parent vertex is articulation point for current one.
                var parentDiscoveryTime = visitedSet[previousVertex.getKey()].discoveryTime;
                if (parentDiscoveryTime <= currentLowDiscoveryTime) {
                    articulationPointsSet[previousVertex.getKey()] = previousVertex;
                }
            }
        },
        allowTraversal: function (_a) {
            var nextVertex = _a.nextVertex;
            return !visitedSet[nextVertex.getKey()];
        },
    };
    // Do Depth First Search traversal over submitted graph.
    depthFirstSearch(graph, startVertex, dfsCallbacks);
    return articulationPointsSet;
}

/**
 * @param graph
 * @param startVertex
 */
function bellmanFord(graph, startVertex) {
    var distances = {};
    var previousVertices = {};
    // Init all distances with infinity assuming that currently we can't reach
    // any of the vertices except start one.
    distances[startVertex.getKey()] = 0;
    graph.getAllVertices().forEach(function (vertex) {
        previousVertices[vertex.getKey()] = null;
        if (vertex.getKey() !== startVertex.getKey()) {
            distances[vertex.getKey()] = Infinity;
        }
    });
    // We need (|V| - 1) iterations.
    for (var iteration = 0; iteration < (graph.getAllVertices().length - 1); iteration += 1) {
        // During each iteration go through all vertices.
        Object.keys(distances).forEach(function (vertexKey) {
            var vertex = graph.getVertexByKey(vertexKey);
            // Go through all vertex edges.
            graph.getNeighbors(vertex).forEach(function (neighbor) {
                var edge = graph.findEdge(vertex, neighbor);
                // Find out if the distance to the neighbor is shorter in this iteration
                // then in previous one.
                var distanceToVertex = distances[vertex.getKey()];
                var distanceToNeighbor = distanceToVertex + edge.weight;
                if (distanceToNeighbor < distances[neighbor.getKey()]) {
                    distances[neighbor.getKey()] = distanceToNeighbor;
                    previousVertices[neighbor.getKey()] = vertex;
                }
            });
        });
    }
    return {
        distances: distances,
        previousVertices: previousVertices,
    };
}

/**
 * 链表结点
 */
var LinkedListNode = /** @class */ (function () {
    function LinkedListNode(value, next) {
        if (next === void 0) { next = null; }
        this.value = value;
        this.next = next;
    }
    LinkedListNode.prototype.toString = function (callback) {
        return callback ? callback(this.value) : "" + this.value;
    };
    return LinkedListNode;
}());

/**
 * 比较器
 */
var Comparator = /** @class */ (function () {
    /**
     * Constructor.
     *
     * @param compareFunction It may be custom compare function that, let's say may compare custom objects together.
     */
    function Comparator(compareFunction) {
        this.compare = compareFunction || Comparator.defaultCompareFunction;
    }
    /**
     * Default comparison function. It just assumes that "a" and "b" are strings or numbers.
     * @param a
     * @param b
     * @returns
     */
    Comparator.defaultCompareFunction = function (a, b) {
        if (a === b) {
            return 0;
        }
        return a < b ? -1 : 1;
    };
    /**
     * Checks if two variables are equal.
     *
     * @param a
     * @param b
     * @return
     */
    Comparator.prototype.equal = function (a, b) {
        return this.compare(a, b) === 0;
    };
    /**
     * Checks if variable "a" is less than "b".
     *
     * @param a
     * @param  b
     * @return
     */
    Comparator.prototype.lessThan = function (a, b) {
        return this.compare(a, b) < 0;
    };
    /**
     * Checks if variable "a" is greater than "b".
     *
     * @param a
     * @param  b
     * @return
     */
    Comparator.prototype.greaterThan = function (a, b) {
        return this.compare(a, b) > 0;
    };
    /**
     * Checks if variable "a" is less than or equal to "b".
     *
     * @param a
     * @param  b
     * @return
     */
    Comparator.prototype.lessThanOrEqual = function (a, b) {
        return this.lessThan(a, b) || this.equal(a, b);
    };
    /**
     * Checks if variable "a" is greater than or equal to "b".
     *
     * @param a
     * @param b
     * @return
     */
    Comparator.prototype.greaterThanOrEqual = function (a, b) {
        return this.greaterThan(a, b) || this.equal(a, b);
    };
    /**
     * Reverses the comparison order.
     */
    Comparator.prototype.reverse = function () {
        var compareOriginal = this.compare;
        this.compare = function (a, b) { return compareOriginal(b, a); };
    };
    return Comparator;
}());

/**
 * 链表
 */
var LinkedList = /** @class */ (function () {
    /**
     * 构建链表
     *
     * @param comparatorFunction 比较函数
     */
    function LinkedList(comparatorFunction) {
        this.head = null;
        this.tail = null;
        this.compare = new Comparator(comparatorFunction);
    }
    /**
     * 添加新结点到表头
     *
     * @param value 新结点值
     * @return 新增的结点
     */
    LinkedList.prototype.prepend = function (value) {
        // Make new node to be a head.
        var newNode = new LinkedListNode(value, this.head);
        this.head = newNode;
        // If there is no tail yet let's make new node a tail.
        if (!this.tail) {
            this.tail = newNode;
        }
        return this;
    };
    /**
     * 添加新结点到表尾
     *
     * @param value 新结点值
     * @return 新增的结点
     */
    LinkedList.prototype.append = function (value) {
        var newNode = new LinkedListNode(value);
        // If there is no head yet let's make new node a head.
        if (!this.head) {
            this.head = newNode;
            this.tail = newNode;
            return this;
        }
        // Attach new node to the end of linked list.
        this.tail.next = newNode;
        this.tail = newNode;
        return this;
    };
    /**
     * 插入新结点
     *
     * @param value 新结点值
     * @param rawIndex 插入索引
     * @return 自身
     */
    LinkedList.prototype.insert = function (value, rawIndex) {
        var index = rawIndex < 0 ? 0 : rawIndex;
        if (index === 0) {
            this.prepend(value);
        }
        else {
            var count = 1;
            var currentNode = this.head;
            var newNode = new LinkedListNode(value);
            while (currentNode) {
                if (count === index)
                    { break; }
                currentNode = currentNode.next;
                count += 1;
            }
            if (currentNode) {
                newNode.next = currentNode.next;
                currentNode.next = newNode;
            }
            else if (this.tail) {
                this.tail.next = newNode;
                this.tail = newNode;
            }
            else {
                this.head = newNode;
                this.tail = newNode;
            }
        }
        return this;
    };
    /**
     * 删除链表中第一个与指定值相等的结点
     *
     * @param value 结点值
     * @return 被删除的结点
     */
    LinkedList.prototype.delete = function (value) {
        if (!this.head) {
            return null;
        }
        var deletedNode = null;
        // If the head must be deleted then make next node that is different
        // from the head to be a new head.
        while (this.head && this.compare.equal(this.head.value, value)) {
            deletedNode = this.head;
            this.head = this.head.next;
        }
        var currentNode = this.head;
        if (currentNode !== null) {
            // If next node must be deleted then make next node to be a next next one.
            while (currentNode.next) {
                if (this.compare.equal(currentNode.next.value, value)) {
                    deletedNode = currentNode.next;
                    currentNode.next = currentNode.next.next;
                }
                else {
                    currentNode = currentNode.next;
                }
            }
        }
        // Check if tail must be deleted.
        if (this.compare.equal(this.tail.value, value)) {
            this.tail = currentNode;
        }
        return deletedNode;
    };
    /**
     * 查找结点
     *
     * @param {Object} findParams
     * @param {*} findParams.value
     * @param {function} [findParams.callback]
     *
     * @return 查找到的结点
     */
    LinkedList.prototype.find = function (_a) {
        var _b = _a.value, value = _b === void 0 ? undefined : _b, _c = _a.callback, callback = _c === void 0 ? undefined : _c;
        if (!this.head) {
            return null;
        }
        var currentNode = this.head;
        while (currentNode) {
            // If callback is specified then try to find node by callback.
            if (callback && callback(currentNode.value)) {
                return currentNode;
            }
            // If value is specified then try to compare by value..
            if (value !== undefined && this.compare.equal(currentNode.value, value)) {
                return currentNode;
            }
            currentNode = currentNode.next;
        }
        return null;
    };
    /**
     * 删除表尾
     *
     * @return 被删除的结点
     */
    LinkedList.prototype.deleteTail = function () {
        var deletedTail = this.tail;
        if (this.head === this.tail) {
            // There is only one node in linked list.
            this.head = null;
            this.tail = null;
            return deletedTail;
        }
        // If there are many nodes in linked list...
        // Rewind to the last node and delete "next" link for the node before the last one.
        var currentNode = this.head;
        while (currentNode.next) {
            if (!currentNode.next.next) {
                currentNode.next = null;
            }
            else {
                currentNode = currentNode.next;
            }
        }
        this.tail = currentNode;
        return deletedTail;
    };
    /**
     * 删除表头
     *
     * @return 被删除结点
     */
    LinkedList.prototype.deleteHead = function () {
        if (!this.head) {
            return null;
        }
        var deletedHead = this.head;
        if (this.head.next) {
            this.head = this.head.next;
        }
        else {
            this.head = null;
            this.tail = null;
        }
        return deletedHead;
    };
    /**
     * 从数组中初始化链表
     *
     * @param values - Array of values that need to be converted to linked list.
     * @return 自身
     */
    LinkedList.prototype.fromArray = function (values) {
        var _this = this;
        values.forEach(function (value) { return _this.append(value); });
        return this;
    };
    /**
     * 转换为数组
     */
    LinkedList.prototype.toArray = function () {
        var nodes = [];
        var currentNode = this.head;
        while (currentNode) {
            nodes.push(currentNode);
            currentNode = currentNode.next;
        }
        return nodes;
    };
    /**
     * 转换为字符串
     *
     * @param callback
     */
    LinkedList.prototype.toString = function (callback) {
        return this.toArray().map(function (node) { return node.toString(callback); }).toString();
    };
    /**
     * Reverse a linked list.
     *
     * @returns 自身
     */
    LinkedList.prototype.reverse = function () {
        var currNode = this.head;
        var prevNode = null;
        var nextNode = null;
        while (currNode) {
            // Store next node.
            nextNode = currNode.next;
            // Change next node of the current node so it would link to previous node.
            currNode.next = prevNode;
            // Move prevNode and currNode nodes one step forward.
            prevNode = currNode;
            currNode = nextNode;
        }
        // Reset head and tail.
        this.tail = this.head;
        this.head = prevNode;
        return this;
    };
    return LinkedList;
}());

/**
 * 队列，只能从后面进，前面出
 * 使用单向链表实现
 */
var Queue = /** @class */ (function () {
    function Queue() {
        // We're going to implement Queue based on LinkedList since the two
        // structures are quite similar. Namely, they both operate mostly on
        // the elements at the beginning and the end. Compare enqueue/dequeue
        // operations of Queue with append/deleteHead operations of LinkedList.
        this.linkedList = new LinkedList();
    }
    /**
     * 是否为空
     */
    Queue.prototype.isEmpty = function () {
        return !this.linkedList.head;
    };
    /**
     * Read the element at the front of the queue without removing it.
     *
     * 读取队列前面的元素，但不删除它。
     */
    Queue.prototype.peek = function () {
        if (this.isEmpty()) {
            return null;
        }
        return this.linkedList.head.value;
    };
    /**
     * 入队
     *
     * 在队列的末尾(链表的尾部)添加一个新元素。
     * 这个元素将在它前面的所有元素之后被处理。
     *
     * Add a new element to the end of the queue (the tail of the linked list).
     * This element will be processed after all elements ahead of it.
     *
     * @param value 元素值
     */
    Queue.prototype.enqueue = function (value) {
        this.linkedList.append(value);
    };
    /**
     * 出队
     *
     * 删除队列前面的元素(链表的头)。如果队列为空，则返回null。
     *
     * Remove the element at the front of the queue (the head of the linked list).
     * If the queue is empty, return null.
     *
     * @return 出队的元素
     */
    Queue.prototype.dequeue = function () {
        var removedHead = this.linkedList.deleteHead();
        return removedHead ? removedHead.value : null;
    };
    /**
     * 转换为字符串
     *
     * @param callback 值输出为字符串函数
     */
    Queue.prototype.toString = function (callback) {
        // Return string representation of the queue's linked list.
        return this.linkedList.toString(callback);
    };
    return Queue;
}());

/**
 * @param callbacks
 */
function initCallbacks$2(callbacks) {
    if (callbacks === void 0) { callbacks = {}; }
    var initiatedCallback = callbacks;
    var stubCallback = function () { };
    var allowTraversalCallback = (function () {
        var seen = {};
        return function (_a) {
            var nextVertex = _a.nextVertex;
            if (!seen[nextVertex.getKey()]) {
                seen[nextVertex.getKey()] = true;
                return true;
            }
            return false;
        };
    })();
    initiatedCallback.allowTraversal = callbacks.allowTraversal || allowTraversalCallback;
    initiatedCallback.enterVertex = callbacks.enterVertex || stubCallback;
    initiatedCallback.leaveVertex = callbacks.leaveVertex || stubCallback;
    return initiatedCallback;
}
/**
 * @param graph
 * @param startVertex
 * @param originalCallbacks
 */
function breadthFirstSearch(graph, startVertex, originalCallbacks) {
    var callbacks = initCallbacks$2(originalCallbacks);
    var vertexQueue = new Queue();
    // Do initial queue setup.
    vertexQueue.enqueue(startVertex);
    var previousVertex = null;
    var _loop_1 = function () {
        var currentVertex = vertexQueue.dequeue();
        callbacks.enterVertex({ currentVertex: currentVertex, previousVertex: previousVertex });
        // Add all neighbors to the queue for future traversals.
        // eslint-disable-next-line no-loop-func
        graph.getNeighbors(currentVertex).forEach(function (nextVertex) {
            if (callbacks.allowTraversal({ previousVertex: previousVertex, currentVertex: currentVertex, nextVertex: nextVertex })) {
                vertexQueue.enqueue(nextVertex);
            }
        });
        callbacks.leaveVertex({ currentVertex: currentVertex, previousVertex: previousVertex });
        // Memorize current vertex before next loop.
        previousVertex = currentVertex;
    };
    // Traverse all vertices from the queue.
    while (!vertexQueue.isEmpty()) {
        _loop_1();
    }
}

/**
 * Helper class for visited vertex metadata.
 */
var VisitMetadata = /** @class */ (function () {
    function VisitMetadata(_a) {
        var discoveryTime = _a.discoveryTime, lowDiscoveryTime = _a.lowDiscoveryTime;
        this.discoveryTime = discoveryTime;
        this.lowDiscoveryTime = lowDiscoveryTime;
    }
    return VisitMetadata;
}());
/**
 * @param graph
 */
function graphBridges(graph) {
    // Set of vertices we've already visited during DFS.
    var visitedSet = {};
    // Set of bridges.
    var bridges = {};
    // Time needed to discover to the current vertex.
    var discoveryTime = 0;
    // Peek the start vertex for DFS traversal.
    var startVertex = graph.getAllVertices()[0];
    var dfsCallbacks = {
        enterVertex: function (_a) {
            var currentVertex = _a.currentVertex;
            // Tick discovery time.
            discoveryTime += 1;
            // Put current vertex to visited set.
            visitedSet[currentVertex.getKey()] = new VisitMetadata({
                discoveryTime: discoveryTime,
                lowDiscoveryTime: discoveryTime,
            });
        },
        leaveVertex: function (_a) {
            var currentVertex = _a.currentVertex, previousVertex = _a.previousVertex;
            if (previousVertex === null) {
                // Don't do anything for the root vertex if it is already current (not previous one).
                return;
            }
            // Check if current node is connected to any early node other then previous one.
            visitedSet[currentVertex.getKey()].lowDiscoveryTime = currentVertex.getNeighbors()
                .filter(function (earlyNeighbor) { return earlyNeighbor.getKey() !== previousVertex.getKey(); })
                .reduce(
            /**
             * @param {number} lowestDiscoveryTime
             * @param {GraphVertex} neighbor
             */
            function (lowestDiscoveryTime, neighbor) {
                var neighborLowTime = visitedSet[neighbor.getKey()].lowDiscoveryTime;
                return neighborLowTime < lowestDiscoveryTime ? neighborLowTime : lowestDiscoveryTime;
            }, visitedSet[currentVertex.getKey()].lowDiscoveryTime);
            // Compare low discovery times. In case if current low discovery time is less than the one
            // in previous vertex then update previous vertex low time.
            var currentLowDiscoveryTime = visitedSet[currentVertex.getKey()].lowDiscoveryTime;
            var previousLowDiscoveryTime = visitedSet[previousVertex.getKey()].lowDiscoveryTime;
            if (currentLowDiscoveryTime < previousLowDiscoveryTime) {
                visitedSet[previousVertex.getKey()].lowDiscoveryTime = currentLowDiscoveryTime;
            }
            // Compare current vertex low discovery time with parent discovery time. Check if there
            // are any short path (back edge) exists. If we can't get to current vertex other then
            // via parent then the parent vertex is articulation point for current one.
            var parentDiscoveryTime = visitedSet[previousVertex.getKey()].discoveryTime;
            if (parentDiscoveryTime < currentLowDiscoveryTime) {
                var bridge = graph.findEdge(previousVertex, currentVertex);
                bridges[bridge.getKey()] = bridge;
            }
        },
        allowTraversal: function (_a) {
            var nextVertex = _a.nextVertex;
            return !visitedSet[nextVertex.getKey()];
        },
    };
    // Do Depth First Search traversal over submitted graph.
    depthFirstSearch(graph, startVertex, dfsCallbacks);
    return bridges;
}

/**
 * Detect cycle in directed graph using Depth First Search.
 *
 * @param graph
 */
function detectDirectedCycle(graph) {
    var cycle = null;
    // Will store parents (previous vertices) for all visited nodes.
    // This will be needed in order to specify what path exactly is a cycle.
    var dfsParentMap = {};
    // White set (UNVISITED) contains all the vertices that haven't been visited at all.
    var whiteSet = {};
    // Gray set (VISITING) contains all the vertices that are being visited right now
    // (in current path).
    var graySet = {};
    // Black set (VISITED) contains all the vertices that has been fully visited.
    // Meaning that all children of the vertex has been visited.
    var blackSet = {};
    // If we encounter vertex in gray set it means that we've found a cycle.
    // Because when vertex in gray set it means that its neighbors or its neighbors
    // neighbors are still being explored.
    // Init white set and add all vertices to it.
    graph.getAllVertices().forEach(function (vertex) {
        whiteSet[vertex.getKey()] = vertex;
    });
    // Describe BFS callbacks.
    var callbacks = {
        enterVertex: function (_a) {
            var currentVertex = _a.currentVertex, previousVertex = _a.previousVertex;
            if (graySet[currentVertex.getKey()]) {
                // If current vertex already in grey set it means that cycle is detected.
                // Let's detect cycle path.
                cycle = {};
                var currentCycleVertex = currentVertex;
                var previousCycleVertex = previousVertex;
                while (previousCycleVertex.getKey() !== currentVertex.getKey()) {
                    cycle[currentCycleVertex.getKey()] = previousCycleVertex;
                    currentCycleVertex = previousCycleVertex;
                    previousCycleVertex = dfsParentMap[previousCycleVertex.getKey()];
                }
                cycle[currentCycleVertex.getKey()] = previousCycleVertex;
            }
            else {
                // Otherwise let's add current vertex to gray set and remove it from white set.
                graySet[currentVertex.getKey()] = currentVertex;
                delete whiteSet[currentVertex.getKey()];
                // Update DFS parents list.
                dfsParentMap[currentVertex.getKey()] = previousVertex;
            }
        },
        leaveVertex: function (_a) {
            var currentVertex = _a.currentVertex;
            // If all node's children has been visited let's remove it from gray set
            // and move it to the black set meaning that all its neighbors are visited.
            blackSet[currentVertex.getKey()] = currentVertex;
            delete graySet[currentVertex.getKey()];
        },
        allowTraversal: function (_a) {
            var nextVertex = _a.nextVertex;
            // If cycle was detected we must forbid all further traversing since it will
            // cause infinite traversal loop.
            if (cycle) {
                return false;
            }
            // Allow traversal only for the vertices that are not in black set
            // since all black set vertices have been already visited.
            return !blackSet[nextVertex.getKey()];
        },
    };
    // Start exploring vertices.
    while (Object.keys(whiteSet).length) {
        // Pick fist vertex to start BFS from.
        var firstWhiteKey = Object.keys(whiteSet)[0];
        var startVertex = whiteSet[firstWhiteKey];
        // Do Depth First Search.
        depthFirstSearch(graph, startVertex, callbacks);
    }
    return cycle;
}

/**
 * Detect cycle in undirected graph using Depth First Search.
 *
 * @param graph
 */
function detectUndirectedCycle(graph) {
    var cycle = null;
    // List of vertices that we have visited.
    var visitedVertices = {};
    // List of parents vertices for every visited vertex.
    var parents = {};
    // Callbacks for DFS traversing.
    var callbacks = {
        allowTraversal: function (_a) {
            var currentVertex = _a.currentVertex, nextVertex = _a.nextVertex;
            // Don't allow further traversal in case if cycle has been detected.
            if (cycle) {
                return false;
            }
            // Don't allow traversal from child back to its parent.
            var currentVertexParent = parents[currentVertex.getKey()];
            var currentVertexParentKey = currentVertexParent ? currentVertexParent.getKey() : null;
            return currentVertexParentKey !== nextVertex.getKey();
        },
        enterVertex: function (_a) {
            var currentVertex = _a.currentVertex, previousVertex = _a.previousVertex;
            if (visitedVertices[currentVertex.getKey()]) {
                // Compile cycle path based on parents of previous vertices.
                cycle = {};
                var currentCycleVertex = currentVertex;
                var previousCycleVertex = previousVertex;
                while (previousCycleVertex.getKey() !== currentVertex.getKey()) {
                    cycle[currentCycleVertex.getKey()] = previousCycleVertex;
                    currentCycleVertex = previousCycleVertex;
                    previousCycleVertex = parents[previousCycleVertex.getKey()];
                }
                cycle[currentCycleVertex.getKey()] = previousCycleVertex;
            }
            else {
                // Add next vertex to visited set.
                visitedVertices[currentVertex.getKey()] = currentVertex;
                parents[currentVertex.getKey()] = previousVertex;
            }
        },
    };
    // Start DFS traversing.
    var startVertex = graph.getAllVertices()[0];
    depthFirstSearch(graph, startVertex, callbacks);
    return cycle;
}

/**
 * 并查集结点
 */
var DisjointSetItem = /** @class */ (function () {
    /**
     * 构建 并查集 项
     *
     * @param value 值
     * @param keyCallback 计算键值函数
     */
    function DisjointSetItem(value, keyCallback) {
        this.value = value;
        this.keyCallback = keyCallback;
        this.parent = null;
        this.children = {};
    }
    /**
     * 获取键值
     */
    DisjointSetItem.prototype.getKey = function () {
        // Allow user to define custom key generator.
        if (this.keyCallback) {
            return this.keyCallback(this.value);
        }
        // Otherwise use value as a key by default.
        return this.value;
    };
    /**
     * 获取根结点
     */
    DisjointSetItem.prototype.getRoot = function () {
        return this.isRoot() ? this : this.parent.getRoot();
    };
    /**
     * 是否为根结点
     */
    DisjointSetItem.prototype.isRoot = function () {
        return this.parent === null;
    };
    /**
     * Rank basically means the number of all ancestors.
     *
     * @return 所有子孙结点数量
     */
    DisjointSetItem.prototype.getRank = function () {
        if (this.getChildren().length === 0) {
            return 0;
        }
        var rank = 0;
        this.getChildren().forEach(function (child) {
            // Count child itself.
            rank += 1;
            // Also add all children of current child.
            rank += child.getRank();
        });
        return rank;
    };
    /**
     * 获取子结点列表
     *
     * @return 子结点列表
     */
    DisjointSetItem.prototype.getChildren = function () {
        return Object.values(this.children);
    };
    /**
     * 设置父结点
     *
     * @param parentItem 父结点
     * @param forceSettingParentChild 强制这是父子关系
     * @return 自身
     */
    DisjointSetItem.prototype.setParent = function (parentItem, forceSettingParentChild) {
        if (forceSettingParentChild === void 0) { forceSettingParentChild = true; }
        this.parent = parentItem;
        if (forceSettingParentChild) {
            parentItem.addChild(this);
        }
        return this;
    };
    /**
     * 添加子结点
     *
     * @param childItem 子结点
     * @return 自身
     */
    DisjointSetItem.prototype.addChild = function (childItem) {
        this.children[childItem.getKey()] = childItem;
        childItem.setParent(this, false);
        return this;
    };
    return DisjointSetItem;
}());

/**
 * 并查集
 *
 * 并查集是一种树型的数据结构，用于处理一些不交集（Disjoint Sets）的合并及查询问题。
 *
 * @see https://github.com/trekhleb/javascript-algorithms/blob/master/src/data-structures/disjoint-set/DisjointSet.js
 * @see https://en.wikipedia.org/wiki/Disjoint-set_data_structure
 * @see https://www.youtube.com/watch?v=wU6udHRIkcc&index=14&t=0s&list=PLLXdhg_r2hKA7DPDsunoDZ-Z769jWn4R8
 */
var DisjointSet = /** @class */ (function () {
    /**
     * 构建 并查集
     *
     * @param keyCallback 计算键值函数
     */
    function DisjointSet(keyCallback) {
        this.keyCallback = keyCallback;
        this.items = {};
    }
    /**
     * 创建集合
     *
     * @param itemValue 结点值
     * @return 创建的集合
     */
    DisjointSet.prototype.makeSet = function (itemValue) {
        var disjointSetItem = new DisjointSetItem(itemValue, this.keyCallback);
        if (!this.items[disjointSetItem.getKey()]) {
            // Add new item only in case if it not presented yet.
            this.items[disjointSetItem.getKey()] = disjointSetItem;
        }
        return this;
    };
    /**
     * Find set representation node.
     *
     * @param itemValue 结点值
     */
    DisjointSet.prototype.find = function (itemValue) {
        var templateDisjointItem = new DisjointSetItem(itemValue, this.keyCallback);
        // Try to find item itself;
        var requiredDisjointItem = this.items[templateDisjointItem.getKey()];
        if (!requiredDisjointItem) {
            return null;
        }
        return requiredDisjointItem.getRoot().getKey();
    };
    /**
     * Union by rank.
     *
     * @param valueA 值A
     * @param valueB 值B
     * @return 自身
     */
    DisjointSet.prototype.union = function (valueA, valueB) {
        var rootKeyA = this.find(valueA);
        var rootKeyB = this.find(valueB);
        if (rootKeyA === null || rootKeyB === null) {
            throw new Error('One or two values are not in sets');
        }
        if (rootKeyA === rootKeyB) {
            // In case if both elements are already in the same set then just return its key.
            return this;
        }
        var rootA = this.items[rootKeyA];
        var rootB = this.items[rootKeyB];
        if (rootA.getRank() < rootB.getRank()) {
            // If rootB's tree is bigger then make rootB to be a new root.
            rootB.addChild(rootA);
            return this;
        }
        // If rootA's tree is bigger then make rootA to be a new root.
        rootA.addChild(rootB);
        return this;
    };
    /**
     * 判断两个值是否在相同集合中
     *
     * @param valueA 值A
     * @param valueB 值B
     */
    DisjointSet.prototype.inSameSet = function (valueA, valueB) {
        var rootKeyA = this.find(valueA);
        var rootKeyB = this.find(valueB);
        if (rootKeyA === null || rootKeyB === null) {
            throw new Error('One or two values are not in sets');
        }
        return rootKeyA === rootKeyB;
    };
    return DisjointSet;
}());

/**
 * Detect cycle in undirected graph using disjoint sets.
 *
 * @param graph
 */
function detectUndirectedCycleUsingDisjointSet(graph) {
    // Create initial singleton disjoint sets for each graph vertex.
    var keyExtractor = function (graphVertex) { return graphVertex.getKey(); };
    var disjointSet = new DisjointSet(keyExtractor);
    graph.getAllVertices().forEach(function (graphVertex) { return disjointSet.makeSet(graphVertex); });
    // Go trough all graph edges one by one and check if edge vertices are from the
    // different sets. In this case joint those sets together. Do this until you find
    // an edge where to edge vertices are already in one set. This means that current
    // edge will create a cycle.
    var cycleFound = false;
    graph.getAllEdges().forEach(function (graphEdge) {
        if (disjointSet.inSameSet(graphEdge.startVertex, graphEdge.endVertex)) {
            // Cycle found.
            cycleFound = true;
        }
        else {
            disjointSet.union(graphEdge.startVertex, graphEdge.endVertex);
        }
    });
    return cycleFound;
}

/**
 * 堆
 *
 * Parent class for Min and Max Heaps.
 */
var Heap = /** @class */ (function () {
    /**
     * 构建链表
     *
     * @param comparatorFunction 比较函数
     */
    function Heap(comparatorFunction) {
        var _newTarget = this.constructor;
        if (_newTarget === Heap) {
            throw new TypeError('Cannot construct Heap instance directly');
        }
        // Array representation of the heap.
        this.heapContainer = [];
        this.compare = new Comparator(comparatorFunction);
    }
    /**
     * 获取左边子结点索引
     *
     * @param parentIndex 父结点索引
     * @return 左边子结点索引
     */
    Heap.prototype.getLeftChildIndex = function (parentIndex) {
        return (2 * parentIndex) + 1;
    };
    /**
     * 获取右边子结点索引
     *
     * @param parentIndex 父结点索引
     * @return 右边子结点索引
     */
    Heap.prototype.getRightChildIndex = function (parentIndex) {
        return (2 * parentIndex) + 2;
    };
    /**
     * 获取父结点索引
     *
     * @param childIndex 子结点索引
     * @return 父结点索引
     */
    Heap.prototype.getParentIndex = function (childIndex) {
        return Math.floor((childIndex - 1) / 2);
    };
    /**
     * 是否有父结点
     *
     * @param childIndex 子结点索引
     * @return 是否有父结点
     */
    Heap.prototype.hasParent = function (childIndex) {
        return this.getParentIndex(childIndex) >= 0;
    };
    /**
     * 是否有左子结点
     *
     * @param parentIndex 父结点索引
     * @return 是否有左子结点
     */
    Heap.prototype.hasLeftChild = function (parentIndex) {
        return this.getLeftChildIndex(parentIndex) < this.heapContainer.length;
    };
    /**
     * 是否有右子结点
     *
     * @param parentIndex 父结点索引
     * @return 是否有右子结点
     */
    Heap.prototype.hasRightChild = function (parentIndex) {
        return this.getRightChildIndex(parentIndex) < this.heapContainer.length;
    };
    /**
     * 获取左结点
     *
     * @param parentIndex 父结点索引
     * @return 左结点
     */
    Heap.prototype.leftChild = function (parentIndex) {
        return this.heapContainer[this.getLeftChildIndex(parentIndex)];
    };
    /**
     * 获取右结点
     *
     * @param parentIndex 父结点索引
     * @return 右结点
     */
    Heap.prototype.rightChild = function (parentIndex) {
        return this.heapContainer[this.getRightChildIndex(parentIndex)];
    };
    /**
     * 获取父结点
     *
     * @param childIndex 子结点索引
     * @return 父结点
     */
    Heap.prototype.parent = function (childIndex) {
        return this.heapContainer[this.getParentIndex(childIndex)];
    };
    /**
     * 交换两个结点数据
     *
     * @param indexOne 索引1
     * @param indexTwo 索引2
     */
    Heap.prototype.swap = function (indexOne, indexTwo) {
        var tmp = this.heapContainer[indexTwo];
        this.heapContainer[indexTwo] = this.heapContainer[indexOne];
        this.heapContainer[indexOne] = tmp;
    };
    /**
     * 查看堆顶数据
     *
     * @return 堆顶数据
     */
    Heap.prototype.peek = function () {
        if (this.heapContainer.length === 0) {
            return null;
        }
        return this.heapContainer[0];
    };
    /**
     * 取出堆顶元素
     *
     * @return 堆顶元素
     */
    Heap.prototype.poll = function () {
        if (this.heapContainer.length === 0) {
            return null;
        }
        if (this.heapContainer.length === 1) {
            return this.heapContainer.pop();
        }
        var item = this.heapContainer[0];
        // Move the last element from the end to the head.
        this.heapContainer[0] = this.heapContainer.pop();
        this.heapifyDown();
        return item;
    };
    /**
     * 新增元素
     *
     * @param item 新元素
     * @return 自身
     */
    Heap.prototype.add = function (item) {
        this.heapContainer.push(item);
        this.heapifyUp();
        return this;
    };
    /**
     * 移除所有指定元素
     *
     * @param item 被移除的元素
     * @param comparator 比较器
     * @return 自身
     */
    Heap.prototype.remove = function (item, comparator) {
        if (comparator === void 0) { comparator = this.compare; }
        // Find number of items to remove.
        var numberOfItemsToRemove = this.find(item, comparator).length;
        for (var iteration = 0; iteration < numberOfItemsToRemove; iteration += 1) {
            // We need to find item index to remove each time after removal since
            // indices are being changed after each heapify process.
            var indexToRemove = this.find(item, comparator).pop();
            // If we need to remove last child in the heap then just remove it.
            // There is no need to heapify the heap afterwards.
            if (indexToRemove === (this.heapContainer.length - 1)) {
                this.heapContainer.pop();
            }
            else {
                // Move last element in heap to the vacant (removed) position.
                this.heapContainer[indexToRemove] = this.heapContainer.pop();
                // Get parent.
                var parentItem = this.parent(indexToRemove);
                // If there is no parent or parent is in correct order with the node
                // we're going to delete then heapify down. Otherwise heapify up.
                if (this.hasLeftChild(indexToRemove)
                    && (!parentItem
                        || this.pairIsInCorrectOrder(parentItem, this.heapContainer[indexToRemove]))) {
                    this.heapifyDown(indexToRemove);
                }
                else {
                    this.heapifyUp(indexToRemove);
                }
            }
        }
        return this;
    };
    /**
     * 查找元素所在的所有索引
     *
     * @param item 查找的元素
     * @param comparator 比较器
     * @return 查找的元素所在索引了吧
     */
    Heap.prototype.find = function (item, comparator) {
        if (comparator === void 0) { comparator = this.compare; }
        var foundItemIndices = [];
        for (var itemIndex = 0; itemIndex < this.heapContainer.length; itemIndex += 1) {
            if (comparator.equal(item, this.heapContainer[itemIndex])) {
                foundItemIndices.push(itemIndex);
            }
        }
        return foundItemIndices;
    };
    /**
     * 是否为空
     */
    Heap.prototype.isEmpty = function () {
        return !this.heapContainer.length;
    };
    /**
     * 转换为字符串
     */
    Heap.prototype.toString = function () {
        return this.heapContainer.toString();
    };
    /**
     * 堆冒泡
     *
     * @param customStartIndex 堆冒泡起始索引
     */
    Heap.prototype.heapifyUp = function (customStartIndex) {
        // Take the last element (last in array or the bottom left in a tree)
        // in the heap container and lift it up until it is in the correct
        // order with respect to its parent element.
        var currentIndex = customStartIndex || this.heapContainer.length - 1;
        while (this.hasParent(currentIndex)
            && !this.pairIsInCorrectOrder(this.parent(currentIndex), this.heapContainer[currentIndex])) {
            this.swap(currentIndex, this.getParentIndex(currentIndex));
            currentIndex = this.getParentIndex(currentIndex);
        }
    };
    /**
     * 堆下沉
     *
     * @param customStartIndex 堆下沉起始索引
     */
    Heap.prototype.heapifyDown = function (customStartIndex) {
        if (customStartIndex === void 0) { customStartIndex = 0; }
        // Compare the parent element to its children and swap parent with the appropriate
        // child (smallest child for MinHeap, largest child for MaxHeap).
        // Do the same for next children after swap.
        var currentIndex = customStartIndex;
        var nextIndex = null;
        while (this.hasLeftChild(currentIndex)) {
            if (this.hasRightChild(currentIndex)
                && this.pairIsInCorrectOrder(this.rightChild(currentIndex), this.leftChild(currentIndex))) {
                nextIndex = this.getRightChildIndex(currentIndex);
            }
            else {
                nextIndex = this.getLeftChildIndex(currentIndex);
            }
            if (this.pairIsInCorrectOrder(this.heapContainer[currentIndex], this.heapContainer[nextIndex])) {
                break;
            }
            this.swap(currentIndex, nextIndex);
            currentIndex = nextIndex;
        }
    };
    /**
     * Checks if pair of heap elements is in correct order.
     * For MinHeap the first element must be always smaller or equal.
     * For MaxHeap the first element must be always bigger or equal.
     *
     * @param firstElement 第一个元素
     * @param secondElement 第二个元素
     * @return {boolean}
     */
    Heap.prototype.pairIsInCorrectOrder = function (firstElement, secondElement) {
        throw new Error("\n      You have to implement heap pair comparision method\n      for " + firstElement + " and " + secondElement + " values.\n    ");
    };
    return Heap;
}());

/**
 * 最小堆
 *
 * 所有父结点都小于子结点
 */
var MinHeap = /** @class */ (function (_super) {
    __extends(MinHeap, _super);
    function MinHeap() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    /**
     * Checks if pair of heap elements is in correct order.
     * For MinHeap the first element must be always smaller or equal.
     * For MaxHeap the first element must be always bigger or equal.
     *
     * 检查堆元素对的顺序是否正确。
     * 对于MinHeap，第一个元素必须总是小于等于。
     * 对于MaxHeap，第一个元素必须总是大于或等于。
     *
     * @param firstElement 第一个元素
     * @param secondElement 第二个元素
     * @return 堆元素对的顺序是否正确
     */
    MinHeap.prototype.pairIsInCorrectOrder = function (firstElement, secondElement) {
        return this.compare.lessThanOrEqual(firstElement, secondElement);
    };
    return MinHeap;
}(Heap));

/**
 * 优先队列
 *
 * It is the same as min heap except that when comparing two elements
 * we take into account its priority instead of the element's value.
 */
var PriorityQueue = /** @class */ (function (_super) {
    __extends(PriorityQueue, _super);
    function PriorityQueue() {
        var _this = 
        // Call MinHip constructor first.
        _super.call(this) || this;
        // Setup priorities map.
        _this.priorities = new Map();
        // Use custom comparator for heap elements that will take element priority
        // instead of element value into account.
        _this.compare = new Comparator(_this.comparePriority.bind(_this));
        return _this;
    }
    /**
     * Add item to the priority queue.
     *
     * @param item item we're going to add to the queue.
     * @param priority items priority.
     */
    PriorityQueue.prototype.add = function (item, priority) {
        if (priority === void 0) { priority = 0; }
        this.priorities.set(item, priority);
        _super.prototype.add.call(this, item);
        return this;
    };
    /**
     * Remove item from priority queue.
     *
     * @param item item we're going to remove.
     * @param customFindingComparator custom function for finding the item to remove
     */
    PriorityQueue.prototype.remove = function (item, customFindingComparator) {
        _super.prototype.remove.call(this, item, customFindingComparator);
        this.priorities.delete(item);
        return this;
    };
    /**
     * Change priority of the item in a queue.
     *
     * @param item item we're going to re-prioritize.
     * @param priority new item's priority.
     */
    PriorityQueue.prototype.changePriority = function (item, priority) {
        this.remove(item, new Comparator(this.compareValue));
        this.add(item, priority);
        return this;
    };
    /**
     * Find item by ite value.
     *
     * @param item
     */
    PriorityQueue.prototype.findByValue = function (item) {
        return this.find(item, new Comparator(this.compareValue));
    };
    /**
     * Check if item already exists in a queue.
     *
     * @param item 元素
     */
    PriorityQueue.prototype.hasValue = function (item) {
        return this.findByValue(item).length > 0;
    };
    /**
     * Compares priorities of two items.
     *
     * @param a 元素a
     * @param b 元素b
     */
    PriorityQueue.prototype.comparePriority = function (a, b) {
        if (this.priorities.get(a) === this.priorities.get(b)) {
            return 0;
        }
        return this.priorities.get(a) < this.priorities.get(b) ? -1 : 1;
    };
    /**
     * Compares values of two items.
     * @param a 元素a
     * @param b 元素b
     */
    PriorityQueue.prototype.compareValue = function (a, b) {
        if (a === b) {
            return 0;
        }
        return a < b ? -1 : 1;
    };
    return PriorityQueue;
}(MinHeap));

/**
 * Implementation of Dijkstra algorithm of finding the shortest paths to graph nodes.
 * @param graph graph we're going to traverse.
 * @param startVertex traversal start vertex.
 */
function dijkstra(graph, startVertex) {
    // Init helper variables that we will need for Dijkstra algorithm.
    var distances = {};
    var visitedVertices = {};
    var previousVertices = {};
    var queue = new PriorityQueue();
    // Init all distances with infinity assuming that currently we can't reach
    // any of the vertices except the start one.
    graph.getAllVertices().forEach(function (vertex) {
        distances[vertex.getKey()] = Infinity;
        previousVertices[vertex.getKey()] = null;
    });
    // We are already at the startVertex so the distance to it is zero.
    distances[startVertex.getKey()] = 0;
    // Init vertices queue.
    queue.add(startVertex, distances[startVertex.getKey()]);
    var _loop_1 = function () {
        // Fetch next closest vertex.
        var currentVertex = queue.poll();
        // Iterate over every unvisited neighbor of the current vertex.
        currentVertex.getNeighbors().forEach(function (neighbor) {
            // Don't visit already visited vertices.
            if (!visitedVertices[neighbor.getKey()]) {
                // Update distances to every neighbor from current vertex.
                var edge = graph.findEdge(currentVertex, neighbor);
                var existingDistanceToNeighbor = distances[neighbor.getKey()];
                var distanceToNeighborFromCurrent = distances[currentVertex.getKey()] + edge.weight;
                // If we've found shorter path to the neighbor - update it.
                if (distanceToNeighborFromCurrent < existingDistanceToNeighbor) {
                    distances[neighbor.getKey()] = distanceToNeighborFromCurrent;
                    // Change priority of the neighbor in a queue since it might have became closer.
                    if (queue.hasValue(neighbor)) {
                        queue.changePriority(neighbor, distances[neighbor.getKey()]);
                    }
                    // Remember previous closest vertex.
                    previousVertices[neighbor.getKey()] = currentVertex;
                }
                // Add neighbor to the queue for further visiting.
                if (!queue.hasValue(neighbor)) {
                    queue.add(neighbor, distances[neighbor.getKey()]);
                }
            }
        });
        // Add current vertex to visited ones to avoid visiting it again later.
        visitedVertices[currentVertex.getKey()] = currentVertex;
    };
    // Iterate over the priority queue of vertices until it is empty.
    while (!queue.isEmpty()) {
        _loop_1();
    }
    // Return the set of shortest distances to all vertices and the set of
    // shortest paths to all vertices in a graph.
    return {
        distances: distances,
        previousVertices: previousVertices,
    };
}

/**
 * Fleury's algorithm of finding Eulerian Path (visit all graph edges exactly once).
 *
 * @param graph
 */
function eulerianPath(graph) {
    var eulerianPathVertices = [];
    // Set that contains all vertices with even rank (number of neighbors).
    var evenRankVertices = {};
    // Set that contains all vertices with odd rank (number of neighbors).
    var oddRankVertices = {};
    // Set of all not visited edges.
    var notVisitedEdges = {};
    graph.getAllEdges().forEach(function (vertex) {
        notVisitedEdges[vertex.getKey()] = vertex;
    });
    // Detect whether graph contains Eulerian Circuit or Eulerian Path or none of them.
    /** @params {GraphVertex} vertex */
    graph.getAllVertices().forEach(function (vertex) {
        if (vertex.getDegree() % 2) {
            oddRankVertices[vertex.getKey()] = vertex;
        }
        else {
            evenRankVertices[vertex.getKey()] = vertex;
        }
    });
    // Check whether we're dealing with Eulerian Circuit or Eulerian Path only.
    // Graph would be an Eulerian Circuit in case if all its vertices has even degree.
    // If not all vertices have even degree then graph must contain only two odd-degree
    // vertices in order to have Euler Path.
    var isCircuit = !Object.values(oddRankVertices).length;
    if (!isCircuit && Object.values(oddRankVertices).length !== 2) {
        throw new Error('Eulerian path must contain two odd-ranked vertices');
    }
    // Pick start vertex for traversal.
    var startVertex = null;
    if (isCircuit) {
        // For Eulerian Circuit it doesn't matter from what vertex to start thus we'll just
        // peek a first node.
        var evenVertexKey = Object.keys(evenRankVertices)[0];
        startVertex = evenRankVertices[evenVertexKey];
    }
    else {
        // For Eulerian Path we need to start from one of two odd-degree vertices.
        var oddVertexKey = Object.keys(oddRankVertices)[0];
        startVertex = oddRankVertices[oddVertexKey];
    }
    // Start traversing the graph.
    var currentVertex = startVertex;
    var _loop_1 = function () {
        var _a, _b;
        // Add current vertex to Eulerian path.
        eulerianPathVertices.push(currentVertex);
        // Detect all bridges in graph.
        // We need to do it in order to not delete bridges if there are other edges
        // exists for deletion.
        var bridges = graphBridges(graph);
        // Peek the next edge to delete from graph.
        var currentEdges = currentVertex.getEdges();
        /** @var {GraphEdge} edgeToDelete */
        var edgeToDelete = null;
        if (currentEdges.length === 1) {
            // If there is only one edge left we need to peek it.
            _a = __read(currentEdges, 1), edgeToDelete = _a[0];
        }
        else {
            // If there are many edges left then we need to peek any of those except bridges.
            _b = __read(currentEdges.filter(function (edge) { return !bridges[edge.getKey()]; }), 1), edgeToDelete = _b[0];
        }
        // Detect next current vertex.
        if (currentVertex.getKey() === edgeToDelete.startVertex.getKey()) {
            currentVertex = edgeToDelete.endVertex;
        }
        else {
            currentVertex = edgeToDelete.startVertex;
        }
        // Delete edge from not visited edges set.
        delete notVisitedEdges[edgeToDelete.getKey()];
        // If last edge were deleted then add finish vertex to Eulerian Path.
        if (Object.values(notVisitedEdges).length === 0) {
            eulerianPathVertices.push(currentVertex);
        }
        // Delete the edge from graph.
        graph.deleteEdge(edgeToDelete);
    };
    while (Object.values(notVisitedEdges).length) {
        _loop_1();
    }
    return eulerianPathVertices;
}

/**
 * 图顶点
 */
var GraphVertex = /** @class */ (function () {
    /**
     * 构建图顶点
     *
     * @param value 值
     */
    function GraphVertex(value) {
        if (value === undefined) {
            throw new Error('Graph vertex must have a value');
        }
        /**
         * @param {GraphEdge} edgeA
         * @param {GraphEdge} edgeB
         */
        var edgeComparator = function (edgeA, edgeB) {
            if (edgeA.getKey() === edgeB.getKey()) {
                return 0;
            }
            return edgeA.getKey() < edgeB.getKey() ? -1 : 1;
        };
        // Normally you would store string value like vertex name.
        // But generally it may be any object as well
        this.value = value;
        this.edges = new LinkedList(edgeComparator);
    }
    /**
     * 新增边
     *
     * @param edge 新增的边
     * @returns 自身
     */
    GraphVertex.prototype.addEdge = function (edge) {
        this.edges.append(edge);
        return this;
    };
    /**
     * 删除边
     *
     * @param edge 边
     */
    GraphVertex.prototype.deleteEdge = function (edge) {
        this.edges.delete(edge);
    };
    /**
     * 获取相邻顶点列表
     */
    GraphVertex.prototype.getNeighbors = function () {
        var _this = this;
        var edges = this.edges.toArray();
        var neighborsConverter = function (node) {
            return (node.value.startVertex === _this ? node.value.endVertex : node.value.startVertex);
        };
        // Return either start or end vertex.
        // For undirected graphs it is possible that current vertex will be the end one.
        return edges.map(neighborsConverter);
    };
    /**
     * 获取边列表
     */
    GraphVertex.prototype.getEdges = function () {
        return this.edges.toArray().map(function (linkedListNode) { return linkedListNode.value; });
    };
    /**
     * 获取边的数量
     */
    GraphVertex.prototype.getDegree = function () {
        return this.edges.toArray().length;
    };
    /**
     * 是否存在指定边
     *
     * @param requiredEdge 指定边
     * @returns 是否存在指定边
     */
    GraphVertex.prototype.hasEdge = function (requiredEdge) {
        var edgeNode = this.edges.find({
            callback: function (edge) { return edge === requiredEdge; },
        });
        return !!edgeNode;
    };
    /**
     * 是否有相邻顶点
     *
     * @param vertex 顶点
     * @returns 是否有相邻顶点
     */
    GraphVertex.prototype.hasNeighbor = function (vertex) {
        var vertexNode = this.edges.find({
            callback: function (edge) { return edge.startVertex === vertex || edge.endVertex === vertex; },
        });
        return !!vertexNode;
    };
    /**
     * 查找边
     *
     * @param vertex 顶点
     * @returns 查找到的边
     */
    GraphVertex.prototype.findEdge = function (vertex) {
        var edgeFinder = function (edge) {
            return edge.startVertex === vertex || edge.endVertex === vertex;
        };
        var edge = this.edges.find({ callback: edgeFinder });
        return edge ? edge.value : null;
    };
    /**
     * 获取键值
     */
    GraphVertex.prototype.getKey = function () {
        return this.value;
    };
    /**
     * 删除所有边
     */
    GraphVertex.prototype.deleteAllEdges = function () {
        var _this = this;
        this.getEdges().forEach(function (edge) { return _this.deleteEdge(edge); });
        return this;
    };
    /**
     * 转换为字符串
     *
     * @param callback 转换为字符串函数
     */
    GraphVertex.prototype.toString = function (callback) {
        return callback ? callback(this.value) : "" + this.value;
    };
    return GraphVertex;
}());

/**
 * 图
 */
var Graph = /** @class */ (function () {
    /**
     * 构建图
     *
     * @param isDirected 是否有向
     */
    function Graph(isDirected) {
        if (isDirected === void 0) { isDirected = false; }
        /**
         * 是否有向
         */
        this.isDirected = false;
        this.vertices = {};
        this.edges = {};
        this.isDirected = isDirected;
    }
    /**
     * 新增顶点
     *
     * @param newVertex 新顶点
     */
    Graph.prototype.addVertex = function (newVertex) {
        this.vertices[newVertex.getKey()] = newVertex;
        return this;
    };
    /**
     * 获取顶点
     *
     * @param vertexKey 顶点键值
     */
    Graph.prototype.getVertexByKey = function (vertexKey) {
        return this.vertices[vertexKey];
    };
    /**
     * 获取相邻点
     *
     * @param vertex 顶点
     */
    Graph.prototype.getNeighbors = function (vertex) {
        return vertex.getNeighbors();
    };
    /**
     * 获取所有顶点
     */
    Graph.prototype.getAllVertices = function () {
        return Object.values(this.vertices);
    };
    /**
     * 获取所有边
     */
    Graph.prototype.getAllEdges = function () {
        return Object.values(this.edges);
    };
    /**
     * 新增边
     *
     * @param edge 边
     */
    Graph.prototype.addEdge = function (edge) {
        // Try to find and end start vertices.
        var startVertex = this.getVertexByKey(edge.startVertex.getKey());
        var endVertex = this.getVertexByKey(edge.endVertex.getKey());
        // Insert start vertex if it wasn't inserted.
        if (!startVertex) {
            this.addVertex(edge.startVertex);
            startVertex = this.getVertexByKey(edge.startVertex.getKey());
        }
        // Insert end vertex if it wasn't inserted.
        if (!endVertex) {
            this.addVertex(edge.endVertex);
            endVertex = this.getVertexByKey(edge.endVertex.getKey());
        }
        // Check if edge has been already added.
        if (this.edges[edge.getKey()]) {
            throw new Error('Edge has already been added before');
        }
        else {
            this.edges[edge.getKey()] = edge;
        }
        // Add edge to the vertices.
        if (this.isDirected) {
            // If graph IS directed then add the edge only to start vertex.
            startVertex.addEdge(edge);
        }
        else {
            // If graph ISN'T directed then add the edge to both vertices.
            startVertex.addEdge(edge);
            endVertex.addEdge(edge);
        }
        return this;
    };
    /**
     * 删除边
     *
     * @param edge 边
     */
    Graph.prototype.deleteEdge = function (edge) {
        // Delete edge from the list of edges.
        if (this.edges[edge.getKey()]) {
            delete this.edges[edge.getKey()];
        }
        else {
            throw new Error('Edge not found in graph');
        }
        // Try to find and end start vertices and delete edge from them.
        var startVertex = this.getVertexByKey(edge.startVertex.getKey());
        var endVertex = this.getVertexByKey(edge.endVertex.getKey());
        startVertex.deleteEdge(edge);
        endVertex.deleteEdge(edge);
    };
    /**
     * 查找边
     *
     * @param ertex 起始顶点
     * @param endVertex 结束顶点
     * @return 查找到的边
     */
    Graph.prototype.findEdge = function (startVertex, endVertex) {
        var vertex = this.getVertexByKey(startVertex.getKey());
        if (!vertex) {
            return null;
        }
        return vertex.findEdge(endVertex);
    };
    /**
     * 获取权重
     */
    Graph.prototype.getWeight = function () {
        return this.getAllEdges().reduce(function (weight, graphEdge) {
            return weight + graphEdge.weight;
        }, 0);
    };
    /**
     * Reverse all the edges in directed graph.
     */
    Graph.prototype.reverse = function () {
        var _this = this;
        this.getAllEdges().forEach(function (edge) {
            // Delete straight edge from graph and from vertices.
            _this.deleteEdge(edge);
            // Reverse the edge.
            edge.reverse();
            // Add reversed edge back to the graph and its vertices.
            _this.addEdge(edge);
        });
        return this;
    };
    /**
     * 获取所有顶点索引
     */
    Graph.prototype.getVerticesIndices = function () {
        var verticesIndices = {};
        this.getAllVertices().forEach(function (vertex, index) {
            verticesIndices[vertex.getKey()] = index;
        });
        return verticesIndices;
    };
    /**
     * 获取邻接矩阵
     */
    Graph.prototype.getAdjacencyMatrix = function () {
        var _this = this;
        var vertices = this.getAllVertices();
        var verticesIndices = this.getVerticesIndices();
        // Init matrix with infinities meaning that there is no ways of
        // getting from one vertex to another yet.
        var adjacencyMatrix = Array(vertices.length).fill(null).map(function () {
            return Array(vertices.length).fill(Infinity);
        });
        // Fill the columns.
        vertices.forEach(function (vertex, vertexIndex) {
            vertex.getNeighbors().forEach(function (neighbor) {
                var neighborIndex = verticesIndices[neighbor.getKey()];
                adjacencyMatrix[vertexIndex][neighborIndex] = _this.findEdge(vertex, neighbor).weight;
            });
        });
        return adjacencyMatrix;
    };
    /**
     * 转换为字符串
     */
    Graph.prototype.toString = function () {
        return Object.keys(this.vertices).toString();
    };
    return Graph;
}());

var Sort = /** @class */ (function () {
    function Sort(originalCallbacks) {
        this.callbacks = Sort.initSortingCallbacks(originalCallbacks);
        this.comparator = new Comparator(this.callbacks.compareCallback);
    }
    /**
     * @param originalCallbacks
     */
    Sort.initSortingCallbacks = function (originalCallbacks) {
        var callbacks = originalCallbacks || {};
        var stubCallback = function () { };
        callbacks.compareCallback = callbacks.compareCallback || undefined;
        callbacks.visitingCallback = callbacks.visitingCallback || stubCallback;
        return callbacks;
    };
    Sort.prototype.sort = function (_array) {
        throw new Error('sort method must be implemented');
    };
    return Sort;
}());

var QuickSort = /** @class */ (function (_super) {
    __extends(QuickSort, _super);
    function QuickSort() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    /**
     * @param originalArray
     */
    QuickSort.prototype.sort = function (originalArray) {
        // Clone original array to prevent it from modification.
        var array = __spreadArray([], __read(originalArray));
        // If array has less than or equal to one elements then it is already sorted.
        if (array.length <= 1) {
            return array;
        }
        // Init left and right arrays.
        var leftArray = [];
        var rightArray = [];
        // Take the first element of array as a pivot.
        var pivotElement = array.shift();
        var centerArray = [pivotElement];
        // Split all array elements between left, center and right arrays.
        while (array.length) {
            var currentElement = array.shift();
            // Call visiting callback.
            this.callbacks.visitingCallback(currentElement);
            if (this.comparator.equal(currentElement, pivotElement)) {
                centerArray.push(currentElement);
            }
            else if (this.comparator.lessThan(currentElement, pivotElement)) {
                leftArray.push(currentElement);
            }
            else {
                rightArray.push(currentElement);
            }
        }
        // Sort left and right arrays.
        var leftArraySorted = this.sort(leftArray);
        var rightArraySorted = this.sort(rightArray);
        // Let's now join sorted left array with center array and with sorted right array.
        return leftArraySorted.concat(centerArray, rightArraySorted);
    };
    return QuickSort;
}(Sort));

/**
 * @param graph
 */
function kruskal(graph) {
    // It should fire error if graph is directed since the algorithm works only
    // for undirected graphs.
    if (graph.isDirected) {
        throw new Error('Kruskal\'s algorithms works only for undirected graphs');
    }
    // Init new graph that will contain minimum spanning tree of original graph.
    var minimumSpanningTree = new Graph();
    // Sort all graph edges in increasing order.
    var sortingCallbacks = {
        compareCallback: function (graphEdgeA, graphEdgeB) {
            if (graphEdgeA.weight === graphEdgeB.weight) {
                return 1;
            }
            return graphEdgeA.weight <= graphEdgeB.weight ? -1 : 1;
        },
    };
    var sortedEdges = new QuickSort(sortingCallbacks).sort(graph.getAllEdges());
    // Create disjoint sets for all graph vertices.
    var keyCallback = function (graphVertex) { return graphVertex.getKey(); };
    var disjointSet = new DisjointSet(keyCallback);
    graph.getAllVertices().forEach(function (graphVertex) {
        disjointSet.makeSet(graphVertex);
    });
    // Go through all edges started from the minimum one and try to add them
    // to minimum spanning tree. The criteria of adding the edge would be whether
    // it is forms the cycle or not (if it connects two vertices from one disjoint
    // set or not).
    for (var edgeIndex = 0; edgeIndex < sortedEdges.length; edgeIndex += 1) {
        var currentEdge = sortedEdges[edgeIndex];
        // Check if edge forms the cycle. If it does then skip it.
        if (!disjointSet.inSameSet(currentEdge.startVertex, currentEdge.endVertex)) {
            // Unite two subsets into one.
            disjointSet.union(currentEdge.startVertex, currentEdge.endVertex);
            // Add this edge to spanning tree.
            minimumSpanningTree.addEdge(currentEdge);
        }
    }
    return minimumSpanningTree;
}

/**
 * @param graph
 */
function prim(graph) {
    // It should fire error if graph is directed since the algorithm works only
    // for undirected graphs.
    if (graph.isDirected) {
        throw new Error('Prim\'s algorithms works only for undirected graphs');
    }
    // Init new graph that will contain minimum spanning tree of original graph.
    var minimumSpanningTree = new Graph();
    // This priority queue will contain all the edges that are starting from
    // visited nodes and they will be ranked by edge weight - so that on each step
    // we would always pick the edge with minimal edge weight.
    var edgesQueue = new PriorityQueue();
    // Set of vertices that has been already visited.
    var visitedVertices = {};
    // Vertex from which we will start graph traversal.
    var startVertex = graph.getAllVertices()[0];
    // Add start vertex to the set of visited ones.
    visitedVertices[startVertex.getKey()] = startVertex;
    // Add all edges of start vertex to the queue.
    startVertex.getEdges().forEach(function (graphEdge) {
        edgesQueue.add(graphEdge, graphEdge.weight);
    });
    // Now let's explore all queued edges.
    while (!edgesQueue.isEmpty()) {
        // Fetch next queued edge with minimal weight.
        var currentMinEdge = edgesQueue.poll();
        // Find out the next unvisited minimal vertex to traverse.
        var nextMinVertex = null;
        if (!visitedVertices[currentMinEdge.startVertex.getKey()]) {
            nextMinVertex = currentMinEdge.startVertex;
        }
        else if (!visitedVertices[currentMinEdge.endVertex.getKey()]) {
            nextMinVertex = currentMinEdge.endVertex;
        }
        // If all vertices of current edge has been already visited then skip this round.
        if (nextMinVertex) {
            // Add current min edge to MST.
            minimumSpanningTree.addEdge(currentMinEdge);
            // Add vertex to the set of visited ones.
            visitedVertices[nextMinVertex.getKey()] = nextMinVertex;
            // Add all current vertex's edges to the queue.
            nextMinVertex.getEdges().forEach(function (graphEdge) {
                // Add only vertices that link to unvisited nodes.
                if (!visitedVertices[graphEdge.startVertex.getKey()]
                    || !visitedVertices[graphEdge.endVertex.getKey()]) {
                    edgesQueue.add(graphEdge, graphEdge.weight);
                }
            });
        }
    }
    return minimumSpanningTree;
}

/**
 * 栈
 *
 * 后进先出
 */
var Stack = /** @class */ (function () {
    function Stack() {
        this.linkedList = new LinkedList();
        // We're going to implement Stack based on LinkedList since these
        // structures are quite similar. Compare push/pop operations of the Stack
        // with prepend/deleteHead operations of LinkedList.
        this.linkedList = new LinkedList();
    }
    /**
     * 是否为空
     */
    Stack.prototype.isEmpty = function () {
        // The stack is empty if its linked list doesn't have a head.
        return !this.linkedList.head;
    };
    /**
     * 查看第一个元素值
     */
    Stack.prototype.peek = function () {
        if (this.isEmpty()) {
            // If the linked list is empty then there is nothing to peek from.
            return null;
        }
        // Just read the value from the start of linked list without deleting it.
        return this.linkedList.head.value;
    };
    /**
     * 入栈
     *
     * @param value 元素值
     */
    Stack.prototype.push = function (value) {
        // Pushing means to lay the value on top of the stack. Therefore let's just add
        // the new value at the start of the linked list.
        this.linkedList.prepend(value);
    };
    /**
     * 出栈
     */
    Stack.prototype.pop = function () {
        // Let's try to delete the first node (the head) from the linked list.
        // If there is no head (the linked list is empty) just return null.
        var removedHead = this.linkedList.deleteHead();
        return removedHead ? removedHead.value : null;
    };
    /**
     * 转换为数组
     */
    Stack.prototype.toArray = function () {
        return this.linkedList
            .toArray()
            .map(function (linkedListNode) { return linkedListNode.value; });
    };
    /**
     * 转换为字符串
     *
     * @param callback 值输出为字符串函数
     */
    Stack.prototype.toString = function (callback) {
        return this.linkedList.toString(callback);
    };
    return Stack;
}());

/**
 * @param graph
 */
function getVerticesSortedByDfsFinishTime(graph) {
    // Set of all visited vertices during DFS pass.
    var visitedVerticesSet = {};
    // Stack of vertices by finish time.
    // All vertices in this stack are ordered by finished time in decreasing order.
    // Vertex that has been finished first will be at the bottom of the stack and
    // vertex that has been finished last will be at the top of the stack.
    var verticesByDfsFinishTime = new Stack();
    // Set of all vertices we're going to visit.
    var notVisitedVerticesSet = {};
    graph.getAllVertices().forEach(function (vertex) {
        notVisitedVerticesSet[vertex.getKey()] = vertex;
    });
    // Specify DFS traversal callbacks.
    var dfsCallbacks = {
        enterVertex: function (_a) {
            var currentVertex = _a.currentVertex;
            // Add current vertex to visited set.
            visitedVerticesSet[currentVertex.getKey()] = currentVertex;
            // Delete current vertex from not visited set.
            delete notVisitedVerticesSet[currentVertex.getKey()];
        },
        leaveVertex: function (_a) {
            var currentVertex = _a.currentVertex;
            // Push vertex to the stack when leaving it.
            // This will make stack to be ordered by finish time in decreasing order.
            verticesByDfsFinishTime.push(currentVertex);
        },
        // Don't allow to traverse the nodes that have been already visited.
        allowTraversal: function (_a) {
            var nextVertex = _a.nextVertex;
            return !visitedVerticesSet[nextVertex.getKey()];
        },
    };
    // Do FIRST DFS PASS traversal for all graph vertices to fill the verticesByFinishTime stack.
    while (Object.values(notVisitedVerticesSet).length) {
        // Peek any vertex to start DFS traversal from.
        var startVertexKey = Object.keys(notVisitedVerticesSet)[0];
        var startVertex = notVisitedVerticesSet[startVertexKey];
        delete notVisitedVerticesSet[startVertexKey];
        depthFirstSearch(graph, startVertex, dfsCallbacks);
    }
    return verticesByDfsFinishTime;
}
/**
 * @param graph
 * @param verticesByFinishTime
 */
function getSCCSets(graph, verticesByFinishTime) {
    // Array of arrays of strongly connected vertices.
    var stronglyConnectedComponentsSets = [];
    // Array that will hold all vertices that are being visited during one DFS run.
    var stronglyConnectedComponentsSet = [];
    // Visited vertices set.
    var visitedVerticesSet = {};
    // Callbacks for DFS traversal.
    var dfsCallbacks = {
        enterVertex: function (_a) {
            var currentVertex = _a.currentVertex;
            // Add current vertex to SCC set of current DFS round.
            stronglyConnectedComponentsSet.push(currentVertex);
            // Add current vertex to visited set.
            visitedVerticesSet[currentVertex.getKey()] = currentVertex;
        },
        leaveVertex: function (_a) {
            var previousVertex = _a.previousVertex;
            // Once DFS traversal is finished push the set of found strongly connected
            // components during current DFS round to overall strongly connected components set.
            // The sign that traversal is about to be finished is that we came back to start vertex
            // which doesn't have parent.
            if (previousVertex === null) {
                stronglyConnectedComponentsSets.push(__spreadArray([], __read(stronglyConnectedComponentsSet)));
            }
        },
        allowTraversal: function (_a) {
            var nextVertex = _a.nextVertex;
            // Don't allow traversal of already visited vertices.
            return !visitedVerticesSet[nextVertex.getKey()];
        },
    };
    while (!verticesByFinishTime.isEmpty()) {
        /** @var {GraphVertex} startVertex */
        var startVertex = verticesByFinishTime.pop();
        // Reset the set of strongly connected vertices.
        stronglyConnectedComponentsSet = [];
        // Don't do DFS on already visited vertices.
        if (!visitedVerticesSet[startVertex.getKey()]) {
            // Do DFS traversal.
            depthFirstSearch(graph, startVertex, dfsCallbacks);
        }
    }
    return stronglyConnectedComponentsSets;
}
/**
 * Kosaraju's algorithm.
 *
 * @param graph
 */
function stronglyConnectedComponents(graph) {
    // In this algorithm we will need to do TWO DFS PASSES overt the graph.
    // Get stack of vertices ordered by DFS finish time.
    // All vertices in this stack are ordered by finished time in decreasing order:
    // Vertex that has been finished first will be at the bottom of the stack and
    // vertex that has been finished last will be at the top of the stack.
    var verticesByFinishTime = getVerticesSortedByDfsFinishTime(graph);
    // Reverse the graph.
    graph.reverse();
    // Do DFS once again on reversed graph.
    return getSCCSets(graph, verticesByFinishTime);
}

/**
 * @param graph
 */
function topologicalSort(graph) {
    // Create a set of all vertices we want to visit.
    var unvisitedSet = {};
    graph.getAllVertices().forEach(function (vertex) {
        unvisitedSet[vertex.getKey()] = vertex;
    });
    // Create a set for all vertices that we've already visited.
    var visitedSet = {};
    // Create a stack of already ordered vertices.
    var sortedStack = new Stack();
    var dfsCallbacks = {
        enterVertex: function (_a) {
            var currentVertex = _a.currentVertex;
            // Add vertex to visited set in case if all its children has been explored.
            visitedSet[currentVertex.getKey()] = currentVertex;
            // Remove this vertex from unvisited set.
            delete unvisitedSet[currentVertex.getKey()];
        },
        leaveVertex: function (_a) {
            var currentVertex = _a.currentVertex;
            // If the vertex has been totally explored then we may push it to stack.
            sortedStack.push(currentVertex);
        },
        allowTraversal: function (_a) {
            var nextVertex = _a.nextVertex;
            return !visitedSet[nextVertex.getKey()];
        },
    };
    // Let's go and do DFS for all unvisited nodes.
    while (Object.keys(unvisitedSet).length) {
        var currentVertexKey = Object.keys(unvisitedSet)[0];
        var currentVertex = unvisitedSet[currentVertexKey];
        // Do DFS for current node.
        depthFirstSearch(graph, currentVertex, dfsCallbacks);
    }
    return sortedStack.toArray();
}

/**
 * Get all possible paths
 * @param {GraphVertex} startVertex
 * @param {GraphVertex[][]} [paths]
 * @param {GraphVertex[]} [path]
 */
function findAllPaths(startVertex, paths, path) {
    if (paths === void 0) { paths = []; }
    if (path === void 0) { path = []; }
    // Clone path.
    var currentPath = __spreadArray([], __read(path));
    // Add startVertex to the path.
    currentPath.push(startVertex);
    // Generate visited set from path.
    var visitedSet = currentPath.reduce(function (accumulator, vertex) {
        var updatedAccumulator = __assign({}, accumulator);
        updatedAccumulator[vertex.getKey()] = vertex;
        return updatedAccumulator;
    }, {});
    // Get all unvisited neighbors of startVertex.
    var unvisitedNeighbors = startVertex.getNeighbors().filter(function (neighbor) {
        return !visitedSet[neighbor.getKey()];
    });
    // If there no unvisited neighbors then treat current path as complete and save it.
    if (!unvisitedNeighbors.length) {
        paths.push(currentPath);
        return paths;
    }
    // Go through all the neighbors.
    for (var neighborIndex = 0; neighborIndex < unvisitedNeighbors.length; neighborIndex += 1) {
        var currentUnvisitedNeighbor = unvisitedNeighbors[neighborIndex];
        findAllPaths(currentUnvisitedNeighbor, paths, currentPath);
    }
    return paths;
}
/**
 * @param adjacencyMatrix
 * @param verticesIndices
 * @param cycle
 */
function getCycleWeight(adjacencyMatrix, verticesIndices, cycle) {
    var weight = 0;
    for (var cycleIndex = 1; cycleIndex < cycle.length; cycleIndex += 1) {
        var fromVertex = cycle[cycleIndex - 1];
        var toVertex = cycle[cycleIndex];
        var fromVertexIndex = verticesIndices[fromVertex.getKey()];
        var toVertexIndex = verticesIndices[toVertex.getKey()];
        weight += adjacencyMatrix[fromVertexIndex][toVertexIndex];
    }
    return weight;
}
/**
 * BRUTE FORCE approach to solve Traveling Salesman Problem.
 *
 * @param graph
 */
function bfTravellingSalesman(graph) {
    // Pick starting point from where we will traverse the graph.
    var startVertex = graph.getAllVertices()[0];
    // BRUTE FORCE.
    // Generate all possible paths from startVertex.
    var allPossiblePaths = findAllPaths(startVertex);
    // Filter out paths that are not cycles.
    var allPossibleCycles = allPossiblePaths.filter(function (path) {
        /** @var {GraphVertex} */
        var lastVertex = path[path.length - 1];
        var lastVertexNeighbors = lastVertex.getNeighbors();
        return lastVertexNeighbors.includes(startVertex);
    });
    // Go through all possible cycles and pick the one with minimum overall tour weight.
    var adjacencyMatrix = graph.getAdjacencyMatrix();
    var verticesIndices = graph.getVerticesIndices();
    var salesmanPath = [];
    var salesmanPathWeight = null;
    for (var cycleIndex = 0; cycleIndex < allPossibleCycles.length; cycleIndex += 1) {
        var currentCycle = allPossibleCycles[cycleIndex];
        var currentCycleWeight = getCycleWeight(adjacencyMatrix, verticesIndices, currentCycle);
        // If current cycle weight is smaller then previous ones treat current cycle as most optimal.
        if (salesmanPathWeight === null || currentCycleWeight < salesmanPathWeight) {
            salesmanPath = currentCycle;
            salesmanPathWeight = currentCycleWeight;
        }
    }
    // Return the solution.
    return salesmanPath;
}

/**
 * Helper function that returns the color of the pixel.
 * @param img
 * @param coordinate
 */
var getPixel = function (img, _a) {
    var x = _a.x, y = _a.y;
    // The ImageData data array is a flat 1D array.
    // Thus we need to convert x and y coordinates to the linear index.
    var i = y * img.width + x;
    var cellsPerColor = 4; // RGBA
    // For better efficiency, instead of creating a new sub-array we return
    // a pointer to the part of the ImageData array.
    return img.data.subarray(i * cellsPerColor, i * cellsPerColor + cellsPerColor);
};
/**
 * Helper function that sets the color of the pixel.
 *
 * @param img
 * @param coordinate
 * @param color
 */
var setPixel = function (img, _a, color) {
    var x = _a.x, y = _a.y;
    // The ImageData data array is a flat 1D array.
    // Thus we need to convert x and y coordinates to the linear index.
    var i = y * img.width + x;
    var cellsPerColor = 4; // RGBA
    img.data.set(color, i * cellsPerColor);
};

/**
 * Helper function that creates a matrix (2D array) of specific
 * size (w x h) and fills it with specified value.
 *
 * @param w
 * @param h
 * @param filler
 */
function matrix(w, h, filler) {
    return new Array(h)
        .fill(null)
        .map(function () {
        return new Array(w).fill(filler);
    });
}
/**
 * Calculates the energy of a pixel.
 *
 * @param left
 * @param middle
 * @param right
 */
var getPixelEnergy = function (left, middle, right) {
    // Middle pixel is the pixel we're calculating the energy for.
    var _a = __read(middle, 3), mR = _a[0], mG = _a[1], mB = _a[2];
    // Energy from the left pixel (if it exists).
    var lEnergy = 0;
    if (left) {
        var _b = __read(left, 3), lR = _b[0], lG = _b[1], lB = _b[2];
        lEnergy = Math.pow((lR - mR), 2) + Math.pow((lG - mG), 2) + Math.pow((lB - mB), 2);
    }
    // Energy from the right pixel (if it exists).
    var rEnergy = 0;
    if (right) {
        var _c = __read(right, 3), rR = _c[0], rG = _c[1], rB = _c[2];
        rEnergy = Math.pow((rR - mR), 2) + Math.pow((rG - mG), 2) + Math.pow((rB - mB), 2);
    }
    // Resulting pixel energy.
    return Math.sqrt(lEnergy + rEnergy);
};
/**
 * Calculates the energy of each pixel of the image.
 *
 * @param img
 * @param size
 */
var calculateEnergyMap = function (img, _a) {
    var w = _a.w, h = _a.h;
    // Create an empty energy map where each pixel has infinitely high energy.
    // We will update the energy of each pixel.
    var energyMap = matrix(w, h, Infinity);
    for (var y = 0; y < h; y += 1) {
        for (var x = 0; x < w; x += 1) {
            // Left pixel might not exist if we're on the very left edge of the image.
            var left = (x - 1) >= 0 ? getPixel(img, { x: x - 1, y: y }) : null;
            // The color of the middle pixel that we're calculating the energy for.
            var middle = getPixel(img, { x: x, y: y });
            // Right pixel might not exist if we're on the very right edge of the image.
            var right = (x + 1) < w ? getPixel(img, { x: x + 1, y: y }) : null;
            energyMap[y][x] = getPixelEnergy(left, middle, right);
        }
    }
    return energyMap;
};
/**
 * Finds the seam (the sequence of pixels from top to bottom) that has the
 * lowest resulting energy using the Dynamic Programming approach.
 * @param energyMap
 * @param size
 * @returns
 */
var findLowEnergySeam = function (energyMap, _a) {
    var w = _a.w, h = _a.h;
    // The 2D array of the size of w and h, where each pixel contains the
    // seam metadata (pixel energy, pixel coordinate and previous pixel from
    // the lowest energy seam at this point).
    var seamPixelsMap = matrix(w, h, null);
    // Populate the first row of the map by just copying the energies
    // from the energy map.
    for (var x = 0; x < w; x += 1) {
        var y = 0;
        seamPixelsMap[y][x] = {
            energy: energyMap[y][x],
            coordinate: { x: x, y: y },
            previous: null,
        };
    }
    // Populate the rest of the rows.
    for (var y = 1; y < h; y += 1) {
        for (var x = 0; x < w; x += 1) {
            // Find the top adjacent cell with minimum energy.
            // This cell would be the tail of a seam with lowest energy at this point.
            // It doesn't mean that this seam (path) has lowest energy globally.
            // Instead, it means that we found a path with the lowest energy that may lead
            // us to the current pixel with the coordinates x and y.
            var minPrevEnergy = Infinity;
            var minPrevX = x;
            for (var i = (x - 1); i <= (x + 1); i += 1) {
                if (i >= 0 && i < w && seamPixelsMap[y - 1][i].energy < minPrevEnergy) {
                    minPrevEnergy = seamPixelsMap[y - 1][i].energy;
                    minPrevX = i;
                }
            }
            // Update the current cell.
            seamPixelsMap[y][x] = {
                energy: minPrevEnergy + energyMap[y][x],
                coordinate: { x: x, y: y },
                previous: { x: minPrevX, y: y - 1 },
            };
        }
    }
    // Find where the minimum energy seam ends.
    // We need to find the tail of the lowest energy seam to start
    // traversing it from its tail to its head (from the bottom to the top).
    var lastMinCoordinate = null;
    var minSeamEnergy = Infinity;
    for (var x = 0; x < w; x += 1) {
        var y = h - 1;
        if (seamPixelsMap[y][x].energy < minSeamEnergy) {
            minSeamEnergy = seamPixelsMap[y][x].energy;
            lastMinCoordinate = { x: x, y: y };
        }
    }
    // Find the lowest energy energy seam.
    // Once we know where the tail is we may traverse and assemble the lowest
    // energy seam based on the "previous" value of the seam pixel metadata.
    var seam = [];
    var lastMinX = lastMinCoordinate.x, lastMinY = lastMinCoordinate.y;
    // Adding new pixel to the seam path one by one until we reach the top.
    var currentSeam = seamPixelsMap[lastMinY][lastMinX];
    while (currentSeam) {
        seam.push(currentSeam.coordinate);
        var prevMinCoordinates = currentSeam.previous;
        if (!prevMinCoordinates) {
            currentSeam = null;
        }
        else {
            var prevMinX = prevMinCoordinates.x, prevMinY = prevMinCoordinates.y;
            currentSeam = seamPixelsMap[prevMinY][prevMinX];
        }
    }
    return seam;
};
/**
 * Deletes the seam from the image data.
 * We delete the pixel in each row and then shift the rest of the row pixels to the left.
 *
 * @param img
 * @param seam
 * @param size
 */
var deleteSeam = function (img, seam, _a) {
    var w = _a.w;
    seam.forEach(function (_a) {
        var seamX = _a.x, seamY = _a.y;
        for (var x = seamX; x < (w - 1); x += 1) {
            var nextPixel = getPixel(img, { x: x + 1, y: seamY });
            setPixel(img, { x: x, y: seamY }, nextPixel);
        }
    });
};
/**
 * Performs the content-aware image width resizing using the seam carving method.
 *
 * @param args
 */
function resizeImageWidth(_a) {
    var img = _a.img, toWidth = _a.toWidth;
    /**
     * For performance reasons we want to avoid changing the img data array size.
     * Instead we'll just keep the record of the resized image width and height separately.
     */
    var size = { w: img.width, h: img.height };
    // Calculating the number of pixels to remove.
    var pxToRemove = img.width - toWidth;
    var energyMap = null;
    var seam = null;
    // Removing the lowest energy seams one by one.
    for (var i = 0; i < pxToRemove; i += 1) {
        // 1. Calculate the energy map for the current version of the image.
        energyMap = calculateEnergyMap(img, size);
        // 2. Find the seam with the lowest energy based on the energy map.
        seam = findLowEnergySeam(energyMap, size);
        // 3. Delete the seam with the lowest energy seam from the image.
        deleteSeam(img, seam, size);
        // Reduce the image width, and continue iterations.
        size.w -= 1;
    }
    // Returning the resized image and its final size.
    // The img is actually a reference to the ImageData, so technically
    // the caller of the function already has this pointer. But let's
    // still return it for better code readability.
    return { img: img, size: size };
}

/**
 * @param node
 * @param callback
 */
function reverseTraversalRecursive(node, callback) {
    if (node) {
        reverseTraversalRecursive(node.next, callback);
        callback(node.value);
    }
}
/**
 * @param linkedList
 * @param callback
 */
function reverseTraversal(linkedList, callback) {
    reverseTraversalRecursive(linkedList.head, callback);
}

/**
 * @param linkedList
 * @param callback
 */
function traversal(linkedList, callback) {
    var currentNode = linkedList.head;
    while (currentNode) {
        callback(currentNode.value);
        currentNode = currentNode.next;
    }
}

/**
 * ┌───────────────── sign bit
 * │   ┌───────────── exponent bits
 * │   │       ┌───── fraction bits
 * │   │       │
 * X XXXXX XXXXXXXXXX
 *
 */
var precisionConfigs = {
    // @see: https://en.wikipedia.org/wiki/Half-precision_floating-point_format
    half: {
        signBitsCount: 1,
        exponentBitsCount: 5,
        fractionBitsCount: 10,
    },
    // @see: https://en.wikipedia.org/wiki/Single-precision_floating-point_format
    single: {
        signBitsCount: 1,
        exponentBitsCount: 8,
        fractionBitsCount: 23,
    },
    // @see: https://en.wikipedia.org/wiki/Double-precision_floating-point_format
    double: {
        signBitsCount: 1,
        exponentBitsCount: 11,
        fractionBitsCount: 52,
    },
};
/**
 * Converts the binary representation of the floating point number to decimal float number.
 *
 * @param bits sequence of bits that represents the floating point number.
 * @param precisionConfig half/single/double precision config.
 * @return floating point number decoded from its binary representation.
 */
function bitsToFloat(bits, precisionConfig) {
    var signBitsCount = precisionConfig.signBitsCount, exponentBitsCount = precisionConfig.exponentBitsCount;
    // Figuring out the sign.
    var sign = Math.pow((-1), bits[0]); // -1^1 = -1, -1^0 = 1
    // Calculating the exponent value.
    var exponentBias = Math.pow(2, (exponentBitsCount - 1)) - 1;
    var exponentBits = bits.slice(signBitsCount, signBitsCount + exponentBitsCount);
    var exponentUnbiased = exponentBits.reduce(function (exponentSoFar, currentBit, bitIndex) {
        var bitPowerOfTwo = Math.pow(2, (exponentBitsCount - bitIndex - 1));
        return exponentSoFar + currentBit * bitPowerOfTwo;
    }, 0);
    var exponent = exponentUnbiased - exponentBias;
    // Calculating the fraction value.
    var fractionBits = bits.slice(signBitsCount + exponentBitsCount);
    var fraction = fractionBits.reduce(function (fractionSoFar, currentBit, bitIndex) {
        var bitPowerOfTwo = Math.pow(2, -(bitIndex + 1));
        return fractionSoFar + currentBit * bitPowerOfTwo;
    }, 0);
    // Putting all parts together to calculate the final number.
    return sign * (Math.pow(2, exponent)) * (1 + fraction);
}
/**
 *  Converts the 16-bit binary representation of the floating point number to decimal float number.
 *
 * @param bits sequence of bits that represents the floating point number.
 * @return floating point number decoded from its binary representation.
 */
function bitsToFloat16(bits) {
    return bitsToFloat(bits, precisionConfigs.half);
}
/**
 * Converts the 32-bit binary representation of the floating point number to decimal float number.
 *
 * @param bits sequence of bits that represents the floating point number.
 * @return floating point number decoded from its binary representation.
 */
function bitsToFloat32(bits) {
    return bitsToFloat(bits, precisionConfigs.single);
}
/**
 * Converts the 64-bit binary representation of the floating point number to decimal float number.
 *
 * @param bits sequence of bits that represents the floating point number.
 * @return floating point number decoded from its binary representation.
 */
function bitsToFloat64(bits) {
    return bitsToFloat(bits, precisionConfigs.double);
}

// @see: https://en.wikipedia.org/wiki/Single-precision_floating-point_format
var singlePrecisionBytesLength = 4; // 32 bits
// @see: https://en.wikipedia.org/wiki/Double-precision_floating-point_format
var doublePrecisionBytesLength = 8; // 64 bits
var bitsInByte = 8;
/**
 * Converts the float number into its IEEE 754 binary representation.
 * @see: https://en.wikipedia.org/wiki/IEEE_754
 *
 * @param floatNumber float number in decimal format.
 * @param byteLength number of bytes to use to store the float number.
 * @return binary string representation of the float number.
 */
function floatAsBinaryString(floatNumber, byteLength) {
    var numberAsBinaryString = '';
    var arrayBuffer = new ArrayBuffer(byteLength);
    var dataView = new DataView(arrayBuffer);
    var byteOffset = 0;
    var littleEndian = false;
    if (byteLength === singlePrecisionBytesLength) {
        dataView.setFloat32(byteOffset, floatNumber, littleEndian);
    }
    else {
        dataView.setFloat64(byteOffset, floatNumber, littleEndian);
    }
    for (var byteIndex = 0; byteIndex < byteLength; byteIndex += 1) {
        var bits = dataView.getUint8(byteIndex).toString(2);
        if (bits.length < bitsInByte) {
            bits = new Array(bitsInByte - bits.length).fill('0').join('') + bits;
        }
        numberAsBinaryString += bits;
    }
    return numberAsBinaryString;
}
/**
 * Converts the float number into its IEEE 754 64-bits binary representation.
 *
 * @param floatNumber float number in decimal format.
 * @return 64 bits binary string representation of the float number.
 */
function floatAs64BinaryString(floatNumber) {
    return floatAsBinaryString(floatNumber, doublePrecisionBytesLength);
}
/**
 * Converts the float number into its IEEE 754 32-bits binary representation.
 *
 * @param floatNumber float number in decimal format.
 * @return 32 bits binary string representation of the float number.
 */
function floatAs32BinaryString(floatNumber) {
    return floatAsBinaryString(floatNumber, singlePrecisionBytesLength);
}

/**
 * Return the number of bits used in the binary representation of the number.
 *
 * @param number
 */
function bitLength(number) {
    var bitsCounter = 0;
    while ((1 << bitsCounter) <= number) {
        bitsCounter += 1;
    }
    return bitsCounter;
}

/**
 * @param originalNumber
 */
function countSetBits(originalNumber) {
    var setBitsCount = 0;
    var number = originalNumber;
    while (number) {
        // Add last bit of the number to the sum of set bits.
        setBitsCount += number & 1;
        // Shift number right by one bit to investigate other bits.
        number >>>= 1;
    }
    return setBitsCount;
}

/**
 * Counts the number of bits that need to be change in order
 * to convert numberA to numberB.
 *
 * @param numberA
 * @param numberB
 */
function bitsDiff(numberA, numberB) {
    return countSetBits(numberA ^ numberB);
}

/**
 * @param number
 * @param bitPosition zero based.
 */
function clearBit(number, bitPosition) {
    var mask = ~(1 << bitPosition);
    return number & mask;
}

/**
 * @param number
 */
function divideByTwo(number) {
    return number >> 1;
}

/**
 * @param number
 * @param bitPosition zero based.
 */
function getBit(number, bitPosition) {
    return (number >> bitPosition) & 1;
}

/**
 * Add two numbers using only binary operators.
 *
 * This is an implementation of full adders logic circuit.
 * https://en.wikipedia.org/wiki/Adder_(electronics)
 * Inspired by: https://www.youtube.com/watch?v=wvJc9CZcvBc
 *
 * Table(1)
 *  INPUT  | OUT
 *  C Ai Bi | C Si | Row
 * -------- | -----| ---
 *  0  0  0 | 0  0 | 1
 *  0  0  1 | 0  1 | 2
 *  0  1  0 | 0  1 | 3
 *  0  1  1 | 1  0 | 4
 * -------- | ---- | --
 *  1  0  0 | 0  1 | 5
 *  1  0  1 | 1  0 | 6
 *  1  1  0 | 1  0 | 7
 *  1  1  1 | 1  1 | 8
 * ---------------------
 *
 * Legend:
 * INPUT C = Carry in, from the previous less-significant stage
 * INPUT Ai = ith bit of Number A
 * INPUT Bi = ith bit of Number B
 * OUT C = Carry out to the next most-significant stage
 * OUT Si = Bit Sum, ith least significant bit of the result
 *
 *
 * @param a
 * @param b
 */
function fullAdder(a, b) {
    var result = 0;
    var carry = 0;
    // The operands of all bitwise operators are converted to signed
    // 32-bit integers in two's complement format.
    // https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Operators/Bitwise_Operators#Signed_32-bit_integers
    for (var i = 0; i < 32; i += 1) {
        var ai = getBit(a, i);
        var bi = getBit(b, i);
        var carryIn = carry;
        // Calculate binary Ai + Bi without carry (half adder)
        // See Table(1) rows 1 - 4: Si = Ai ^ Bi
        var aiPlusBi = ai ^ bi;
        // Calculate ith bit of the result by adding the carry bit to Ai + Bi
        // For Table(1) rows 5 - 8 carryIn = 1: Si = Ai ^ Bi ^ 1, flip the bit
        // Fpr Table(1) rows 1 - 4 carryIn = 0: Si = Ai ^ Bi ^ 0, a no-op.
        var bitSum = aiPlusBi ^ carryIn;
        // Carry out one to the next most-significant stage
        // when at least one of these is true:
        // 1) Table(1) rows 6, 7: one of Ai OR Bi is 1 AND carryIn = 1
        // 2) Table(1) rows 4, 8: Both Ai AND Bi are 1
        var carryOut = (aiPlusBi & carryIn) | (ai & bi);
        carry = carryOut;
        // Set ith least significant bit of the result to bitSum.
        result |= bitSum << i;
    }
    return result;
}

/**
 * @param number
 */
function isEven(number) {
    return (number & 1) === 0;
}

/**
 * @param number 32-bit integer.
 */
function isPositive(number) {
    // Zero is neither a positive nor a negative number.
    if (number === 0) {
        return false;
    }
    // The most significant 32nd bit can be used to determine whether the number is positive.
    return ((number >> 31) & 1) === 0;
}

/**
 * @param number
 */
function isPowerOfTwo(number) {
    return (number & (number - 1)) === 0;
}

/**
 * @param number
 */
function multiplyByTwo(number) {
    return number << 1;
}

/**
 * Multiply two signed numbers using bitwise operations.
 *
 * If a is zero or b is zero or if both a and b are zeros:
 * multiply(a, b) = 0
 *
 * If b is even:
 * multiply(a, b) = multiply(2a, b/2)
 *
 * If b is odd and b is positive:
 * multiply(a, b) = multiply(2a, (b-1)/2) + a
 *
 * If b is odd and b is negative:
 * multiply(a, b) = multiply(2a, (b+1)/2) - a
 *
 * Time complexity: O(log b)
 *
 * @param a
 * @param b
 */
function multiply(a, b) {
    // If a is zero or b is zero or if both a and b are zeros then the production is also zero.
    if (b === 0 || a === 0) {
        return 0;
    }
    // Otherwise we will have four different cases that are described above.
    var multiplyByOddPositive = function () { return multiply(multiplyByTwo(a), divideByTwo(b - 1)) + a; };
    var multiplyByOddNegative = function () { return multiply(multiplyByTwo(a), divideByTwo(b + 1)) - a; };
    var multiplyByEven = function () { return multiply(multiplyByTwo(a), divideByTwo(b)); };
    var multiplyByOdd = function () { return (isPositive(b) ? multiplyByOddPositive() : multiplyByOddNegative()); };
    return isEven(b) ? multiplyByEven() : multiplyByOdd();
}

/**
 * Multiply to unsigned numbers using bitwise operator.
 *
 * The main idea of bitwise multiplication is that every number may be split
 * to the sum of powers of two:
 *
 * I.e. 19 = 2^4 + 2^1 + 2^0
 *
 * Then multiplying number x by 19 is equivalent of:
 *
 * x * 19 = x * 2^4 + x * 2^1 + x * 2^0
 *
 * Now we need to remember that (x * 2^4) is equivalent of shifting x left by 4 bits (x << 4).
 *
 * @param number1
 * @param number2
 */
function multiplyUnsigned(number1, number2) {
    var result = 0;
    // Let's treat number2 as a multiplier for the number1.
    var multiplier = number2;
    // Multiplier current bit index.
    var bitIndex = 0;
    // Go through all bits of number2.
    while (multiplier !== 0) {
        // Check if current multiplier bit is set.
        if (multiplier & 1) {
            // In case if multiplier's bit at position bitIndex is set
            // it would mean that we need to multiply number1 by the power
            // of bit with index bitIndex and then add it to the result.
            result += (number1 << bitIndex);
        }
        bitIndex += 1;
        multiplier >>= 1;
    }
    return result;
}

/**
 * @param number
 * @param bitPosition zero based.
 */
function setBit(number, bitPosition) {
    return number | (1 << bitPosition);
}

/**
 * Switch the sign of the number using "Twos Complement" approach.
 * @param number
 */
function switchSign(number) {
    return ~number + 1;
}

/**
 * @param radian
 */
function radianToDegree(radian) {
    return radian * (180 / Math.PI);
}

/**
 * 复数 (数学)
 */
var ComplexNumber = /** @class */ (function () {
    /**
     * z = re + im * i
     * z = radius * e^(i * phase)
     *
     */
    function ComplexNumber(_a) {
        var _b = _a === void 0 ? {} : _a, _c = _b.re, re = _c === void 0 ? 0 : _c, _d = _b.im, im = _d === void 0 ? 0 : _d;
        this.re = re;
        this.im = im;
    }
    /**
     * @param addend
     */
    ComplexNumber.prototype.add = function (addend) {
        // Make sure we're dealing with complex number.
        var complexAddend = this.toComplexNumber(addend);
        return new ComplexNumber({
            re: this.re + complexAddend.re,
            im: this.im + complexAddend.im,
        });
    };
    /**
     * @param subtrahend
     */
    ComplexNumber.prototype.subtract = function (subtrahend) {
        // Make sure we're dealing with complex number.
        var complexSubtrahend = this.toComplexNumber(subtrahend);
        return new ComplexNumber({
            re: this.re - complexSubtrahend.re,
            im: this.im - complexSubtrahend.im,
        });
    };
    /**
     * @param multiplicand
     */
    ComplexNumber.prototype.multiply = function (multiplicand) {
        // Make sure we're dealing with complex number.
        var complexMultiplicand = this.toComplexNumber(multiplicand);
        return new ComplexNumber({
            re: this.re * complexMultiplicand.re - this.im * complexMultiplicand.im,
            im: this.re * complexMultiplicand.im + this.im * complexMultiplicand.re,
        });
    };
    /**
     * @param divider
     */
    ComplexNumber.prototype.divide = function (divider) {
        // Make sure we're dealing with complex number.
        var complexDivider = this.toComplexNumber(divider);
        // Get divider conjugate.
        var dividerConjugate = this.conjugate(complexDivider);
        // Multiply dividend by divider's conjugate.
        var finalDivident = this.multiply(dividerConjugate);
        // Calculating final divider using formula (a + bi)(a − bi) = a^2 + b^2
        var finalDivider = (Math.pow(complexDivider.re, 2)) + (Math.pow(complexDivider.im, 2));
        return new ComplexNumber({
            re: finalDivident.re / finalDivider,
            im: finalDivident.im / finalDivider,
        });
    };
    /**
     * @param number
     */
    ComplexNumber.prototype.conjugate = function (number) {
        // Make sure we're dealing with complex number.
        var complexNumber = this.toComplexNumber(number);
        return new ComplexNumber({
            re: complexNumber.re,
            im: -1 * complexNumber.im,
        });
    };
    ComplexNumber.prototype.getRadius = function () {
        return Math.sqrt((Math.pow(this.re, 2)) + (Math.pow(this.im, 2)));
    };
    /**
     * @param inRadians
     */
    ComplexNumber.prototype.getPhase = function (inRadians) {
        if (inRadians === void 0) { inRadians = true; }
        var phase = Math.atan(Math.abs(this.im) / Math.abs(this.re));
        if (this.re < 0 && this.im > 0) {
            phase = Math.PI - phase;
        }
        else if (this.re < 0 && this.im < 0) {
            phase = -(Math.PI - phase);
        }
        else if (this.re > 0 && this.im < 0) {
            phase = -phase;
        }
        else if (this.re === 0 && this.im > 0) {
            phase = Math.PI / 2;
        }
        else if (this.re === 0 && this.im < 0) {
            phase = -Math.PI / 2;
        }
        else if (this.re < 0 && this.im === 0) {
            phase = Math.PI;
        }
        else if (this.re > 0 && this.im === 0) {
            phase = 0;
        }
        else if (this.re === 0 && this.im === 0) {
            // More correctly would be to set 'indeterminate'.
            // But just for simplicity reasons let's set zero.
            phase = 0;
        }
        if (!inRadians) {
            phase = radianToDegree(phase);
        }
        return phase;
    };
    /**
     * @param inRadians
     */
    ComplexNumber.prototype.getPolarForm = function (inRadians) {
        if (inRadians === void 0) { inRadians = true; }
        return {
            radius: this.getRadius(),
            phase: this.getPhase(inRadians),
        };
    };
    /**
     * Convert real numbers to complex number.
     * In case if complex number is provided then lefts it as is.
     *
     * @param number
     */
    ComplexNumber.prototype.toComplexNumber = function (number) {
        if (number instanceof ComplexNumber) {
            return number;
        }
        return new ComplexNumber({ re: number });
    };
    return ComplexNumber;
}());

/**
 * Recursive version of Euclidean Algorithm of finding greatest common divisor (GCD).
 * @param originalA
 * @param originalB
 */
function euclideanAlgorithm(originalA, originalB) {
    // Make input numbers positive.
    var a = Math.abs(originalA);
    var b = Math.abs(originalB);
    // To make algorithm work faster instead of subtracting one number from the other
    // we may use modulo operation.
    return (b === 0) ? a : euclideanAlgorithm(b, a % b);
}

/**
 * Iterative version of Euclidean Algorithm of finding greatest common divisor (GCD).
 * @param originalA
 * @param originalB
 */
function euclideanAlgorithmIterative(originalA, originalB) {
    var _a;
    // Make input numbers positive.
    var a = Math.abs(originalA);
    var b = Math.abs(originalB);
    // Subtract one number from another until both numbers would become the same.
    // This will be out GCD. Also quit the loop if one of the numbers is zero.
    while (a && b && a !== b) {
        _a = __read(a > b ? [a - b, b] : [a, b - a], 2), a = _a[0], b = _a[1];
    }
    // Return the number that is not equal to zero since the last subtraction (it will be a GCD).
    return a || b;
}

/**
 * @typedef {import('../matrix/Matrix.js').Matrix} Matrix
 */
/**
 * Calculates the euclidean distance between 2 matrices.
 *
 * @param a
 * @param b
 *
 * @trows Error
 */
function euclideanDistance(a, b) {
    validateSameShape(a, b);
    var squaresTotal = 0;
    walk(a, function (indices, aCellValue) {
        var bCellValue = getCellAtIndex(b, indices);
        squaresTotal += Math.pow((aCellValue - bCellValue), 2);
    });
    return Number(Math.sqrt(squaresTotal).toFixed(2));
}

/**
 * 阶乘
 * N!
 *
 * @param number 阶乘数
 * @returns 值
 */
function factorial(number) {
    var result = 1;
    for (var i = 2; i <= number; i += 1) {
        result *= i;
    }
    return result;
}

/**
 * 阶乘（递归）
 *
 * @param number 阶乘数
 * @return 值
 */
function factorialRecursive(number) {
    return number > 1 ? number * factorialRecursive(number - 1) : 1;
}

/**
 * Fast Powering Algorithm.
 * Recursive implementation to compute power.
 *
 * Complexity: log(n)
 *
 * @param base Number that will be raised to the power.
 * @param power The power that number will be raised to.
 */
function fastPowering(base, power) {
    if (power === 0) {
        // Anything that is raised to the power of zero is 1.
        return 1;
    }
    if (power % 2 === 0) {
        // If the power is even...
        // we may recursively redefine the result via twice smaller powers:
        // x^8 = x^4 * x^4.
        var multiplier_1 = fastPowering(base, power / 2);
        return multiplier_1 * multiplier_1;
    }
    // If the power is odd...
    // we may recursively redefine the result via twice smaller powers:
    // x^9 = x^4 * x^4 * x.
    var multiplier = fastPowering(base, Math.floor(power / 2));
    return multiplier * multiplier * base;
}

/**
 * Return a fibonacci sequence as an array.
 *
 * @param n
 */
function fibonacci(n) {
    var fibSequence = [1];
    var currentValue = 1;
    var previousValue = 0;
    if (n === 1) {
        return fibSequence;
    }
    var iterationsCounter = n - 1;
    while (iterationsCounter) {
        currentValue += previousValue;
        previousValue = currentValue - previousValue;
        fibSequence.push(currentValue);
        iterationsCounter -= 1;
    }
    return fibSequence;
}

/**
 * Calculate fibonacci number at specific position using Dynamic Programming approach.
 *
 * @param n
 */
function fibonacciNth(n) {
    var currentValue = 1;
    var previousValue = 0;
    if (n === 1) {
        return 1;
    }
    var iterationsCounter = n - 1;
    while (iterationsCounter) {
        currentValue += previousValue;
        previousValue = currentValue - previousValue;
        iterationsCounter -= 1;
    }
    return currentValue;
}

/**
 * Calculate fibonacci number at specific position using closed form function (Binet's formula).
 * @see: https://en.wikipedia.org/wiki/Fibonacci_number#Closed-form_expression
 *
 * @param position Position number of fibonacci sequence (must be number from 1 to 75).
 */
function fibonacciNthClosedForm(position) {
    var topMaxValidPosition = 70;
    // Check that position is valid.
    if (position < 1 || position > topMaxValidPosition) {
        throw new Error("Can't handle position smaller than 1 or greater than " + topMaxValidPosition);
    }
    // Calculate √5 to re-use it in further formulas.
    var sqrt5 = Math.sqrt(5);
    // Calculate φ constant (≈ 1.61803).
    var phi = (1 + sqrt5) / 2;
    // Calculate fibonacci number using Binet's formula.
    return Math.floor((Math.pow(phi, position)) / sqrt5 + 0.5);
}

var CLOSE_TO_ZERO_THRESHOLD$1 = 1e-10;
/**
 * Discrete Fourier Transform (DFT): time to frequencies.
 *
 * Time complexity: O(N^2)
 *
 * @param inputAmplitudes Input signal amplitudes over time (complex
 * numbers with real parts only).
 * @param zeroThreshold Threshold that is used to convert real and imaginary numbers
 * to zero in case if they are smaller then this.
 *
 * @return Array of complex number. Each of the number represents the frequency
 * or signal. All signals together will form input signal over discrete time periods. Each signal's
 * complex number has radius (amplitude) and phase (angle) in polar form that describes the signal.
 *
 * @see https://gist.github.com/anonymous/129d477ddb1c8025c9ac
 * @see https://betterexplained.com/articles/an-interactive-guide-to-the-fourier-transform/
 */
function discreteFourierTransform(inputAmplitudes, zeroThreshold) {
    if (zeroThreshold === void 0) { zeroThreshold = CLOSE_TO_ZERO_THRESHOLD$1; }
    var N = inputAmplitudes.length;
    var signals = [];
    // Go through every discrete frequency.
    for (var frequency = 0; frequency < N; frequency += 1) {
        // Compound signal at current frequency that will ultimately
        // take part in forming input amplitudes.
        var frequencySignal = new ComplexNumber();
        // Go through every discrete point in time.
        for (var timer = 0; timer < N; timer += 1) {
            var currentAmplitude = inputAmplitudes[timer];
            // Calculate rotation angle.
            var rotationAngle = -1 * (2 * Math.PI) * frequency * (timer / N);
            // Remember that e^ix = cos(x) + i * sin(x);
            var dataPointContribution = new ComplexNumber({
                re: Math.cos(rotationAngle),
                im: Math.sin(rotationAngle),
            }).multiply(currentAmplitude);
            // Add this data point's contribution.
            frequencySignal = frequencySignal.add(dataPointContribution);
        }
        // Close to zero? You're zero.
        if (Math.abs(frequencySignal.re) < zeroThreshold) {
            frequencySignal.re = 0;
        }
        if (Math.abs(frequencySignal.im) < zeroThreshold) {
            frequencySignal.im = 0;
        }
        // Average contribution at this frequency.
        // The 1/N factor is usually moved to the reverse transform (going from frequencies
        // back to time). This is allowed, though it would be nice to have 1/N in the forward
        // transform since it gives the actual sizes for the time spikes.
        frequencySignal = frequencySignal.divide(N);
        // Add current frequency signal to the list of compound signals.
        signals[frequency] = frequencySignal;
    }
    return signals;
}

/**
 * Returns the number which is the flipped binary representation of input.
 *
 * @param input
 * @param bitsCount
 */
function reverseBits(input, bitsCount) {
    var reversedBits = 0;
    for (var bitIndex = 0; bitIndex < bitsCount; bitIndex += 1) {
        reversedBits *= 2;
        if (Math.floor(input / (1 << bitIndex)) % 2 === 1) {
            reversedBits += 1;
        }
    }
    return reversedBits;
}
/**
 * Returns the radix-2 fast fourier transform of the given array.
 * Optionally computes the radix-2 inverse fast fourier transform.
 *
 * @param inputData
 * @param inverse
 */
function fastFourierTransform(inputData, inverse) {
    if (inverse === void 0) { inverse = false; }
    var bitsCount = bitLength(inputData.length - 1);
    var N = 1 << bitsCount;
    while (inputData.length < N) {
        inputData.push(new ComplexNumber());
    }
    var output = [];
    for (var dataSampleIndex = 0; dataSampleIndex < N; dataSampleIndex += 1) {
        output[dataSampleIndex] = inputData[reverseBits(dataSampleIndex, bitsCount)];
    }
    for (var blockLength = 2; blockLength <= N; blockLength *= 2) {
        var imaginarySign = inverse ? -1 : 1;
        var phaseStep = new ComplexNumber({
            re: Math.cos((2 * Math.PI) / blockLength),
            im: imaginarySign * Math.sin((2 * Math.PI) / blockLength),
        });
        for (var blockStart = 0; blockStart < N; blockStart += blockLength) {
            var phase = new ComplexNumber({ re: 1, im: 0 });
            for (var signalId = blockStart; signalId < (blockStart + blockLength / 2); signalId += 1) {
                var component = output[signalId + blockLength / 2].multiply(phase);
                var upd1 = output[signalId].add(component);
                var upd2 = output[signalId].subtract(component);
                output[signalId] = upd1;
                output[signalId + blockLength / 2] = upd2;
                phase = phase.multiply(phaseStep);
            }
        }
    }
    if (inverse) {
        for (var signalId = 0; signalId < N; signalId += 1) {
            // output[signalId] /= N;
            output[signalId].divide(N);
        }
    }
    return output;
}

var CLOSE_TO_ZERO_THRESHOLD = 1e-10;
/**
 * Inverse Discrete Fourier Transform (IDFT): frequencies to time.
 *
 * Time complexity: O(N^2)
 *
 * @param frequencies Frequencies summands of the final signal.
 * @param zeroThreshold Threshold that is used to convert real and imaginary numbers
 * to zero in case if they are smaller then this.
 *
 * @return Discrete amplitudes distributed in time.
 */
function inverseDiscreteFourierTransform(frequencies, zeroThreshold) {
    if (zeroThreshold === void 0) { zeroThreshold = CLOSE_TO_ZERO_THRESHOLD; }
    var N = frequencies.length;
    var amplitudes = [];
    // Go through every discrete point of time.
    for (var timer = 0; timer < N; timer += 1) {
        // Compound amplitude at current time.
        var amplitude = new ComplexNumber();
        // Go through all discrete frequencies.
        for (var frequency = 0; frequency < N; frequency += 1) {
            var currentFrequency = frequencies[frequency];
            // Calculate rotation angle.
            var rotationAngle = (2 * Math.PI) * frequency * (timer / N);
            // Remember that e^ix = cos(x) + i * sin(x);
            var frequencyContribution = new ComplexNumber({
                re: Math.cos(rotationAngle),
                im: Math.sin(rotationAngle),
            }).multiply(currentFrequency);
            amplitude = amplitude.add(frequencyContribution);
        }
        // Close to zero? You're zero.
        if (Math.abs(amplitude.re) < zeroThreshold) {
            amplitude.re = 0;
        }
        if (Math.abs(amplitude.im) < zeroThreshold) {
            amplitude.im = 0;
        }
        // Add current frequency signal to the list of compound signals.
        amplitudes[timer] = amplitude.re;
    }
    return amplitudes;
}

/**
 * Returns the evaluation of a polynomial function at a certain point.
 * Uses straightforward approach with powers.
 *
 * @param coefficients i.e. [4, 3, 2] for (4 * x^2 + 3 * x + 2)
 * @param xVal
 */
function classicPolynome(coefficients, xVal) {
    return coefficients.reverse().reduce(function (accumulator, currentCoefficient, index) {
        return accumulator + currentCoefficient * (Math.pow(xVal, index));
    }, 0);
}

/**
 * Returns the evaluation of a polynomial function at a certain point.
 * Uses Horner's rule.
 *
 * @param coefficients i.e. [4, 3, 2] for (4 * x^2 + 3 * x + 2)
 * @param xVal
 */
function hornerMethod(coefficients, xVal) {
    return coefficients.reduce(function (accumulator, currentCoefficient) {
        return accumulator * xVal + currentCoefficient;
    }, 0);
}

/**
 * @param number
 */
function integerPartition(number) {
    // Create partition matrix for solving this task using Dynamic Programming.
    var partitionMatrix = Array(number + 1).fill(null).map(function () {
        return Array(number + 1).fill(null);
    });
    // Fill partition matrix with initial values.
    // Let's fill the first row that represents how many ways we would have
    // to combine the numbers 1, 2, 3, ..., n with number 0. We would have zero
    // ways obviously since with zero number we may form only zero.
    for (var numberIndex = 1; numberIndex <= number; numberIndex += 1) {
        partitionMatrix[0][numberIndex] = 0;
    }
    // Let's fill the first column. It represents the number of ways we can form
    // number zero out of numbers 0, 0 and 1, 0 and 1 and 2, 0 and 1 and 2 and 3, ...
    // Obviously there is only one way we could form number 0
    // and it is with number 0 itself.
    for (var summandIndex = 0; summandIndex <= number; summandIndex += 1) {
        partitionMatrix[summandIndex][0] = 1;
    }
    // Now let's go through other possible options of how we could form number m out of
    // summands 0, 1, ..., m using Dynamic Programming approach.
    for (var summandIndex = 1; summandIndex <= number; summandIndex += 1) {
        for (var numberIndex = 1; numberIndex <= number; numberIndex += 1) {
            if (summandIndex > numberIndex) {
                // If summand number is bigger then current number itself then just it won't add
                // any new ways of forming the number. Thus we may just copy the number from row above.
                partitionMatrix[summandIndex][numberIndex] = partitionMatrix[summandIndex - 1][numberIndex];
            }
            else {
                /*
                 * The number of combinations would equal to number of combinations of forming the same
                 * number but WITHOUT current summand number PLUS number of combinations of forming the
                 * <current number - current summand> number but WITH current summand.
                 *
                 * Example:
                 * Number of ways to form 5 using summands {0, 1, 2} would equal the SUM of:
                 * - number of ways to form 5 using summands {0, 1} (we've excluded summand 2)
                 * - number of ways to form 3 (because 5 - 2 = 3) using summands {0, 1, 2}
                 * (we've included summand 2)
                */
                var combosWithoutSummand = partitionMatrix[summandIndex - 1][numberIndex];
                var combosWithSummand = partitionMatrix[summandIndex][numberIndex - summandIndex];
                partitionMatrix[summandIndex][numberIndex] = combosWithoutSummand + combosWithSummand;
            }
        }
    }
    return partitionMatrix[number][number];
}

/**
 * 是否为2的幂 2^n
 *
 * @param number 被检查的值
 */
function isPowerOfTwo1(number) {
    // 1 (2^0) is the smallest power of two.
    if (number < 1) {
        return false;
    }
    // Let's find out if we can divide the number by two
    // many times without remainder.
    var dividedNumber = number;
    while (dividedNumber !== 1) {
        if (dividedNumber % 2 !== 0) {
            // For every case when remainder isn't zero we can say that this number
            // couldn't be a result of power of two.
            return false;
        }
        dividedNumber /= 2;
    }
    return true;
}

/**
 * 是否为2的幂 2^n
 *
 * @param number 被检查的值
 */
function isPowerOfTwoBitwise(number) {
    // 1 (2^0) is the smallest power of two.
    if (number < 1) {
        return false;
    }
    /*
     * Powers of two in binary look like this:
     * 1: 0001
     * 2: 0010
     * 4: 0100
     * 8: 1000
     *
     * Note that there is always exactly 1 bit set. The only exception is with a signed integer.
     * e.g. An 8-bit signed integer with a value of -128 looks like:
     * 10000000
     *
     * So after checking that the number is greater than zero, we can use a clever little bit
     * hack to test that one and only one bit is set.
     */
    return (number & (number - 1)) === 0;
}

/**
 * @param a
 * @param b
 */
function leastCommonMultiple(a, b) {
    return ((a === 0) || (b === 0)) ? 0 : Math.abs(a * b) / euclideanAlgorithm(a, b);
}

/*
 * Let circleRadius is the radius of circle.
 * circleRadius is also the side length of the inscribed hexagon
 */
var circleRadius = 1;
/**
 * @param sideLength
 * @param splitCounter
 */
function getNGonSideLength(sideLength, splitCounter) {
    if (splitCounter <= 0) {
        return sideLength;
    }
    var halfSide = sideLength / 2;
    // Liu Hui used the Gou Gu (Pythagorean theorem) theorem repetitively.
    var perpendicular = Math.sqrt((Math.pow(circleRadius, 2)) - (Math.pow(halfSide, 2)));
    var excessRadius = circleRadius - perpendicular;
    var splitSideLength = Math.sqrt((Math.pow(excessRadius, 2)) + (Math.pow(halfSide, 2)));
    return getNGonSideLength(splitSideLength, splitCounter - 1);
}
/**
 * @param splitCount
 */
function getNGonSideCount(splitCount) {
    // Liu Hui began with an inscribed hexagon (6-gon).
    var hexagonSidesCount = 6;
    // On every split iteration we make N-gons: 6-gon, 12-gon, 24-gon, 48-gon and so on.
    return hexagonSidesCount * (splitCount ? Math.pow(2, splitCount) : 1);
}
/**
 * Calculate the π value using Liu Hui's π algorithm
 *
 * @param splitCount number of times we're going to split 6-gon.
 *  On each split we will receive 12-gon, 24-gon and so on.
 */
function liuHui(splitCount) {
    if (splitCount === void 0) { splitCount = 1; }
    var nGonSideLength = getNGonSideLength(circleRadius, splitCount - 1);
    var nGonSideCount = getNGonSideCount(splitCount - 1);
    var nGonPerimeter = nGonSideLength * nGonSideCount;
    var approximateCircleArea = (nGonPerimeter / 2) * circleRadius;
    // Return approximate value of pi.
    return approximateCircleArea / (Math.pow(circleRadius, 2));
}

/**
 * @param lineNumber - zero based.
 */
function pascalTriangle(lineNumber) {
    var currentLine = [1];
    var currentLineSize = lineNumber + 1;
    for (var numIndex = 1; numIndex < currentLineSize; numIndex += 1) {
        // See explanation of this formula in README.
        currentLine[numIndex] = (currentLine[numIndex - 1] * (lineNumber - numIndex + 1)) / numIndex;
    }
    return currentLine;
}

/**
 * @param lineNumber zero based.
 */
function pascalTriangleRecursive(lineNumber) {
    if (lineNumber === 0) {
        return [1];
    }
    var currentLineSize = lineNumber + 1;
    var previousLineSize = currentLineSize - 1;
    // Create container for current line values.
    var currentLine = [];
    // We'll calculate current line based on previous one.
    var previousLine = pascalTriangleRecursive(lineNumber - 1);
    // Let's go through all elements of current line except the first and
    // last one (since they were and will be filled with 1's) and calculate
    // current coefficient based on previous line.
    for (var numIndex = 0; numIndex < currentLineSize; numIndex += 1) {
        var leftCoefficient = (numIndex - 1) >= 0 ? previousLine[numIndex - 1] : 0;
        var rightCoefficient = numIndex < previousLineSize ? previousLine[numIndex] : 0;
        currentLine[numIndex] = leftCoefficient + rightCoefficient;
    }
    return currentLine;
}

/**
 * @param number
 */
function trialDivision(number) {
    // Check if number is integer.
    if (number % 1 !== 0) {
        return false;
    }
    if (number <= 1) {
        // If number is less than one then it isn't prime by definition.
        return false;
    }
    if (number <= 3) {
        // All numbers from 2 to 3 are prime.
        return true;
    }
    // If the number is not divided by 2 then we may eliminate all further even dividers.
    if (number % 2 === 0) {
        return false;
    }
    // If there is no dividers up to square root of n then there is no higher dividers as well.
    var dividerLimit = Math.sqrt(number);
    for (var divider = 3; divider <= dividerLimit; divider += 2) {
        if (number % divider === 0) {
            return false;
        }
    }
    return true;
}

/**
 * Finds prime factors of a number.
 *
 * @param n the number that is going to be split into prime factors.
 * @returns array of prime factors.
 */
function primeFactors(n) {
    // Clone n to avoid function arguments override.
    var nn = n;
    // Array that stores the all the prime factors.
    var factors = [];
    // Running the loop till sqrt(n) instead of n to optimise time complexity from O(n) to O(sqrt(n)).
    for (var factor = 2; factor <= Math.sqrt(nn); factor += 1) {
        // Check that factor divides n without a reminder.
        while (nn % factor === 0) {
            // Overriding the value of n.
            nn /= factor;
            // Saving the factor.
            factors.push(factor);
        }
    }
    // The ultimate reminder should be a last prime factor,
    // unless it is not 1 (since 1 is not a prime number).
    if (nn !== 1) {
        factors.push(nn);
    }
    return factors;
}
/**
 * Hardy-Ramanujan approximation of prime factors count.
 *
 * @param n
 * @returns approximate number of prime factors.
 */
function hardyRamanujan(n) {
    return Math.log(Math.log(n));
}

/**
 * @param degree
 */
function degreeToRadian(degree) {
    return degree * (Math.PI / 180);
}

/**
 * @param maxNumber
 */
function sieveOfEratosthenes(maxNumber) {
    var isPrime = new Array(maxNumber + 1).fill(true);
    isPrime[0] = false;
    isPrime[1] = false;
    var primes = [];
    for (var number = 2; number <= maxNumber; number += 1) {
        if (isPrime[number] === true) {
            primes.push(number);
            /*
             * Optimisation.
             * Start marking multiples of `p` from `p * p`, and not from `2 * p`.
             * The reason why this works is because, at that point, smaller multiples
             * of `p` will have already been marked `false`.
             *
             * Warning: When working with really big numbers, the following line may cause overflow
             * In that case, it can be changed to:
             * let nextNumber = 2 * number;
             */
            var nextNumber = number * number;
            while (nextNumber <= maxNumber) {
                isPrime[nextNumber] = false;
                nextNumber += number;
            }
        }
    }
    return primes;
}

/**
 * Calculates the square root of the number with given tolerance (precision)
 * by using Newton's method.
 *
 * @param number the number we want to find a square root for.
 * @param tolerance how many precise numbers after the floating point we want to get.
 */
function squareRoot(number, tolerance) {
    if (tolerance === void 0) { tolerance = 0; }
    // For now we won't support operations that involves manipulation with complex numbers.
    if (number < 0) {
        throw new Error('The method supports only positive integers');
    }
    // Handle edge case with finding the square root of zero.
    if (number === 0) {
        return 0;
    }
    // We will start approximation from value 1.
    var root = 1;
    // Delta is a desired distance between the number and the square of the root.
    // - if tolerance=0 then delta=1
    // - if tolerance=1 then delta=0.1
    // - if tolerance=2 then delta=0.01
    // - and so on...
    var requiredDelta = 1 / (Math.pow(10, tolerance));
    // Approximating the root value to the point when we get a desired precision.
    while (Math.abs(number - (Math.pow(root, 2))) > requiredDelta) {
        // Newton's method reduces in this case to the so-called Babylonian method.
        // These methods generally yield approximate results, but can be made arbitrarily
        // precise by increasing the number of calculation steps.
        root -= ((Math.pow(root, 2)) - number) / (2 * root);
    }
    // Cut off undesired floating digits and return the root value.
    return Math.round(root * (Math.pow(10, tolerance))) / (Math.pow(10, tolerance));
}

/**
 * Classifies the point in space based on k-Means algorithm.
 *
 * @param data array of dataSet points, i.e. [[0, 1], [3, 4], [5, 7]]
 * @param k number of clusters
 * @return the class of the point
 */
function KMeans(data, k) {
    if (k === void 0) { k = 1; }
    if (!data) {
        throw new Error('The data is empty');
    }
    // Assign k clusters locations equal to the location of initial k points.
    var dataDim = data[0].length;
    var clusterCenters = data.slice(0, k);
    // Continue optimization till convergence.
    // Centroids should not be moving once optimized.
    // Calculate distance of each candidate vector from each cluster center.
    // Assign cluster number to each data vector according to minimum distance.
    // Matrix of distance from each data point to each cluster centroid.
    var distances = zeros([data.length, k]);
    // Vector data points' classes. The value of -1 means that no class has bee assigned yet.
    var classes = Array(data.length).fill(-1);
    var iterate = true;
    while (iterate) {
        iterate = false;
        // Calculate and store the distance of each data point from each cluster.
        for (var dataIndex = 0; dataIndex < data.length; dataIndex += 1) {
            for (var clusterIndex = 0; clusterIndex < k; clusterIndex += 1) {
                distances[dataIndex][clusterIndex] = euclideanDistance([clusterCenters[clusterIndex]], [data[dataIndex]]);
            }
            // Assign the closest cluster number to each dataSet point.
            var closestClusterIdx = distances[dataIndex].indexOf(Math.min.apply(Math, __spreadArray([], __read(distances[dataIndex]))));
            // Check if data point class has been changed and we still need to re-iterate.
            if (classes[dataIndex] !== closestClusterIdx) {
                iterate = true;
            }
            classes[dataIndex] = closestClusterIdx;
        }
        // Recalculate cluster centroid values via all dimensions of the points under it.
        for (var clusterIndex = 0; clusterIndex < k; clusterIndex += 1) {
            // Reset cluster center coordinates since we need to recalculate them.
            clusterCenters[clusterIndex] = Array(dataDim).fill(0);
            var clusterSize = 0;
            for (var dataIndex = 0; dataIndex < data.length; dataIndex += 1) {
                if (classes[dataIndex] === clusterIndex) {
                    // Register one more data point of current cluster.
                    clusterSize += 1;
                    for (var dimensionIndex = 0; dimensionIndex < dataDim; dimensionIndex += 1) {
                        // Add data point coordinates to the cluster center coordinates.
                        clusterCenters[clusterIndex][dimensionIndex] += data[dataIndex][dimensionIndex];
                    }
                }
            }
            // Calculate the average for each cluster center coordinate.
            for (var dimensionIndex = 0; dimensionIndex < dataDim; dimensionIndex += 1) {
                clusterCenters[clusterIndex][dimensionIndex] = parseFloat(Number(clusterCenters[clusterIndex][dimensionIndex] / clusterSize).toFixed(2));
            }
        }
    }
    // Return the clusters assigned.
    return classes;
}

/**
 * Classifies the point in space based on k-nearest neighbors algorithm.
 *
 * @param dataSet array of data points, i.e. [[0, 1], [3, 4], [5, 7]]
 * @param labels array of classes (labels), i.e. [1, 1, 2]
 * @param toClassify the point in space that needs to be classified, i.e. [5, 4]
 * @param k number of nearest neighbors which will be taken into account (preferably odd)
 * @return the class of the point
 */
function kNN(dataSet, labels, toClassify, k) {
    if (k === void 0) { k = 3; }
    if (!dataSet || !labels || !toClassify) {
        throw new Error('Either dataSet or labels or toClassify were not set');
    }
    // Calculate distance from toClassify to each point for all dimensions in dataSet.
    // Store distance and point's label into distances list.
    var distances = [];
    for (var i = 0; i < dataSet.length; i += 1) {
        distances.push({
            dist: euclideanDistance([dataSet[i]], [toClassify]),
            label: labels[i],
        });
    }
    // Sort distances list (from closer point to further ones).
    // Take initial k values, count with class index
    var kNearest = distances.sort(function (a, b) {
        if (a.dist === b.dist) {
            return 0;
        }
        return a.dist < b.dist ? -1 : 1;
    }).slice(0, k);
    // Count the number of instances of each class in top k members.
    var labelsCounter = {};
    var topClass = 0;
    var topClassCount = 0;
    for (var i = 0; i < kNearest.length; i += 1) {
        if (kNearest[i].label in labelsCounter) {
            labelsCounter[kNearest[i].label] += 1;
        }
        else {
            labelsCounter[kNearest[i].label] = 1;
        }
        if (labelsCounter[kNearest[i].label] > topClassCount) {
            topClassCount = labelsCounter[kNearest[i].label];
            topClass = kNearest[i].label;
        }
    }
    // Return the class with highest count.
    return topClass;
}

/**
 * Binary search implementation.
 *
 * @param sortedArray
 * @param seekElement
 * @param comparatorCallback
 */
function binarySearch(sortedArray, seekElement, comparatorCallback) {
    // Let's create comparator from the comparatorCallback function.
    // Comparator object will give us common comparison methods like equal() and lessThen().
    var comparator = new Comparator(comparatorCallback);
    // These two indices will contain current array (sub-array) boundaries.
    var startIndex = 0;
    var endIndex = sortedArray.length - 1;
    // Let's continue to split array until boundaries are collapsed
    // and there is nothing to split anymore.
    while (startIndex <= endIndex) {
        // Let's calculate the index of the middle element.
        var middleIndex = startIndex + Math.floor((endIndex - startIndex) / 2);
        // If we've found the element just return its position.
        if (comparator.equal(sortedArray[middleIndex], seekElement)) {
            return middleIndex;
        }
        // Decide which half to choose for seeking next: left or right one.
        if (comparator.lessThan(sortedArray[middleIndex], seekElement)) {
            // Go to the right half of the array.
            startIndex = middleIndex + 1;
        }
        else {
            // Go to the left half of the array.
            endIndex = middleIndex - 1;
        }
    }
    // Return -1 if we have not found anything.
    return -1;
}

/**
 * Interpolation search implementation.
 *
 * @param sortedArray sorted array with uniformly distributed values
 * @param seekElement
 */
function interpolationSearch(sortedArray, seekElement) {
    var leftIndex = 0;
    var rightIndex = sortedArray.length - 1;
    while (leftIndex <= rightIndex) {
        var rangeDelta = sortedArray[rightIndex] - sortedArray[leftIndex];
        var indexDelta = rightIndex - leftIndex;
        var valueDelta = seekElement - sortedArray[leftIndex];
        // If valueDelta is less then zero it means that there is no seek element
        // exists in array since the lowest element from the range is already higher
        // then seek element.
        if (valueDelta < 0) {
            return -1;
        }
        // If range delta is zero then subarray contains all the same numbers
        // and thus there is nothing to search for unless this range is all
        // consists of seek number.
        if (!rangeDelta) {
            // By doing this we're also avoiding division by zero while
            // calculating the middleIndex later.
            return sortedArray[leftIndex] === seekElement ? leftIndex : -1;
        }
        // Do interpolation of the middle index.
        var middleIndex = leftIndex + Math.floor((valueDelta * indexDelta) / rangeDelta);
        // If we've found the element just return its position.
        if (sortedArray[middleIndex] === seekElement) {
            return middleIndex;
        }
        // Decide which half to choose for seeking next: left or right one.
        if (sortedArray[middleIndex] < seekElement) {
            // Go to the right half of the array.
            leftIndex = middleIndex + 1;
        }
        else {
            // Go to the left half of the array.
            rightIndex = middleIndex - 1;
        }
    }
    return -1;
}

/**
 * Jump (block) search implementation.
 *
 * @param sortedArray
 * @param seekElement
 * @param comparatorCallback
 */
function jumpSearch(sortedArray, seekElement, comparatorCallback) {
    var comparator = new Comparator(comparatorCallback);
    var arraySize = sortedArray.length;
    if (!arraySize) {
        // We can't find anything in empty array.
        return -1;
    }
    // Calculate optimal jump size.
    // Total number of comparisons in the worst case will be ((arraySize/jumpSize) + jumpSize - 1).
    // The value of the function ((arraySize/jumpSize) + jumpSize - 1) will be minimum
    // when jumpSize = √array.length.
    var jumpSize = Math.floor(Math.sqrt(arraySize));
    // Find the block where the seekElement belong to.
    var blockStart = 0;
    var blockEnd = jumpSize;
    while (comparator.greaterThan(seekElement, sortedArray[Math.min(blockEnd, arraySize) - 1])) {
        // Jump to the next block.
        blockStart = blockEnd;
        blockEnd += jumpSize;
        // If our next block is out of array then we couldn't found the element.
        if (blockStart > arraySize) {
            return -1;
        }
    }
    // Do linear search for seekElement in subarray starting from blockStart.
    var currentIndex = blockStart;
    while (currentIndex < Math.min(blockEnd, arraySize)) {
        if (comparator.equal(sortedArray[currentIndex], seekElement)) {
            return currentIndex;
        }
        currentIndex += 1;
    }
    return -1;
}

/**
 * Linear search implementation.
 *
 * @param array
 * @param seekElement
 * @param comparatorCallback
 */
function linearSearch(array, seekElement, comparatorCallback) {
    var comparator = new Comparator(comparatorCallback);
    var foundIndices = [];
    array.forEach(function (element, index) {
        if (comparator.equal(element, seekElement)) {
            foundIndices.push(index);
        }
    });
    return foundIndices;
}

/**
 * Generates Cartesian Product of two sets.
 *
 * @param setA
 * @param setB
 */
function cartesianProduct(setA, setB) {
    // Check if input sets are not empty.
    // Otherwise return null since we can't generate Cartesian Product out of them.
    if (!setA || !setB || !setA.length || !setB.length) {
        return null;
    }
    // Init product set.
    var product = [];
    // Now, let's go through all elements of a first and second set and form all possible pairs.
    for (var indexA = 0; indexA < setA.length; indexA += 1) {
        for (var indexB = 0; indexB < setB.length; indexB += 1) {
            // Add current product pair to the product set.
            product.push([setA[indexA], setB[indexB]]);
        }
    }
    // Return cartesian product set.
    return product;
}

/**
 * @param candidates candidate numbers we're picking from.
 * @param remainingSum remaining sum after adding candidates to currentCombination.
 * @param finalCombinations resulting list of combinations.
 * @param currentCombination currently explored candidates.
 * @param startFrom index of the candidate to start further exploration from.
 */
function combinationSumRecursive(candidates, remainingSum, finalCombinations, currentCombination, startFrom) {
    if (finalCombinations === void 0) { finalCombinations = []; }
    if (currentCombination === void 0) { currentCombination = []; }
    if (startFrom === void 0) { startFrom = 0; }
    if (remainingSum < 0) {
        // By adding another candidate we've gone below zero.
        // This would mean that the last candidate was not acceptable.
        return finalCombinations;
    }
    if (remainingSum === 0) {
        // If after adding the previous candidate our remaining sum
        // became zero - we need to save the current combination since it is one
        // of the answers we're looking for.
        finalCombinations.push(currentCombination.slice());
        return finalCombinations;
    }
    // If we haven't reached zero yet let's continue to add all
    // possible candidates that are left.
    for (var candidateIndex = startFrom; candidateIndex < candidates.length; candidateIndex += 1) {
        var currentCandidate = candidates[candidateIndex];
        // Let's try to add another candidate.
        currentCombination.push(currentCandidate);
        // Explore further option with current candidate being added.
        combinationSumRecursive(candidates, remainingSum - currentCandidate, finalCombinations, currentCombination, candidateIndex);
        // BACKTRACKING.
        // Let's get back, exclude current candidate and try another ones later.
        currentCombination.pop();
    }
    return finalCombinations;
}
/**
 * Backtracking algorithm of finding all possible combination for specific sum.
 *
 * @param candidates
 * @param target
 */
function combinationSum(candidates, target) {
    return combinationSumRecursive(candidates, target);
}

/**
 * @param comboOptions
 * @param comboLength
 */
function combineWithoutRepetitions(comboOptions, comboLength) {
    // If the length of the combination is 1 then each element of the original array
    // is a combination itself.
    if (comboLength === 1) {
        return comboOptions.map(function (comboOption) { return [comboOption]; });
    }
    // Init combinations array.
    var combos = [];
    // Extract characters one by one and concatenate them to combinations of smaller lengths.
    // We need to extract them because we don't want to have repetitions after concatenation.
    comboOptions.forEach(function (currentOption, optionIndex) {
        // Generate combinations of smaller size.
        var smallerCombos = combineWithoutRepetitions(comboOptions.slice(optionIndex + 1), comboLength - 1);
        // Concatenate currentOption with all combinations of smaller size.
        smallerCombos.forEach(function (smallerCombo) {
            combos.push([currentOption].concat(smallerCombo));
        });
    });
    return combos;
}

/**
 * @param comboOptions
 * @param comboLength
 */
function combineWithRepetitions(comboOptions, comboLength) {
    // If the length of the combination is 1 then each element of the original array
    // is a combination itself.
    if (comboLength === 1) {
        return comboOptions.map(function (comboOption) { return [comboOption]; });
    }
    // Init combinations array.
    var combos = [];
    // Remember characters one by one and concatenate them to combinations of smaller lengths.
    // We don't extract elements here because the repetitions are allowed.
    comboOptions.forEach(function (currentOption, optionIndex) {
        // Generate combinations of smaller size.
        var smallerCombos = combineWithRepetitions(comboOptions.slice(optionIndex), comboLength - 1);
        // Concatenate currentOption with all combinations of smaller size.
        smallerCombos.forEach(function (smallerCombo) {
            combos.push([currentOption].concat(smallerCombo));
        });
    });
    return combos;
}

/**
 * @param originalArray
 */
function fisherYates(originalArray) {
    var _a;
    // Clone array from preventing original array from modification (for testing purpose).
    var array = originalArray.slice(0);
    for (var i = (array.length - 1); i > 0; i -= 1) {
        var randomIndex = Math.floor(Math.random() * (i + 1));
        _a = __read([array[randomIndex], array[i]], 2), array[i] = _a[0], array[randomIndex] = _a[1];
    }
    return array;
}

var MergeSort = /** @class */ (function (_super) {
    __extends(MergeSort, _super);
    function MergeSort() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    MergeSort.prototype.sort = function (originalArray) {
        // Call visiting callback.
        this.callbacks.visitingCallback(null);
        // If array is empty or consists of one element then return this array since it is sorted.
        if (originalArray.length <= 1) {
            return originalArray;
        }
        // Split array on two halves.
        var middleIndex = Math.floor(originalArray.length / 2);
        var leftArray = originalArray.slice(0, middleIndex);
        var rightArray = originalArray.slice(middleIndex, originalArray.length);
        // Sort two halves of split array
        var leftSortedArray = this.sort(leftArray);
        var rightSortedArray = this.sort(rightArray);
        // Merge two sorted arrays into one.
        return this.mergeSortedArrays(leftSortedArray, rightSortedArray);
    };
    MergeSort.prototype.mergeSortedArrays = function (leftArray, rightArray) {
        var sortedArray = [];
        // Use array pointers to exclude old elements after they have been added to the sorted array.
        var leftIndex = 0;
        var rightIndex = 0;
        while (leftIndex < leftArray.length && rightIndex < rightArray.length) {
            var minElement = null;
            // Find the minimum element between the left and right array.
            if (this.comparator.lessThanOrEqual(leftArray[leftIndex], rightArray[rightIndex])) {
                minElement = leftArray[leftIndex];
                // Increment index pointer to the right
                leftIndex += 1;
            }
            else {
                minElement = rightArray[rightIndex];
                // Increment index pointer to the right
                rightIndex += 1;
            }
            // Add the minimum element to the sorted array.
            sortedArray.push(minElement);
            // Call visiting callback.
            this.callbacks.visitingCallback(minElement);
        }
        // There will be elements remaining from either the left OR the right
        // Concatenate the remaining elements into the sorted array
        return sortedArray
            .concat(leftArray.slice(leftIndex))
            .concat(rightArray.slice(rightIndex));
    };
    return MergeSort;
}(Sort));

var Knapsack = /** @class */ (function () {
    /**
     * @param {KnapsackItem[]} possibleItems
     * @param {number} weightLimit
     */
    function Knapsack(possibleItems, weightLimit) {
        this.selectedItems = [];
        this.weightLimit = weightLimit;
        this.possibleItems = possibleItems;
    }
    Knapsack.prototype.sortPossibleItemsByWeight = function () {
        this.possibleItems = new MergeSort({
            compareCallback: function (itemA, itemB) {
                if (itemA.weight === itemB.weight) {
                    return 0;
                }
                return itemA.weight < itemB.weight ? -1 : 1;
            },
        }).sort(this.possibleItems);
    };
    Knapsack.prototype.sortPossibleItemsByValue = function () {
        this.possibleItems = new MergeSort({
            compareCallback: function (itemA, itemB) {
                if (itemA.value === itemB.value) {
                    return 0;
                }
                return itemA.value > itemB.value ? -1 : 1;
            },
        }).sort(this.possibleItems);
    };
    Knapsack.prototype.sortPossibleItemsByValuePerWeightRatio = function () {
        this.possibleItems = new MergeSort({
            compareCallback: function (itemA, itemB) {
                if (itemA.valuePerWeightRatio === itemB.valuePerWeightRatio) {
                    return 0;
                }
                return itemA.valuePerWeightRatio > itemB.valuePerWeightRatio ? -1 : 1;
            },
        }).sort(this.possibleItems);
    };
    // Solve 0/1 knapsack problem
    // Dynamic Programming approach.
    Knapsack.prototype.solveZeroOneKnapsackProblem = function () {
        // We do two sorts because in case of equal weights but different values
        // we need to take the most valuable items first.
        this.sortPossibleItemsByValue();
        this.sortPossibleItemsByWeight();
        this.selectedItems = [];
        // Create knapsack values matrix.
        var numberOfRows = this.possibleItems.length;
        var numberOfColumns = this.weightLimit;
        var knapsackMatrix = Array(numberOfRows).fill(null).map(function () {
            return Array(numberOfColumns + 1).fill(null);
        });
        // Fill the first column with zeros since it would mean that there is
        // no items we can add to knapsack in case if weight limitation is zero.
        for (var itemIndex_1 = 0; itemIndex_1 < this.possibleItems.length; itemIndex_1 += 1) {
            knapsackMatrix[itemIndex_1][0] = 0;
        }
        // Fill the first row with max possible values we would get by just adding
        // or not adding the first item to the knapsack.
        for (var weightIndex_1 = 1; weightIndex_1 <= this.weightLimit; weightIndex_1 += 1) {
            var itemIndex_2 = 0;
            var itemWeight = this.possibleItems[itemIndex_2].weight;
            var itemValue = this.possibleItems[itemIndex_2].value;
            knapsackMatrix[itemIndex_2][weightIndex_1] = itemWeight <= weightIndex_1 ? itemValue : 0;
        }
        // Go through combinations of how we may add items to knapsack and
        // define what weight/value we would receive using Dynamic Programming
        // approach.
        for (var itemIndex_3 = 1; itemIndex_3 < this.possibleItems.length; itemIndex_3 += 1) {
            for (var weightIndex_2 = 1; weightIndex_2 <= this.weightLimit; weightIndex_2 += 1) {
                var currentItemWeight = this.possibleItems[itemIndex_3].weight;
                var currentItemValue = this.possibleItems[itemIndex_3].value;
                if (currentItemWeight > weightIndex_2) {
                    // In case if item's weight is bigger then currently allowed weight
                    // then we can't add it to knapsack and the max possible value we can
                    // gain at the moment is the max value we got for previous item.
                    knapsackMatrix[itemIndex_3][weightIndex_2] = knapsackMatrix[itemIndex_3 - 1][weightIndex_2];
                }
                else {
                    // Else we need to consider the max value we can gain at this point by adding
                    // current value or just by keeping the previous item for current weight.
                    knapsackMatrix[itemIndex_3][weightIndex_2] = Math.max(currentItemValue + knapsackMatrix[itemIndex_3 - 1][weightIndex_2 - currentItemWeight], knapsackMatrix[itemIndex_3 - 1][weightIndex_2]);
                }
            }
        }
        // Now let's trace back the knapsack matrix to see what items we're going to add
        // to the knapsack.
        var itemIndex = this.possibleItems.length - 1;
        var weightIndex = this.weightLimit;
        while (itemIndex > 0) {
            var currentItem = this.possibleItems[itemIndex];
            var prevItem = this.possibleItems[itemIndex - 1];
            // Check if matrix value came from top (from previous item).
            // In this case this would mean that we need to include previous item
            // to the list of selected items.
            if (knapsackMatrix[itemIndex][weightIndex]
                && knapsackMatrix[itemIndex][weightIndex] === knapsackMatrix[itemIndex - 1][weightIndex]) {
                // Check if there are several items with the same weight but with the different values.
                // We need to add highest item in the matrix that is possible to get the highest value.
                var prevSumValue = knapsackMatrix[itemIndex - 1][weightIndex];
                var prevPrevSumValue = knapsackMatrix[itemIndex - 2][weightIndex];
                if (!prevSumValue
                    || (prevSumValue && prevPrevSumValue !== prevSumValue)) {
                    this.selectedItems.push(prevItem);
                }
            }
            else if (knapsackMatrix[itemIndex - 1][weightIndex - currentItem.weight]) {
                this.selectedItems.push(prevItem);
                weightIndex -= currentItem.weight;
            }
            itemIndex -= 1;
        }
    };
    // Solve unbounded knapsack problem.
    // Greedy approach.
    Knapsack.prototype.solveUnboundedKnapsackProblem = function () {
        this.sortPossibleItemsByValue();
        this.sortPossibleItemsByValuePerWeightRatio();
        for (var itemIndex = 0; itemIndex < this.possibleItems.length; itemIndex += 1) {
            if (this.totalWeight < this.weightLimit) {
                var currentItem = this.possibleItems[itemIndex];
                // Detect how much of current items we can push to knapsack.
                var availableWeight = this.weightLimit - this.totalWeight;
                var maxPossibleItemsCount = Math.floor(availableWeight / currentItem.weight);
                if (maxPossibleItemsCount > currentItem.itemsInStock) {
                    // If we have more items in stock then it is allowed to add
                    // let's add the maximum allowed number of them.
                    currentItem.quantity = currentItem.itemsInStock;
                }
                else if (maxPossibleItemsCount) {
                    // In case if we don't have specified number of items in stock
                    // let's add only items we have in stock.
                    currentItem.quantity = maxPossibleItemsCount;
                }
                this.selectedItems.push(currentItem);
            }
        }
    };
    Object.defineProperty(Knapsack.prototype, "totalValue", {
        get: function () {
            return this.selectedItems.reduce(function (accumulator, item) {
                return accumulator + item.totalValue;
            }, 0);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(Knapsack.prototype, "totalWeight", {
        get: function () {
            return this.selectedItems.reduce(function (accumulator, item) {
                return accumulator + item.totalWeight;
            }, 0);
        },
        enumerable: false,
        configurable: true
    });
    return Knapsack;
}());

var KnapsackItem = /** @class */ (function () {
    /**
     */
    function KnapsackItem(_a) {
        var value = _a.value, weight = _a.weight, _b = _a.itemsInStock, itemsInStock = _b === void 0 ? 1 : _b;
        this.value = value;
        this.weight = weight;
        this.itemsInStock = itemsInStock;
        this.quantity = 1;
    }
    Object.defineProperty(KnapsackItem.prototype, "totalValue", {
        get: function () {
            return this.value * this.quantity;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(KnapsackItem.prototype, "totalWeight", {
        get: function () {
            return this.weight * this.quantity;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(KnapsackItem.prototype, "valuePerWeightRatio", {
        // This coefficient shows how valuable the 1 unit of weight is
        // for current item.
        get: function () {
            return this.value / this.weight;
        },
        enumerable: false,
        configurable: true
    });
    KnapsackItem.prototype.toString = function () {
        return "v" + this.value + " w" + this.weight + " x " + this.quantity;
    };
    return KnapsackItem;
}());

/**
 * @param set1
 * @param set2
 */
function longestCommonSubsequence(set1, set2) {
    // Init LCS matrix.
    var lcsMatrix = Array(set2.length + 1).fill(null).map(function () { return Array(set1.length + 1).fill(null); });
    // Fill first row with zeros.
    for (var columnIndex_1 = 0; columnIndex_1 <= set1.length; columnIndex_1 += 1) {
        lcsMatrix[0][columnIndex_1] = 0;
    }
    // Fill first column with zeros.
    for (var rowIndex_1 = 0; rowIndex_1 <= set2.length; rowIndex_1 += 1) {
        lcsMatrix[rowIndex_1][0] = 0;
    }
    // Fill rest of the column that correspond to each of two strings.
    for (var rowIndex_2 = 1; rowIndex_2 <= set2.length; rowIndex_2 += 1) {
        for (var columnIndex_2 = 1; columnIndex_2 <= set1.length; columnIndex_2 += 1) {
            if (set1[columnIndex_2 - 1] === set2[rowIndex_2 - 1]) {
                lcsMatrix[rowIndex_2][columnIndex_2] = lcsMatrix[rowIndex_2 - 1][columnIndex_2 - 1] + 1;
            }
            else {
                lcsMatrix[rowIndex_2][columnIndex_2] = Math.max(lcsMatrix[rowIndex_2 - 1][columnIndex_2], lcsMatrix[rowIndex_2][columnIndex_2 - 1]);
            }
        }
    }
    // Calculate LCS based on LCS matrix.
    if (!lcsMatrix[set2.length][set1.length]) {
        // If the length of largest common string is zero then return empty string.
        return [''];
    }
    var longestSequence = [];
    var columnIndex = set1.length;
    var rowIndex = set2.length;
    while (columnIndex > 0 || rowIndex > 0) {
        if (set1[columnIndex - 1] === set2[rowIndex - 1]) {
            // Move by diagonal left-top.
            longestSequence.unshift(set1[columnIndex - 1]);
            columnIndex -= 1;
            rowIndex -= 1;
        }
        else if (lcsMatrix[rowIndex][columnIndex] === lcsMatrix[rowIndex][columnIndex - 1]) {
            // Move left.
            columnIndex -= 1;
        }
        else {
            // Move up.
            rowIndex -= 1;
        }
    }
    return longestSequence;
}

/**
 * Dynamic programming approach to find longest increasing subsequence.
 * Complexity: O(n * n)
 *
 * @param sequence
 */
function dpLongestIncreasingSubsequence(sequence) {
    // Create array with longest increasing substrings length and
    // fill it with 1-s that would mean that each element of the sequence
    // is itself a minimum increasing subsequence.
    var lengthsArray = Array(sequence.length).fill(1);
    var previousElementIndex = 0;
    var currentElementIndex = 1;
    while (currentElementIndex < sequence.length) {
        if (sequence[previousElementIndex] < sequence[currentElementIndex]) {
            // If current element is bigger then the previous one then
            // current element is a part of increasing subsequence which
            // length is by one bigger then the length of increasing subsequence
            // for previous element.
            var newLength = lengthsArray[previousElementIndex] + 1;
            if (newLength > lengthsArray[currentElementIndex]) {
                // Increase only if previous element would give us bigger subsequence length
                // then we already have for current element.
                lengthsArray[currentElementIndex] = newLength;
            }
        }
        // Move previous element index right.
        previousElementIndex += 1;
        // If previous element index equals to current element index then
        // shift current element right and reset previous element index to zero.
        if (previousElementIndex === currentElementIndex) {
            currentElementIndex += 1;
            previousElementIndex = 0;
        }
    }
    // Find the biggest element in lengthsArray.
    // This number is the biggest length of increasing subsequence.
    var longestIncreasingLength = 0;
    for (var i = 0; i < lengthsArray.length; i += 1) {
        if (lengthsArray[i] > longestIncreasingLength) {
            longestIncreasingLength = lengthsArray[i];
        }
    }
    return longestIncreasingLength;
}

/**
 * Brute Force solution.
 * Complexity: O(n^2)
 *
 * @param inputArray
 */
function bfMaximumSubarray(inputArray) {
    var maxSubarrayStartIndex = 0;
    var maxSubarrayLength = 0;
    var maxSubarraySum = null;
    for (var startIndex = 0; startIndex < inputArray.length; startIndex += 1) {
        var subarraySum = 0;
        for (var arrLength = 1; arrLength <= (inputArray.length - startIndex); arrLength += 1) {
            subarraySum += inputArray[startIndex + (arrLength - 1)];
            if (maxSubarraySum === null || subarraySum > maxSubarraySum) {
                maxSubarraySum = subarraySum;
                maxSubarrayStartIndex = startIndex;
                maxSubarrayLength = arrLength;
            }
        }
    }
    return inputArray.slice(maxSubarrayStartIndex, maxSubarrayStartIndex + maxSubarrayLength);
}

/**
 * Divide and Conquer solution.
 * Complexity: O(n^2) in case if no memoization applied
 *
 * @param inputArray
 */
function dcMaximumSubarraySum(inputArray) {
    /**
     * We are going through the inputArray array and for each element we have two options:
     * - to pick
     * - not to pick
     *
     * Also keep in mind, that the maximum sub-array must be contiguous. It means if we picked
     * the element, we need to continue picking the next elements or stop counting the max sum.
     *
     * @param elementIndex the index of the element we're deciding to pick or not
     * @param mustPick to pick or not to pick the element
     * @returns maximum subarray sum that we'll get
     */
    function solveRecursively(elementIndex, mustPick) {
        if (elementIndex >= inputArray.length) {
            return mustPick ? 0 : -Infinity;
        }
        return Math.max(
        // Option #1: Pick the current element, and continue picking next one.
        inputArray[elementIndex] + solveRecursively(elementIndex + 1, true), 
        // Option #2: Don't pick the current element.
        mustPick ? 0 : solveRecursively(elementIndex + 1, false));
    }
    return solveRecursively(0, false);
}

/**
 * Dynamic Programming solution.
 * Complexity: O(n)
 *
 * @param inputArray
 */
function dpMaximumSubarray(inputArray) {
    // We iterate through the inputArray once, using a greedy approach to keep track of the maximum
    // sum we've seen so far and the current sum.
    //
    // The currentSum variable gets reset to 0 every time it drops below 0.
    //
    // The maxSum variable is set to -Infinity so that if all numbers are negative, the highest
    // negative number will constitute the maximum subarray.
    var maxSum = -Infinity;
    var currentSum = 0;
    // We need to keep track of the starting and ending indices that contributed to our maxSum
    // so that we can return the actual subarray. From the beginning let's assume that whole array
    // is contributing to maxSum.
    var maxStartIndex = 0;
    var maxEndIndex = inputArray.length - 1;
    var currentStartIndex = 0;
    inputArray.forEach(function (currentNumber, currentIndex) {
        currentSum += currentNumber;
        // Update maxSum and the corresponding indices if we have found a new max.
        if (maxSum < currentSum) {
            maxSum = currentSum;
            maxStartIndex = currentStartIndex;
            maxEndIndex = currentIndex;
        }
        // Reset currentSum and currentStartIndex if currentSum drops below 0.
        if (currentSum < 0) {
            currentSum = 0;
            currentStartIndex = currentIndex + 1;
        }
    });
    return inputArray.slice(maxStartIndex, maxEndIndex + 1);
}

/**
 * @param permutationOptions
 */
function permutateWithoutRepetitions(permutationOptions) {
    if (permutationOptions.length === 1) {
        return [permutationOptions];
    }
    // Init permutations array.
    var permutations = [];
    // Get all permutations for permutationOptions excluding the first element.
    var smallerPermutations = permutateWithoutRepetitions(permutationOptions.slice(1));
    // Insert first option into every possible position of every smaller permutation.
    var firstOption = permutationOptions[0];
    for (var permIndex = 0; permIndex < smallerPermutations.length; permIndex += 1) {
        var smallerPermutation = smallerPermutations[permIndex];
        // Insert first option into every possible position of smallerPermutation.
        for (var positionIndex = 0; positionIndex <= smallerPermutation.length; positionIndex += 1) {
            var permutationPrefix = smallerPermutation.slice(0, positionIndex);
            var permutationSuffix = smallerPermutation.slice(positionIndex);
            permutations.push(permutationPrefix.concat([firstOption], permutationSuffix));
        }
    }
    return permutations;
}

/**
 * @param permutationOptions
 * @param permutationLength
 */
function permutateWithRepetitions(permutationOptions, permutationLength) {
    if (permutationLength === void 0) { permutationLength = permutationOptions.length; }
    if (permutationLength === 1) {
        return permutationOptions.map(function (permutationOption) { return [permutationOption]; });
    }
    // Init permutations array.
    var permutations = [];
    // Get smaller permutations.
    var smallerPermutations = permutateWithRepetitions(permutationOptions, permutationLength - 1);
    // Go through all options and join it to the smaller permutations.
    permutationOptions.forEach(function (currentOption) {
        smallerPermutations.forEach(function (smallerPermutation) {
            permutations.push([currentOption].concat(smallerPermutation));
        });
    });
    return permutations;
}

/**
 * @param originalSet Original set of elements we're forming power-set of.
 * @param allSubsets All subsets that have been formed so far.
 * @param currentSubSet Current subset that we're forming at the moment.
 * @param startAt The position of in original set we're starting to form current subset.
 * @return All subsets of original set.
 */
function btPowerSetRecursive(originalSet, allSubsets, currentSubSet, startAt) {
    if (allSubsets === void 0) { allSubsets = [[]]; }
    if (currentSubSet === void 0) { currentSubSet = []; }
    if (startAt === void 0) { startAt = 0; }
    // Let's iterate over originalSet elements that may be added to the subset
    // without having duplicates. The value of startAt prevents adding the duplicates.
    for (var position = startAt; position < originalSet.length; position += 1) {
        // Let's push current element to the subset
        currentSubSet.push(originalSet[position]);
        // Current subset is already valid so let's memorize it.
        // We do array destruction here to save the clone of the currentSubSet.
        // We need to save a clone since the original currentSubSet is going to be
        // mutated in further recursive calls.
        allSubsets.push(__spreadArray([], __read(currentSubSet)));
        // Let's try to generate all other subsets for the current subset.
        // We're increasing the position by one to avoid duplicates in subset.
        btPowerSetRecursive(originalSet, allSubsets, currentSubSet, position + 1);
        // BACKTRACK. Exclude last element from the subset and try the next valid one.
        currentSubSet.pop();
    }
    // Return all subsets of a set.
    return allSubsets;
}
/**
 * Find power-set of a set using BACKTRACKING approach.
 *
 * @param originalSet
 */
function btPowerSet(originalSet) {
    return btPowerSetRecursive(originalSet);
}

/**
 * Find power-set of a set using BITWISE approach.
 *
 * @param originalSet
 */
function bwPowerSet(originalSet) {
    var subSets = [];
    // We will have 2^n possible combinations (where n is a length of original set).
    // It is because for every element of original set we will decide whether to include
    // it or not (2 options for each set element).
    var numberOfCombinations = Math.pow(2, originalSet.length);
    // Each number in binary representation in a range from 0 to 2^n does exactly what we need:
    // it shows by its bits (0 or 1) whether to include related element from the set or not.
    // For example, for the set {1, 2, 3} the binary number of 0b010 would mean that we need to
    // include only "2" to the current set.
    for (var combinationIndex = 0; combinationIndex < numberOfCombinations; combinationIndex += 1) {
        var subSet = [];
        for (var setElementIndex = 0; setElementIndex < originalSet.length; setElementIndex += 1) {
            // Decide whether we need to include current element into the subset or not.
            if (combinationIndex & (1 << setElementIndex)) {
                subSet.push(originalSet[setElementIndex]);
            }
        }
        // Add current subset to the list of all subsets.
        subSets.push(subSet);
    }
    return subSets;
}

/**
 * @param set1
 * @param set2
 */
function shortestCommonSupersequence(set1, set2) {
    // Let's first find the longest common subsequence of two sets.
    var lcs = longestCommonSubsequence(set1, set2);
    // If LCS is empty then the shortest common supersequence would be just
    // concatenation of two sequences.
    if (lcs.length === 1 && lcs[0] === '') {
        return set1.concat(set2);
    }
    // Now let's add elements of set1 and set2 in order before/inside/after the LCS.
    var supersequence = [];
    var setIndex1 = 0;
    var setIndex2 = 0;
    var lcsIndex = 0;
    var setOnHold1 = false;
    var setOnHold2 = false;
    while (lcsIndex < lcs.length) {
        // Add elements of the first set to supersequence in correct order.
        if (setIndex1 < set1.length) {
            if (!setOnHold1 && set1[setIndex1] !== lcs[lcsIndex]) {
                supersequence.push(set1[setIndex1]);
                setIndex1 += 1;
            }
            else {
                setOnHold1 = true;
            }
        }
        // Add elements of the second set to supersequence in correct order.
        if (setIndex2 < set2.length) {
            if (!setOnHold2 && set2[setIndex2] !== lcs[lcsIndex]) {
                supersequence.push(set2[setIndex2]);
                setIndex2 += 1;
            }
            else {
                setOnHold2 = true;
            }
        }
        // Add LCS element to the supersequence in correct order.
        if (setOnHold1 && setOnHold2) {
            supersequence.push(lcs[lcsIndex]);
            lcsIndex += 1;
            setIndex1 += 1;
            setIndex2 += 1;
            setOnHold1 = false;
            setOnHold2 = false;
        }
    }
    // Attach set1 leftovers.
    if (setIndex1 < set1.length) {
        supersequence = supersequence.concat(set1.slice(setIndex1));
    }
    // Attach set2 leftovers.
    if (setIndex2 < set2.length) {
        supersequence = supersequence.concat(set2.slice(setIndex2));
    }
    return supersequence;
}

var BubbleSort = /** @class */ (function (_super) {
    __extends(BubbleSort, _super);
    function BubbleSort() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    BubbleSort.prototype.sort = function (originalArray) {
        var _a;
        // Flag that holds info about whether the swap has occur or not.
        var swapped = false;
        // Clone original array to prevent its modification.
        var array = __spreadArray([], __read(originalArray));
        for (var i = 1; i < array.length; i += 1) {
            swapped = false;
            // Call visiting callback.
            this.callbacks.visitingCallback(array[i]);
            for (var j = 0; j < array.length - i; j += 1) {
                // Call visiting callback.
                this.callbacks.visitingCallback(array[j]);
                // Swap elements if they are in wrong order.
                if (this.comparator.lessThan(array[j + 1], array[j])) {
                    _a = __read([array[j + 1], array[j]], 2), array[j] = _a[0], array[j + 1] = _a[1];
                    // Register the swap.
                    swapped = true;
                }
            }
            // If there were no swaps then array is already sorted and there is
            // no need to proceed.
            if (!swapped) {
                return array;
            }
        }
        return array;
    };
    return BubbleSort;
}(Sort));

/**
 * 计数排序（Counting sort）
 */
var CountingSort = /** @class */ (function (_super) {
    __extends(CountingSort, _super);
    function CountingSort() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    /**
     * @param originalArray
     * @param smallestElement
     * @param biggestElement
     */
    CountingSort.prototype.sort = function (originalArray, smallestElement, biggestElement) {
        var _this = this;
        if (smallestElement === void 0) { smallestElement = undefined; }
        if (biggestElement === void 0) { biggestElement = undefined; }
        // Init biggest and smallest elements in array in order to build number bucket array later.
        var detectedSmallestElement = smallestElement || 0;
        var detectedBiggestElement = biggestElement || 0;
        if (smallestElement === undefined || biggestElement === undefined) {
            originalArray.forEach(function (element) {
                // Visit element.
                _this.callbacks.visitingCallback(element);
                // Detect biggest element.
                if (_this.comparator.greaterThan(element, detectedBiggestElement)) {
                    detectedBiggestElement = element;
                }
                // Detect smallest element.
                if (_this.comparator.lessThan(element, detectedSmallestElement)) {
                    detectedSmallestElement = element;
                }
            });
        }
        // Init buckets array.
        // This array will hold frequency of each number from originalArray.
        var buckets = Array(detectedBiggestElement - detectedSmallestElement + 1).fill(0);
        originalArray.forEach(function (element) {
            // Visit element.
            _this.callbacks.visitingCallback(element);
            buckets[element - detectedSmallestElement] += 1;
        });
        // Add previous frequencies to the current one for each number in bucket
        // to detect how many numbers less then current one should be standing to
        // the left of current one.
        for (var bucketIndex = 1; bucketIndex < buckets.length; bucketIndex += 1) {
            buckets[bucketIndex] += buckets[bucketIndex - 1];
        }
        // Now let's shift frequencies to the right so that they show correct numbers.
        // I.e. if we won't shift right than the value of buckets[5] will display how many
        // elements less than 5 should be placed to the left of 5 in sorted array
        // INCLUDING 5th. After shifting though this number will not include 5th anymore.
        buckets.pop();
        buckets.unshift(0);
        // Now let's assemble sorted array.
        var sortedArray = Array(originalArray.length).fill(null);
        for (var elementIndex = 0; elementIndex < originalArray.length; elementIndex += 1) {
            // Get the element that we want to put into correct sorted position.
            var element = originalArray[elementIndex];
            // Visit element.
            this.callbacks.visitingCallback(element);
            // Get correct position of this element in sorted array.
            var elementSortedPosition = buckets[element - detectedSmallestElement];
            // Put element into correct position in sorted array.
            sortedArray[elementSortedPosition] = element;
            // Increase position of current element in the bucket for future correct placements.
            buckets[element - detectedSmallestElement] += 1;
        }
        // Return sorted array.
        return sortedArray;
    };
    return CountingSort;
}(Sort));

var InsertionSort = /** @class */ (function (_super) {
    __extends(InsertionSort, _super);
    function InsertionSort() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    InsertionSort.prototype.sort = function (originalArray) {
        var _a;
        var array = __spreadArray([], __read(originalArray));
        // Go through all array elements...
        for (var i = 1; i < array.length; i += 1) {
            var currentIndex = i;
            // Call visiting callback.
            this.callbacks.visitingCallback(array[i]);
            // Check if previous element is greater than current element.
            // If so, swap the two elements.
            while (array[currentIndex - 1] !== undefined
                && this.comparator.lessThan(array[currentIndex], array[currentIndex - 1])) {
                // Call visiting callback.
                this.callbacks.visitingCallback(array[currentIndex - 1]);
                // Swap the elements.
                _a = __read([
                    array[currentIndex],
                    array[currentIndex - 1] ], 2), array[currentIndex - 1] = _a[0], array[currentIndex] = _a[1];
                // Shift current index left.
                currentIndex -= 1;
            }
        }
        return array;
    };
    return InsertionSort;
}(Sort));

var QuickSortInPlace = /** @class */ (function (_super) {
    __extends(QuickSortInPlace, _super);
    function QuickSortInPlace() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    /** Sorting in place avoids unnecessary use of additional memory, but modifies input array.
     *
     * This process is difficult to describe, but much clearer with a visualization:
     * @see: http://www.algomation.com/algorithm/quick-sort-visualization
     *
     * @param originalArray Not sorted array.
     * @param inputLowIndex
     * @param inputHighIndex
     * @param recursiveCall
     * @return Sorted array.
     */
    QuickSortInPlace.prototype.sort = function (originalArray, inputLowIndex, inputHighIndex, recursiveCall) {
        var _this = this;
        if (inputLowIndex === void 0) { inputLowIndex = 0; }
        if (inputHighIndex === void 0) { inputHighIndex = originalArray.length - 1; }
        if (recursiveCall === void 0) { recursiveCall = false; }
        // Copies array on initial call, and then sorts in place.
        var array = recursiveCall ? originalArray : __spreadArray([], __read(originalArray));
        /**
         * The partitionArray() operates on the subarray between lowIndex and highIndex, inclusive.
         * It arbitrarily chooses the last element in the subarray as the pivot.
         * Then, it partially sorts the subarray into elements than are less than the pivot,
         * and elements that are greater than or equal to the pivot.
         * Each time partitionArray() is executed, the pivot element is in its final sorted position.
         *
         * @param lowIndex
         * @param highIndex
         */
        var partitionArray = function (lowIndex, highIndex) {
            /**
             * Swaps two elements in array.
             * @param leftIndex
             * @param rightIndex
             */
            var swap = function (leftIndex, rightIndex) {
                var temp = array[leftIndex];
                array[leftIndex] = array[rightIndex];
                array[rightIndex] = temp;
            };
            var pivot = array[highIndex];
            // visitingCallback is used for time-complexity analysis.
            _this.callbacks.visitingCallback(pivot);
            var partitionIndex = lowIndex;
            for (var currentIndex = lowIndex; currentIndex < highIndex; currentIndex += 1) {
                if (_this.comparator.lessThan(array[currentIndex], pivot)) {
                    swap(partitionIndex, currentIndex);
                    partitionIndex += 1;
                }
            }
            // The element at the partitionIndex is guaranteed to be greater than or equal to pivot.
            // All elements to the left of partitionIndex are guaranteed to be less than pivot.
            // Swapping the pivot with the partitionIndex therefore places the pivot in its
            // final sorted position.
            swap(partitionIndex, highIndex);
            return partitionIndex;
        };
        // Base case is when low and high converge.
        if (inputLowIndex < inputHighIndex) {
            var partitionIndex = partitionArray(inputLowIndex, inputHighIndex);
            var RECURSIVE_CALL = true;
            this.sort(array, inputLowIndex, partitionIndex - 1, RECURSIVE_CALL);
            this.sort(array, partitionIndex + 1, inputHighIndex, RECURSIVE_CALL);
        }
        return array;
    };
    return QuickSortInPlace;
}(Sort));

// Using charCode (a = 97, b = 98, etc), we can map characters to buckets from 0 - 25
var BASE_CHAR_CODE = 97;
var NUMBER_OF_POSSIBLE_DIGITS = 10;
var ENGLISH_ALPHABET_LENGTH = 26;
/**
 * 基数排序（Radix sort）
 */
var RadixSort = /** @class */ (function (_super) {
    __extends(RadixSort, _super);
    function RadixSort() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    /**
     * @param originalArray
     */
    RadixSort.prototype.sort = function (originalArray) {
        // Assumes all elements of array are of the same type
        var isArrayOfNumbers = this.isArrayOfNumbers(originalArray);
        var sortedArray = __spreadArray([], __read(originalArray));
        var numPasses = this.determineNumPasses(sortedArray);
        for (var currentIndex = 0; currentIndex < numPasses; currentIndex += 1) {
            var buckets = isArrayOfNumbers
                ? this.placeElementsInNumberBuckets(sortedArray, currentIndex)
                : this.placeElementsInCharacterBuckets(sortedArray, currentIndex, numPasses);
            // Flatten buckets into sortedArray, and repeat at next index
            sortedArray = buckets.reduce(function (acc, val) {
                return __spreadArray(__spreadArray([], __read(acc)), __read(val));
            }, []);
        }
        return sortedArray;
    };
    /**
     * @param array
     * @param index
     */
    RadixSort.prototype.placeElementsInNumberBuckets = function (array, index) {
        var _this = this;
        // See below. These are used to determine which digit to use for bucket allocation
        var modded = Math.pow(10, (index + 1));
        var divided = Math.pow(10, index);
        var buckets = this.createBuckets(NUMBER_OF_POSSIBLE_DIGITS);
        array.forEach(function (element) {
            _this.callbacks.visitingCallback(element);
            if (element < divided) {
                buckets[0].push(element);
            }
            else {
                /**
                 * Say we have element of 1,052 and are currently on index 1 (starting from 0). This means
                 * we want to use '5' as the bucket. `modded` would be 10 ** (1 + 1), which
                 * is 100. So we take 1,052 % 100 (52) and divide it by 10 (5.2) and floor it (5).
                 */
                var currentDigit = Math.floor((element % modded) / divided);
                buckets[currentDigit].push(element);
            }
        });
        return buckets;
    };
    /**
     * @param array
     * @param index
     * @param numPasses
     */
    RadixSort.prototype.placeElementsInCharacterBuckets = function (array, index, numPasses) {
        var _this = this;
        var buckets = this.createBuckets(ENGLISH_ALPHABET_LENGTH);
        array.forEach(function (element) {
            _this.callbacks.visitingCallback(element);
            var currentBucket = _this.getCharCodeOfElementAtIndex(element, index, numPasses);
            buckets[currentBucket].push(element);
        });
        return buckets;
    };
    /**
     * @param element
     * @param index
     * @param numPasses
     */
    RadixSort.prototype.getCharCodeOfElementAtIndex = function (element, index, numPasses) {
        // Place element in last bucket if not ready to organize
        if ((numPasses - index) > element.length) {
            return ENGLISH_ALPHABET_LENGTH - 1;
        }
        /**
         * If each character has been organized, use first character to determine bucket,
         * otherwise iterate backwards through element
         */
        var charPos = index > element.length - 1 ? 0 : element.length - index - 1;
        return element.toLowerCase().charCodeAt(charPos) - BASE_CHAR_CODE;
    };
    /**
     * Number of passes is determined by the length of the longest element in the array.
     * For integers, this log10(num), and for strings, this would be the length of the string.
     */
    RadixSort.prototype.determineNumPasses = function (array) {
        return this.getLengthOfLongestElement(array);
    };
    /**
     * @param array
     */
    RadixSort.prototype.getLengthOfLongestElement = function (array) {
        if (this.isArrayOfNumbers(array)) {
            return Math.floor(Math.log10(Math.max.apply(Math, __spreadArray([], __read(array))))) + 1;
        }
        return array.reduce(function (acc, val) { return (val.length > acc ? val.length : acc); }, -Infinity);
    };
    /**
     * @param array
     */
    RadixSort.prototype.isArrayOfNumbers = function (array) {
        // Assumes all elements of array are of the same type
        return this.isNumber(array[0]);
    };
    /**
     * @param numBuckets
     */
    RadixSort.prototype.createBuckets = function (numBuckets) {
        /**
         * Mapping buckets to an array instead of filling them with
         * an array prevents each bucket from containing a reference to the same array
         */
        return new Array(numBuckets).fill(null).map(function () { return []; });
    };
    /**
     * @param element
     */
    RadixSort.prototype.isNumber = function (element) {
        return Number.isInteger(element);
    };
    return RadixSort;
}(Sort));

var SelectionSort = /** @class */ (function (_super) {
    __extends(SelectionSort, _super);
    function SelectionSort() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    SelectionSort.prototype.sort = function (originalArray) {
        var _a;
        // Clone original array to prevent its modification.
        var array = __spreadArray([], __read(originalArray));
        for (var i = 0; i < array.length - 1; i += 1) {
            var minIndex = i;
            // Call visiting callback.
            this.callbacks.visitingCallback(array[i]);
            // Find minimum element in the rest of array.
            for (var j = i + 1; j < array.length; j += 1) {
                // Call visiting callback.
                this.callbacks.visitingCallback(array[j]);
                if (this.comparator.lessThan(array[j], array[minIndex])) {
                    minIndex = j;
                }
            }
            // If new minimum element has been found then swap it with current i-th element.
            if (minIndex !== i) {
                _a = __read([array[minIndex], array[i]], 2), array[i] = _a[0], array[minIndex] = _a[1];
            }
        }
        return array;
    };
    return SelectionSort;
}(Sort));

var ShellSort = /** @class */ (function (_super) {
    __extends(ShellSort, _super);
    function ShellSort() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    ShellSort.prototype.sort = function (originalArray) {
        // Prevent original array from mutations.
        var array = __spreadArray([], __read(originalArray));
        // Define a gap distance.
        var gap = Math.floor(array.length / 2);
        // Until gap is bigger then zero do elements comparisons and swaps.
        while (gap > 0) {
            // Go and compare all distant element pairs.
            for (var i = 0; i < (array.length - gap); i += 1) {
                var currentIndex = i;
                var gapShiftedIndex = i + gap;
                while (currentIndex >= 0) {
                    // Call visiting callback.
                    this.callbacks.visitingCallback(array[currentIndex]);
                    // Compare and swap array elements if needed.
                    if (this.comparator.lessThan(array[gapShiftedIndex], array[currentIndex])) {
                        var tmp = array[currentIndex];
                        array[currentIndex] = array[gapShiftedIndex];
                        array[gapShiftedIndex] = tmp;
                    }
                    gapShiftedIndex = currentIndex;
                    currentIndex -= gap;
                }
            }
            // Shrink the gap.
            gap = Math.floor(gap / 2);
        }
        // Return sorted copy of an original array.
        return array;
    };
    return ShellSort;
}(Sort));

/**
 * Picks the random item based on its weight.
 * The items with higher weight will be picked more often (with a higher probability).
 *
 * For example:
 * - items = ['banana', 'orange', 'apple']
 * - weights = [0, 0.2, 0.8]
 * - weightedRandom(items, weights) in 80% of cases will return 'apple', in 20% of cases will return
 * 'orange' and it will never return 'banana' (because probability of picking the banana is 0%)
 *
 * @param items
 * @param weights
 */
/* eslint-disable consistent-return */
function weightedRandom(items, weights) {
    if (items.length !== weights.length) {
        throw new Error('Items and weights must be of the same size');
    }
    if (!items.length) {
        throw new Error('Items must not be empty');
    }
    // Preparing the cumulative weights array.
    // For example:
    // - weights = [1, 4, 3]
    // - cumulativeWeights = [1, 5, 8]
    var cumulativeWeights = [];
    for (var i = 0; i < weights.length; i += 1) {
        cumulativeWeights[i] = weights[i] + (cumulativeWeights[i - 1] || 0);
    }
    // Getting the random number in a range of [0...sum(weights)]
    // For example:
    // - weights = [1, 4, 3]
    // - maxCumulativeWeight = 8
    // - range for the random number is [0...8]
    var maxCumulativeWeight = cumulativeWeights[cumulativeWeights.length - 1];
    var randomNumber = maxCumulativeWeight * Math.random();
    // Picking the random item based on its weight.
    // The items with higher weight will be picked more often.
    for (var itemIndex = 0; itemIndex < items.length; itemIndex += 1) {
        if (cumulativeWeights[itemIndex] >= randomNumber) {
            return {
                item: items[itemIndex],
                index: itemIndex,
            };
        }
    }
}

/**
 * @param a
 * @param b
 */
function hammingDistance(a, b) {
    if (a.length !== b.length) {
        throw new Error('Strings must be of the same length');
    }
    var distance = 0;
    for (var i = 0; i < a.length; i += 1) {
        if (a[i] !== b[i]) {
            distance += 1;
        }
    }
    return distance;
}

/**
 * @see https://www.youtube.com/watch?v=GTJr8OvyEVQ
 * @param word
 */
function buildPatternTable(word) {
    var patternTable = [0];
    var prefixIndex = 0;
    var suffixIndex = 1;
    while (suffixIndex < word.length) {
        if (word[prefixIndex] === word[suffixIndex]) {
            patternTable[suffixIndex] = prefixIndex + 1;
            suffixIndex += 1;
            prefixIndex += 1;
        }
        else if (prefixIndex === 0) {
            patternTable[suffixIndex] = 0;
            suffixIndex += 1;
        }
        else {
            prefixIndex = patternTable[prefixIndex - 1];
        }
    }
    return patternTable;
}
/**
 * @param text
 * @param word
 */
function knuthMorrisPratt(text, word) {
    if (word.length === 0) {
        return 0;
    }
    var textIndex = 0;
    var wordIndex = 0;
    var patternTable = buildPatternTable(word);
    while (textIndex < text.length) {
        if (text[textIndex] === word[wordIndex]) {
            // We've found a match.
            if (wordIndex === word.length - 1) {
                return (textIndex - word.length) + 1;
            }
            wordIndex += 1;
            textIndex += 1;
        }
        else if (wordIndex > 0) {
            wordIndex = patternTable[wordIndex - 1];
        }
        else {
            // wordIndex = 0;
            textIndex += 1;
        }
    }
    return -1;
}

/**
 * 最长公共子串
 *
 * @param string1
 * @param string2
 */
function longestCommonSubstring(string1, string2) {
    // Convert strings to arrays to treat unicode symbols length correctly.
    // For example:
    // '𐌵'.length === 2
    // [...'𐌵'].length === 1
    var s1 = __spreadArray([], __read(string1));
    var s2 = __spreadArray([], __read(string2));
    // Init the matrix of all substring lengths to use Dynamic Programming approach.
    var substringMatrix = Array(s2.length + 1).fill(null).map(function () {
        return Array(s1.length + 1).fill(null);
    });
    // Fill the first row and first column with zeros to provide initial values.
    for (var columnIndex = 0; columnIndex <= s1.length; columnIndex += 1) {
        substringMatrix[0][columnIndex] = 0;
    }
    for (var rowIndex = 0; rowIndex <= s2.length; rowIndex += 1) {
        substringMatrix[rowIndex][0] = 0;
    }
    // Build the matrix of all substring lengths to use Dynamic Programming approach.
    var longestSubstringLength = 0;
    var longestSubstringColumn = 0;
    var longestSubstringRow = 0;
    for (var rowIndex = 1; rowIndex <= s2.length; rowIndex += 1) {
        for (var columnIndex = 1; columnIndex <= s1.length; columnIndex += 1) {
            if (s1[columnIndex - 1] === s2[rowIndex - 1]) {
                substringMatrix[rowIndex][columnIndex] = substringMatrix[rowIndex - 1][columnIndex - 1] + 1;
            }
            else {
                substringMatrix[rowIndex][columnIndex] = 0;
            }
            // Try to find the biggest length of all common substring lengths
            // and to memorize its last character position (indices)
            if (substringMatrix[rowIndex][columnIndex] > longestSubstringLength) {
                longestSubstringLength = substringMatrix[rowIndex][columnIndex];
                longestSubstringColumn = columnIndex;
                longestSubstringRow = rowIndex;
            }
        }
    }
    if (longestSubstringLength === 0) {
        // Longest common substring has not been found.
        return '';
    }
    // Detect the longest substring from the matrix.
    var longestSubstring = '';
    while (substringMatrix[longestSubstringRow][longestSubstringColumn] > 0) {
        longestSubstring = s1[longestSubstringColumn - 1] + longestSubstring;
        longestSubstringRow -= 1;
        longestSubstringColumn -= 1;
    }
    return longestSubstring;
}

/**
 * 是否回文
 *
 * @param string
 */
function isPalindrome(string) {
    var left = 0;
    var right = string.length - 1;
    while (left < right) {
        if (string[left] !== string[right]) {
            return false;
        }
        left += 1;
        right -= 1;
    }
    return true;
}

/**
 * 卡普-拉宾算法（Karp–Rabin algorithm）
 *
 * @param text - Text that may contain the searchable word.
 * @param word - Word that is being searched in text.
 * @return Position of the word in text.
 */
function rabinKarp(text, word) {
    var hasher = new PolynomialHash();
    // Calculate word hash that we will use for comparison with other substring hashes.
    var wordHash = hasher.hash(word);
    var prevFrame = null;
    var currentFrameHash = null;
    // Go through all substring of the text that may match.
    for (var charIndex = 0; charIndex <= (text.length - word.length); charIndex += 1) {
        var currentFrame = text.substring(charIndex, charIndex + word.length);
        // Calculate the hash of current substring.
        if (currentFrameHash === null) {
            currentFrameHash = hasher.hash(currentFrame);
        }
        else {
            currentFrameHash = hasher.roll(currentFrameHash, prevFrame, currentFrame);
        }
        prevFrame = currentFrame;
        // Compare the hash of current substring and seeking string.
        // In case if hashes match let's make sure that substrings are equal.
        // In case of hash collision the strings may not be equal.
        if (wordHash === currentFrameHash
            && text.substr(charIndex, word.length) === word) {
            return charIndex;
        }
    }
    return -1;
}

var ZERO_OR_MORE_CHARS = '*';
var ANY_CHAR = '.';
/**
 * Dynamic programming approach.
 *
 * @param string
 * @param pattern
 */
function regularExpressionMatching(string, pattern) {
    /*
      * Let's initiate dynamic programming matrix for this string and pattern.
      * We will have pattern characters on top (as columns) and string characters
      * will be placed to the left of the table (as rows).
      *
      * Example:
      *
      *     a * b . b
      *   - - - - - -
      * a - - - - - -
      * a - - - - - -
      * b - - - - - -
      * y - - - - - -
      * b - - - - - -
     */
    var matchMatrix = Array(string.length + 1).fill(null).map(function () {
        return Array(pattern.length + 1).fill(null);
    });
    // Let's fill the top-left cell with true. This would mean that empty
    // string '' matches to empty pattern ''.
    matchMatrix[0][0] = true;
    // Let's fill the first row of the matrix with false. That would mean that
    // empty string can't match any non-empty pattern.
    //
    // Example:
    // string: ''
    // pattern: 'a.z'
    //
    // The one exception here is patterns like a*b* that matches the empty string.
    for (var columnIndex = 1; columnIndex <= pattern.length; columnIndex += 1) {
        var patternIndex = columnIndex - 1;
        if (pattern[patternIndex] === ZERO_OR_MORE_CHARS) {
            matchMatrix[0][columnIndex] = matchMatrix[0][columnIndex - 2];
        }
        else {
            matchMatrix[0][columnIndex] = false;
        }
    }
    // Let's fill the first column with false. That would mean that empty pattern
    // can't match any non-empty string.
    //
    // Example:
    // string: 'ab'
    // pattern: ''
    for (var rowIndex = 1; rowIndex <= string.length; rowIndex += 1) {
        matchMatrix[rowIndex][0] = false;
    }
    // Not let's go through every letter of the pattern and every letter of
    // the string and compare them one by one.
    for (var rowIndex = 1; rowIndex <= string.length; rowIndex += 1) {
        for (var columnIndex = 1; columnIndex <= pattern.length; columnIndex += 1) {
            // Take into account that fact that matrix contain one extra column and row.
            var stringIndex = rowIndex - 1;
            var patternIndex = columnIndex - 1;
            if (pattern[patternIndex] === ZERO_OR_MORE_CHARS) {
                /*
                 * In case if current pattern character is special '*' character we have
                 * two options:
                 *
                 * 1. Since * char allows it previous char to not be presented in a string we
                 * need to check if string matches the pattern without '*' char and without the
                 * char that goes before '*'. That would mean to go two positions left on the
                 * same row.
                 *
                 * 2. Since * char allows it previous char to be presented in a string many times we
                 * need to check if char before * is the same as current string char. If they are the
                 * same that would mean that current string matches the current pattern in case if
                 * the string WITHOUT current char matches the same pattern. This would mean to go
                 * one position up in the same row.
                 */
                if (matchMatrix[rowIndex][columnIndex - 2] === true) {
                    matchMatrix[rowIndex][columnIndex] = true;
                }
                else if ((pattern[patternIndex - 1] === string[stringIndex]
                    || pattern[patternIndex - 1] === ANY_CHAR)
                    && matchMatrix[rowIndex - 1][columnIndex] === true) {
                    matchMatrix[rowIndex][columnIndex] = true;
                }
                else {
                    matchMatrix[rowIndex][columnIndex] = false;
                }
            }
            else if (pattern[patternIndex] === string[stringIndex]
                || pattern[patternIndex] === ANY_CHAR) {
                /*
                 * In case if current pattern char is the same as current string char
                 * or it may be any character (in case if pattern contains '.' char)
                 * we need to check if there was a match for the pattern and for the
                 * string by WITHOUT current char. This would mean that we may copy
                 * left-top diagonal value.
                 *
                 * Example:
                 *
                 *   a b
                 * a 1 -
                 * b - 1
                 */
                matchMatrix[rowIndex][columnIndex] = matchMatrix[rowIndex - 1][columnIndex - 1];
            }
            else {
                /*
                 * In case if pattern char and string char are different we may
                 * treat this case as "no-match".
                 *
                 * Example:
                 *
                 *   a b
                 * a - -
                 * c - 0
                 */
                matchMatrix[rowIndex][columnIndex] = false;
            }
        }
    }
    return matchMatrix[string.length][pattern.length];
}

// The string separator that is being used for "word" and "text" concatenation.
var SEPARATOR = '$';
/**
 * @param zString
 */
function buildZArray(zString) {
    // Initiate zArray and fill it with zeros.
    var zArray = new Array(zString.length).fill(null).map(function () { return 0; });
    // Z box boundaries.
    var zBoxLeftIndex = 0;
    var zBoxRightIndex = 0;
    // Position of current zBox character that is also a position of
    // the same character in prefix.
    // For example:
    // Z string: ab$xxabxx
    // Indices:  012345678
    // Prefix:   ab.......
    // Z box:    .....ab..
    // Z box shift for 'a' would be 0 (0-position in prefix and 0-position in Z box)
    // Z box shift for 'b' would be 1 (1-position in prefix and 1-position in Z box)
    var zBoxShift = 0;
    // Go through all characters of the zString.
    for (var charIndex = 1; charIndex < zString.length; charIndex += 1) {
        if (charIndex > zBoxRightIndex) {
            // We're OUTSIDE of Z box. In other words this is a case when we're
            // starting from Z box of size 1.
            // In this case let's make current character to be a Z box of length 1.
            zBoxLeftIndex = charIndex;
            zBoxRightIndex = charIndex;
            // Now let's go and check current and the following characters to see if
            // they are the same as a prefix. By doing this we will also expand our
            // Z box. For example if starting from current position we will find 3
            // more characters that are equal to the ones in the prefix we will expand
            // right Z box boundary by 3.
            while (zBoxRightIndex < zString.length
                && zString[zBoxRightIndex - zBoxLeftIndex] === zString[zBoxRightIndex]) {
                // Expanding Z box right boundary.
                zBoxRightIndex += 1;
            }
            // Now we may calculate how many characters starting from current position
            // are are the same as the prefix. We may calculate it by difference between
            // right and left Z box boundaries.
            zArray[charIndex] = zBoxRightIndex - zBoxLeftIndex;
            // Move right Z box boundary left by one position just because we've used
            // [zBoxRightIndex - zBoxLeftIndex] index calculation above.
            zBoxRightIndex -= 1;
        }
        else {
            // We're INSIDE of Z box.
            // Calculate corresponding Z box shift. Because we want to copy the values
            // from zArray that have been calculated before.
            zBoxShift = charIndex - zBoxLeftIndex;
            // Check if the value that has been already calculated before
            // leaves us inside of Z box or it goes beyond the checkbox
            // right boundary.
            if (zArray[zBoxShift] < (zBoxRightIndex - charIndex) + 1) {
                // If calculated value don't force us to go outside Z box
                // then we're safe and we may simply use previously calculated value.
                zArray[charIndex] = zArray[zBoxShift];
            }
            else {
                // In case if previously calculated values forces us to go outside of Z box
                // we can't safely copy previously calculated zArray value. It is because
                // we are sure that there is no further prefix matches outside of Z box.
                // Thus such values must be re-calculated and reduced to certain point.
                // To do so we need to shift left boundary of Z box to current position.
                zBoxLeftIndex = charIndex;
                // And start comparing characters one by one as we normally do for the case
                // when we are outside of checkbox.
                while (zBoxRightIndex < zString.length
                    && zString[zBoxRightIndex - zBoxLeftIndex] === zString[zBoxRightIndex]) {
                    zBoxRightIndex += 1;
                }
                zArray[charIndex] = zBoxRightIndex - zBoxLeftIndex;
                zBoxRightIndex -= 1;
            }
        }
    }
    // Return generated zArray.
    return zArray;
}
/**
 * @param text
 * @param word
 */
function zAlgorithm(text, word) {
    // The list of word's positions in text. Word may be found in the same text
    // in several different positions. Thus it is an array.
    var wordPositions = [];
    // Concatenate word and string. Word will be a prefix to a string.
    var zString = "" + word + SEPARATOR + text;
    // Generate Z-array for concatenated string.
    var zArray = buildZArray(zString);
    // Based on Z-array properties each cell will tell us the length of the match between
    // the string prefix and current sub-text. Thus we're may find all positions in zArray
    // with the number that equals to the length of the word (zString prefix) and based on
    // that positions we'll be able to calculate word positions in text.
    for (var charIndex = 1; charIndex < zArray.length; charIndex += 1) {
        if (zArray[charIndex] === word.length) {
            // Since we did concatenation to form zString we need to subtract prefix
            // and separator lengths.
            var wordPosition = charIndex - word.length - SEPARATOR.length;
            wordPositions.push(wordPosition);
        }
    }
    // Return the list of word positions.
    return wordPositions;
}

/**
 * @param callbacks
 */
function initCallbacks$1(callbacks) {
    if (callbacks === void 0) { callbacks = {}; }
    var initiatedCallback = callbacks;
    var stubCallback = function () { };
    var defaultAllowTraversal = function () { return true; };
    initiatedCallback.allowTraversal = callbacks.allowTraversal || defaultAllowTraversal;
    initiatedCallback.enterNode = callbacks.enterNode || stubCallback;
    initiatedCallback.leaveNode = callbacks.leaveNode || stubCallback;
    return initiatedCallback;
}
/**
 * 广度优先搜索算法（英语：Breadth-First Search，缩写为BFS）
 *
 * @param rootNode
 * @param originalCallbacks
 */
function breadthFirstSearch1(rootNode, originalCallbacks) {
    var callbacks = initCallbacks$1(originalCallbacks);
    var nodeQueue = new Queue();
    // Do initial queue setup.
    nodeQueue.enqueue(rootNode);
    while (!nodeQueue.isEmpty()) {
        var currentNode = nodeQueue.dequeue();
        callbacks.enterNode(currentNode);
        // Add all children to the queue for future traversals.
        // Traverse left branch.
        if (currentNode.left && callbacks.allowTraversal(currentNode, currentNode.left)) {
            nodeQueue.enqueue(currentNode.left);
        }
        // Traverse right branch.
        if (currentNode.right && callbacks.allowTraversal(currentNode, currentNode.right)) {
            nodeQueue.enqueue(currentNode.right);
        }
        callbacks.leaveNode(currentNode);
    }
}

/**
 * Extend missing traversal callbacks with default callbacks.
 *
 * @param callbacks The object that contains traversal callbacks.
 * @returns  Traversal callbacks extended with defaults callbacks.
 */
function initCallbacks(callbacks) {
    if (callbacks === void 0) { callbacks = {}; }
    // Init empty callbacks object.
    var initiatedCallbacks = {};
    // Empty callback that we will use in case if user didn't provide real callback function.
    var stubCallback = function () { };
    // By default we will allow traversal of every node
    // in case if user didn't provide a callback for that.
    var defaultAllowTraversalCallback = function () { return true; };
    // Copy original callbacks to our initiatedCallbacks object or use default callbacks instead.
    initiatedCallbacks.allowTraversal = callbacks.allowTraversal || defaultAllowTraversalCallback;
    initiatedCallbacks.enterNode = callbacks.enterNode || stubCallback;
    initiatedCallbacks.leaveNode = callbacks.leaveNode || stubCallback;
    // Returned processed list of callbacks.
    return initiatedCallbacks;
}
/**
 * Recursive depth-first-search traversal for binary.
 *
 * @param node binary tree node that we will start traversal from.
 * @param callbacks the object that contains traversal callbacks.
 */
function depthFirstSearchRecursive(node, callbacks) {
    // Call the "enterNode" callback to notify that the node is going to be entered.
    callbacks.enterNode(node);
    // Traverse left branch only if case if traversal of the left node is allowed.
    if (node.left && callbacks.allowTraversal(node, node.left)) {
        depthFirstSearchRecursive(node.left, callbacks);
    }
    // Traverse right branch only if case if traversal of the right node is allowed.
    if (node.right && callbacks.allowTraversal(node, node.right)) {
        depthFirstSearchRecursive(node.right, callbacks);
    }
    // Call the "leaveNode" callback to notify that traversal
    // of the current node and its children is finished.
    callbacks.leaveNode(node);
}
/**
 * Perform depth-first-search traversal of the rootNode.
 * For every traversal step call "allowTraversal", "enterNode" and "leaveNode" callbacks.
 * See TraversalCallbacks type definition for more details about the shape of callbacks object.
 *
 * @param rootNode The node from which we start traversing.
 * @param callbacks Traversal callbacks.
 */
function depthFirstSearch1(rootNode, callbacks) {
    // In case if user didn't provide some callback we need to replace them with default ones.
    var processedCallbacks = initCallbacks(callbacks);
    // Now, when we have all necessary callbacks we may proceed to recursive traversal.
    depthFirstSearchRecursive(rootNode, processedCallbacks);
}

/**
 * Finds the maximum profit from selling and buying the stocks.
 * ACCUMULATOR APPROACH.
 *
 * @param prices Array of stock prices, i.e. [7, 6, 4, 3, 1]
 * @param visit Visiting callback to calculate the number of iterations.
 * @return The maximum profit
 */
function accumulatorBestTimeToBuySellStocks(prices, visit) {
    if (visit === void 0) { visit = function () { }; }
    visit();
    var profit = 0;
    for (var day = 1; day < prices.length; day += 1) {
        visit();
        // Add the increase of the price from yesterday till today (if there was any) to the profit.
        profit += Math.max(prices[day] - prices[day - 1], 0);
    }
    return profit;
}

/**
 * Finds the maximum profit from selling and buying the stocks.
 * DYNAMIC PROGRAMMING APPROACH.
 *
 * @param prices Array of stock prices, i.e. [7, 6, 4, 3, 1]
 * @param visit Visiting callback to calculate the number of iterations.
 * @return The maximum profit
 */
function dpBestTimeToBuySellStocks(prices, visit) {
    if (visit === void 0) { visit = function () { }; }
    visit();
    var lastBuy = -prices[0];
    var lastSold = 0;
    for (var day = 1; day < prices.length; day += 1) {
        visit();
        var curBuy = Math.max(lastBuy, lastSold - prices[day]);
        var curSold = Math.max(lastSold, lastBuy + prices[day]);
        lastBuy = curBuy;
        lastSold = curSold;
    }
    return lastSold;
}

/**
 * Finds the maximum profit from selling and buying the stocks.
 * DIVIDE & CONQUER APPROACH.
 *
 * @param prices Array of stock prices, i.e. [7, 6, 4, 3, 1]
 * @param visit Visiting callback to calculate the number of iterations.
 * @return The maximum profit
 */
function dqBestTimeToBuySellStocks(prices, visit) {
    if (visit === void 0) { visit = function () { }; }
    /**
     * Recursive implementation of the main function. It is hidden from the users.
     *
     * @param buy Whether we're allow to sell or to buy now
     * @param day Current day of trading (current index of prices array)
     * @returns Max profit from buying/selling
     */
    var recursiveBuyerSeller = function (buy, day) {
        // Registering the recursive call visit to calculate the complexity.
        visit();
        // Quitting the recursion if this is the last day of trading (prices array ended).
        if (day === prices.length) {
            return 0;
        }
        // If we're buying - we're loosing money (-1), if we're selling we're getting money (+1).
        var operationSign = buy ? -1 : +1;
        return Math.max(
        // Option 1: Don't do anything.
        recursiveBuyerSeller(buy, day + 1), 
        // Option 2: Sell or Buy at the current price.
        operationSign * prices[day] + recursiveBuyerSeller(!buy, day + 1));
    };
    var buy = true;
    var day = 0;
    return recursiveBuyerSeller(buy, day);
}

/**
 * Finds the maximum profit from selling and buying the stocks.
 * PEAK VALLEY APPROACH.
 *
 * @param prices Array of stock prices, i.e. [7, 6, 4, 3, 1]
 * @param visit Visiting callback to calculate the number of iterations.
 * @return The maximum profit
 */
function peakvalleyBestTimeToBuySellStocks(prices, visit) {
    if (visit === void 0) { visit = function () { }; }
    visit();
    var profit = 0;
    var low = prices[0];
    var high = prices[0];
    prices.slice(1).forEach(function (currentPrice) {
        visit();
        if (currentPrice < high) {
            // If price went down, we need to sell.
            profit += high - low;
            low = currentPrice;
            high = currentPrice;
        }
        else {
            // If price went up, we don't need to do anything but increase a high record.
            high = currentPrice;
        }
    });
    // In case if price went up during the last day
    // and we didn't have chance to sell inside the forEach loop.
    profit += high - low;
    return profit;
}

function hanoiTowerRecursive(_a) {
    var numberOfDiscs = _a.numberOfDiscs, fromPole = _a.fromPole, withPole = _a.withPole, toPole = _a.toPole, moveCallback = _a.moveCallback;
    if (numberOfDiscs === 1) {
        // Base case with just one disc.
        moveCallback(fromPole.peek(), fromPole.toArray(), toPole.toArray());
        var disc = fromPole.pop();
        toPole.push(disc);
    }
    else {
        // In case if there are more discs then move them recursively.
        // Expose the bottom disc on fromPole stack.
        hanoiTowerRecursive({
            numberOfDiscs: numberOfDiscs - 1,
            fromPole: fromPole,
            withPole: toPole,
            toPole: withPole,
            moveCallback: moveCallback,
        });
        // Move the disc that was exposed to its final destination.
        hanoiTowerRecursive({
            numberOfDiscs: 1,
            fromPole: fromPole,
            withPole: withPole,
            toPole: toPole,
            moveCallback: moveCallback,
        });
        // Move temporary tower from auxiliary pole to its final destination.
        hanoiTowerRecursive({
            numberOfDiscs: numberOfDiscs - 1,
            fromPole: withPole,
            withPole: fromPole,
            toPole: toPole,
            moveCallback: moveCallback,
        });
    }
}
function hanoiTower(_a) {
    // Each of three poles of Tower of Hanoi puzzle is represented as a stack
    // that might contain elements (discs). Each disc is represented as a number.
    // Larger discs have bigger number equivalent.
    var numberOfDiscs = _a.numberOfDiscs, moveCallback = _a.moveCallback, _b = _a.fromPole, fromPole = _b === void 0 ? new Stack() : _b, _c = _a.withPole, withPole = _c === void 0 ? new Stack() : _c, _d = _a.toPole, toPole = _d === void 0 ? new Stack() : _d;
    // Let's create the discs and put them to the fromPole.
    for (var discSize = numberOfDiscs; discSize > 0; discSize -= 1) {
        fromPole.push(discSize);
    }
    hanoiTowerRecursive({
        numberOfDiscs: numberOfDiscs,
        fromPole: fromPole,
        withPole: withPole,
        toPole: toPole,
        moveCallback: moveCallback,
    });
}

/**
 * BACKTRACKING approach of solving Jump Game.
 *
 * This is the inefficient solution where we try every single jump
 * pattern that takes us from the first position to the last.
 * We start from the first position and jump to every index that
 * is reachable. We repeat the process until last index is reached.
 * When stuck, backtrack.
 *
 * @param numbers array of possible jump length.
 * @param startIndex index from where we start jumping.
 * @param currentJumps current jumps path.
 */
function backtrackingJumpGame(numbers, startIndex, currentJumps) {
    if (startIndex === void 0) { startIndex = 0; }
    if (currentJumps === void 0) { currentJumps = []; }
    if (startIndex === numbers.length - 1) {
        // We've jumped directly to last cell. This situation is a solution.
        return true;
    }
    // Check what the longest jump we could make from current position.
    // We don't need to jump beyond the array.
    var maxJumpLength = Math.min(numbers[startIndex], // Jump is within array.
    numbers.length - 1 - startIndex);
    // Let's start jumping from startIndex and see whether any
    // jump is successful and has reached the end of the array.
    for (var jumpLength = maxJumpLength; jumpLength > 0; jumpLength -= 1) {
        // Try next jump.
        var nextIndex = startIndex + jumpLength;
        currentJumps.push(nextIndex);
        var isJumpSuccessful = backtrackingJumpGame(numbers, nextIndex, currentJumps);
        // Check if current jump was successful.
        if (isJumpSuccessful) {
            return true;
        }
        // BACKTRACKING.
        // If previous jump wasn't successful then retreat and try the next one.
        currentJumps.pop();
    }
    return false;
}

/**
 * DYNAMIC PROGRAMMING BOTTOM-UP approach of solving Jump Game.
 *
 * This comes out as an optimisation of DYNAMIC PROGRAMMING TOP-DOWN approach.
 *
 * The observation to make here is that we only ever jump to the right.
 * This means that if we start from the right of the array, every time we
 * will query a position to our right, that position has already be
 * determined as being GOOD or BAD. This means we don't need to recurse
 * anymore, as we will always hit the memo table.
 *
 * We call a position in the array a "good" one if starting at that
 * position, we can reach the last index. Otherwise, that index
 * is called a "bad" one.
 *
 * @param numbers array of possible jump length.
 */
function dpBottomUpJumpGame(numbers) {
    // Init cells goodness table.
    var cellsGoodness = Array(numbers.length).fill(undefined);
    // Mark the last cell as "good" one since it is where we ultimately want to get.
    cellsGoodness[cellsGoodness.length - 1] = true;
    // Go throw all cells starting from the one before the last
    // one (since the last one is "good" already) and fill cellsGoodness table.
    for (var cellIndex = numbers.length - 2; cellIndex >= 0; cellIndex -= 1) {
        var maxJumpLength = Math.min(numbers[cellIndex], numbers.length - 1 - cellIndex);
        for (var jumpLength = maxJumpLength; jumpLength > 0; jumpLength -= 1) {
            var nextIndex = cellIndex + jumpLength;
            if (cellsGoodness[nextIndex] === true) {
                cellsGoodness[cellIndex] = true;
                // Once we detected that current cell is good one we don't need to
                // do further cells checking.
                break;
            }
        }
    }
    // Now, if the zero's cell is good one then we can jump from it to the end of the array.
    return cellsGoodness[0] === true;
}

/**
 * DYNAMIC PROGRAMMING TOP-DOWN approach of solving Jump Game.
 *
 * This comes out as an optimisation of BACKTRACKING approach.
 *
 * It relies on the observation that once we determine that a certain
 * index is good / bad, this result will never change. This means that
 * we can store the result and not need to recompute it every time.
 *
 * We call a position in the array a "good" one if starting at that
 * position, we can reach the last index. Otherwise, that index
 * is called a "bad" one.
 *
 * @param numbers array of possible jump length.
 * @param startIndex index from where we start jumping.
 * @param currentJumps current jumps path.
 * @param cellsGoodness holds information about whether cell is "good" or "bad"
 */
function dpTopDownJumpGame(numbers, startIndex, currentJumps, cellsGoodness) {
    if (startIndex === void 0) { startIndex = 0; }
    if (currentJumps === void 0) { currentJumps = []; }
    if (cellsGoodness === void 0) { cellsGoodness = []; }
    if (startIndex === numbers.length - 1) {
        // We've jumped directly to last cell. This situation is a solution.
        return true;
    }
    // Init cell goodness table if it is empty.
    // This is DYNAMIC PROGRAMMING feature.
    var currentCellsGoodness = __spreadArray([], __read(cellsGoodness));
    if (!currentCellsGoodness.length) {
        numbers.forEach(function () { return currentCellsGoodness.push(undefined); });
        // Mark the last cell as "good" one since it is where we ultimately want to get.
        currentCellsGoodness[cellsGoodness.length - 1] = true;
    }
    // Check what the longest jump we could make from current position.
    // We don't need to jump beyond the array.
    var maxJumpLength = Math.min(numbers[startIndex], // Jump is within array.
    numbers.length - 1 - startIndex);
    // Let's start jumping from startIndex and see whether any
    // jump is successful and has reached the end of the array.
    for (var jumpLength = maxJumpLength; jumpLength > 0; jumpLength -= 1) {
        // Try next jump.
        var nextIndex = startIndex + jumpLength;
        // Jump only into "good" or "unknown" cells.
        // This is top-down dynamic programming optimisation of backtracking algorithm.
        if (currentCellsGoodness[nextIndex] !== false) {
            currentJumps.push(nextIndex);
            var isJumpSuccessful = dpTopDownJumpGame(numbers, nextIndex, currentJumps, currentCellsGoodness);
            // Check if current jump was successful.
            if (isJumpSuccessful) {
                return true;
            }
            // BACKTRACKING.
            // If previous jump wasn't successful then retreat and try the next one.
            currentJumps.pop();
            // Mark current cell as "bad" to avoid its deep visiting later.
            currentCellsGoodness[nextIndex] = false;
        }
    }
    return false;
}

/**
 * GREEDY approach of solving Jump Game.
 *
 * This comes out as an optimisation of DYNAMIC PROGRAMMING BOTTOM_UP approach.
 *
 * Once we have our code in the bottom-up state, we can make one final,
 * important observation. From a given position, when we try to see if
 * we can jump to a GOOD position, we only ever use one - the first one.
 * In other words, the left-most one. If we keep track of this left-most
 * GOOD position as a separate variable, we can avoid searching for it
 * in the array. Not only that, but we can stop using the array altogether.
 *
 * We call a position in the array a "good" one if starting at that
 * position, we can reach the last index. Otherwise, that index
 * is called a "bad" one.
 *
 * @param numbers array of possible jump length.
 */
function greedyJumpGame(numbers) {
    // The "good" cell is a cell from which we may jump to the last cell of the numbers array.
    // The last cell in numbers array is for sure the "good" one since it is our goal to reach.
    var leftGoodPosition = numbers.length - 1;
    // Go through all numbers from right to left.
    for (var numberIndex = numbers.length - 2; numberIndex >= 0; numberIndex -= 1) {
        // If we can reach the "good" cell from the current one then for sure the current
        // one is also "good". Since after all we'll be able to reach the end of the array
        // from it.
        var maxCurrentJumpLength = numberIndex + numbers[numberIndex];
        if (maxCurrentJumpLength >= leftGoodPosition) {
            leftGoodPosition = numberIndex;
        }
    }
    // If the most left "good" position is the zero's one then we may say that it IS
    // possible jump to the end of the array from the first cell;
    return leftGoodPosition === 0;
}

/**
 * @param chessboard
 * @param position
 */
function getPossibleMoves(chessboard, position) {
    // Generate all knight moves (even those that go beyond the board).
    var possibleMoves = [
        [position[0] - 1, position[1] - 2],
        [position[0] - 2, position[1] - 1],
        [position[0] + 1, position[1] - 2],
        [position[0] + 2, position[1] - 1],
        [position[0] - 2, position[1] + 1],
        [position[0] - 1, position[1] + 2],
        [position[0] + 1, position[1] + 2],
        [position[0] + 2, position[1] + 1] ];
    // Filter out all moves that go beyond the board.
    return possibleMoves.filter(function (move) {
        var boardSize = chessboard.length;
        return move[0] >= 0 && move[1] >= 0 && move[0] < boardSize && move[1] < boardSize;
    });
}
/**
 * @param chessboard
 * @param move
 */
function isMoveAllowed(chessboard, move) {
    return chessboard[move[0]][move[1]] !== 1;
}
/**
 * @param chessboard
 * @param moves
 */
function isBoardCompletelyVisited(chessboard, moves) {
    var totalPossibleMovesCount = Math.pow(chessboard.length, 2);
    var existingMovesCount = moves.length;
    return totalPossibleMovesCount === existingMovesCount;
}
/**
 * @param chessboard
 * @param moves
 */
function knightTourRecursive(chessboard, moves) {
    var currentChessboard = chessboard;
    // If board has been completely visited then we've found a solution.
    if (isBoardCompletelyVisited(currentChessboard, moves)) {
        return true;
    }
    // Get next possible knight moves.
    var lastMove = moves[moves.length - 1];
    var possibleMoves = getPossibleMoves(currentChessboard, lastMove);
    // Try to do next possible moves.
    for (var moveIndex = 0; moveIndex < possibleMoves.length; moveIndex += 1) {
        var currentMove = possibleMoves[moveIndex];
        // Check if current move is allowed. We aren't allowed to go to
        // the same cells twice.
        if (isMoveAllowed(currentChessboard, currentMove)) {
            // Actually do the move.
            moves.push(currentMove);
            currentChessboard[currentMove[0]][currentMove[1]] = 1;
            // If further moves starting from current are successful then
            // return true meaning the solution is found.
            if (knightTourRecursive(currentChessboard, moves)) {
                return true;
            }
            // BACKTRACKING.
            // If current move was unsuccessful then step back and try to do another move.
            moves.pop();
            currentChessboard[currentMove[0]][currentMove[1]] = 0;
        }
    }
    // Return false if we haven't found solution.
    return false;
}
/**
 * @param chessboardSize
 */
function knightTour(chessboardSize) {
    // Init chessboard.
    var chessboard = Array(chessboardSize).fill(null).map(function () { return Array(chessboardSize).fill(0); });
    // Init moves array.
    var moves = [];
    // Do first move and place the knight to the 0x0 cell.
    var firstMove = [0, 0];
    moves.push(firstMove);
    chessboard[firstMove[0]][firstMove[0]] = 1;
    // Recursively try to do the next move.
    var solutionWasFound = knightTourRecursive(chessboard, moves);
    return solutionWasFound ? moves : [];
}

/**
 * Class that represents queen position on the chessboard.
 */
var QueenPosition = /** @class */ (function () {
    /**
     * @param rowIndex
     * @param columnIndex
     */
    function QueenPosition(rowIndex, columnIndex) {
        this.rowIndex = rowIndex;
        this.columnIndex = columnIndex;
    }
    Object.defineProperty(QueenPosition.prototype, "leftDiagonal", {
        get: function () {
            // Each position on the same left (\) diagonal has the same difference of
            // rowIndex and columnIndex. This fact may be used to quickly check if two
            // positions (queens) are on the same left diagonal.
            // @see https://youtu.be/xouin83ebxE?t=1m59s
            return this.rowIndex - this.columnIndex;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(QueenPosition.prototype, "rightDiagonal", {
        get: function () {
            // Each position on the same right diagonal (/) has the same
            // sum of rowIndex and columnIndex. This fact may be used to quickly
            // check if two positions (queens) are on the same right diagonal.
            // @see https://youtu.be/xouin83ebxE?t=1m59s
            return this.rowIndex + this.columnIndex;
        },
        enumerable: false,
        configurable: true
    });
    QueenPosition.prototype.toString = function () {
        return this.rowIndex + "," + this.columnIndex;
    };
    return QueenPosition;
}());

/**
 * @param queensPositions
 * @param rowIndex
 * @param columnIndex
 */
function isSafe(queensPositions, rowIndex, columnIndex) {
    // New position to which the Queen is going to be placed.
    var newQueenPosition = new QueenPosition(rowIndex, columnIndex);
    // Check if new queen position conflicts with any other queens.
    for (var queenIndex = 0; queenIndex < queensPositions.length; queenIndex += 1) {
        var currentQueenPosition = queensPositions[queenIndex];
        if (
        // Check if queen has been already placed.
        currentQueenPosition
            && (
            // Check if there are any queen on the same column.
            newQueenPosition.columnIndex === currentQueenPosition.columnIndex
                // Check if there are any queen on the same row.
                || newQueenPosition.rowIndex === currentQueenPosition.rowIndex
                // Check if there are any queen on the same left diagonal.
                || newQueenPosition.leftDiagonal === currentQueenPosition.leftDiagonal
                // Check if there are any queen on the same right diagonal.
                || newQueenPosition.rightDiagonal === currentQueenPosition.rightDiagonal)) {
            // Can't place queen into current position since there are other queens that
            // are threatening it.
            return false;
        }
    }
    // Looks like we're safe.
    return true;
}
/**
 * @param solutions
 * @param previousQueensPositions
 * @param queensCount
 * @param rowIndex
 */
function nQueensRecursive(solutions, previousQueensPositions, queensCount, rowIndex) {
    // Clone positions array.
    var queensPositions = __spreadArray([], __read(previousQueensPositions)).map(function (queenPosition) {
        return (!queenPosition ? queenPosition : new QueenPosition(queenPosition.rowIndex, queenPosition.columnIndex));
    });
    if (rowIndex === queensCount) {
        // We've successfully reached the end of the board.
        // Store solution to the list of solutions.
        solutions.push(queensPositions);
        // Solution found.
        return true;
    }
    // Let's try to put queen at row rowIndex into its safe column position.
    for (var columnIndex = 0; columnIndex < queensCount; columnIndex += 1) {
        if (isSafe(queensPositions, rowIndex, columnIndex)) {
            // Place current queen to its current position.
            queensPositions[rowIndex] = new QueenPosition(rowIndex, columnIndex);
            // Try to place all other queens as well.
            nQueensRecursive(solutions, queensPositions, queensCount, rowIndex + 1);
            // BACKTRACKING.
            // Remove the queen from the row to avoid isSafe() returning false.
            queensPositions[rowIndex] = null;
        }
    }
    return false;
}
/**
 * @param queensCount
 */
function nQueens(queensCount) {
    // Init NxN chessboard with zeros.
    // const chessboard = Array(queensCount).fill(null).map(() => Array(queensCount).fill(0));
    // This array will hold positions or coordinates of each of
    // N queens in form of [rowIndex, columnIndex].
    var queensPositions = Array(queensCount).fill(null);
    /** @var {QueenPosition[][]} solutions */
    var solutions = [];
    // Solve problem recursively.
    nQueensRecursive(solutions, queensPositions, queensCount, 0);
    return solutions;
}

/**
 * BRUTE FORCE approach of solving Trapping Rain Water problem.
 *
 * @param terraces
 */
function bfRainTerraces(terraces) {
    var waterAmount = 0;
    for (var terraceIndex = 0; terraceIndex < terraces.length; terraceIndex += 1) {
        // Get left most high terrace.
        var leftHighestLevel = 0;
        for (var leftIndex = terraceIndex - 1; leftIndex >= 0; leftIndex -= 1) {
            leftHighestLevel = Math.max(leftHighestLevel, terraces[leftIndex]);
        }
        // Get right most high terrace.
        var rightHighestLevel = 0;
        for (var rightIndex = terraceIndex + 1; rightIndex < terraces.length; rightIndex += 1) {
            rightHighestLevel = Math.max(rightHighestLevel, terraces[rightIndex]);
        }
        // Add current terrace water amount.
        var terraceBoundaryLevel = Math.min(leftHighestLevel, rightHighestLevel);
        if (terraceBoundaryLevel > terraces[terraceIndex]) {
            // Terrace will be able to store the water if the lowest of two left and right highest
            // terraces are still higher than the current one.
            waterAmount += terraceBoundaryLevel - terraces[terraceIndex];
        }
    }
    return waterAmount;
}

/**
 * DYNAMIC PROGRAMMING approach of solving Trapping Rain Water problem.
 *
 * @param terraces
 */
function dpRainTerraces(terraces) {
    var _a;
    var waterAmount = 0;
    // Init arrays that will keep the list of left and right maximum levels for specific positions.
    var leftMaxLevels = new Array(terraces.length).fill(0);
    var rightMaxLevels = new Array(terraces.length).fill(0);
    // Calculate the highest terrace level from the LEFT relative to the current terrace.
    _a = __read(terraces, 1), leftMaxLevels[0] = _a[0];
    for (var terraceIndex = 1; terraceIndex < terraces.length; terraceIndex += 1) {
        leftMaxLevels[terraceIndex] = Math.max(terraces[terraceIndex], leftMaxLevels[terraceIndex - 1]);
    }
    // Calculate the highest terrace level from the RIGHT relative to the current terrace.
    rightMaxLevels[terraces.length - 1] = terraces[terraces.length - 1];
    for (var terraceIndex = terraces.length - 2; terraceIndex >= 0; terraceIndex -= 1) {
        rightMaxLevels[terraceIndex] = Math.max(terraces[terraceIndex], rightMaxLevels[terraceIndex + 1]);
    }
    // Not let's go through all terraces one by one and calculate how much water
    // each terrace may accumulate based on previously calculated values.
    for (var terraceIndex = 0; terraceIndex < terraces.length; terraceIndex += 1) {
        // Pick the lowest from the left/right highest terraces.
        var currentTerraceBoundary = Math.min(leftMaxLevels[terraceIndex], rightMaxLevels[terraceIndex]);
        if (currentTerraceBoundary > terraces[terraceIndex]) {
            waterAmount += currentTerraceBoundary - terraces[terraceIndex];
        }
    }
    return waterAmount;
}

/**
 * Recursive Staircase Problem (Brute Force Solution).
 *
 * @param stairsNum Number of stairs to climb on.
 * @return Number of ways to climb a staircase.
 */
function recursiveStaircaseBF(stairsNum) {
    if (stairsNum <= 0) {
        // There is no way to go down - you climb the stairs only upwards.
        // Also if you're standing on the ground floor that you don't need to do any further steps.
        return 0;
    }
    if (stairsNum === 1) {
        // There is only one way to go to the first step.
        return 1;
    }
    if (stairsNum === 2) {
        // There are two ways to get to the second steps: (1 + 1) or (2).
        return 2;
    }
    // Sum up how many steps we need to take after doing one step up with the number of
    // steps we need to take after doing two steps up.
    return recursiveStaircaseBF(stairsNum - 1) + recursiveStaircaseBF(stairsNum - 2);
}

/**
 * Recursive Staircase Problem (Dynamic Programming Solution).
 *
 * @param stairsNum Number of stairs to climb on.
 * @return Number of ways to climb a staircase.
 */
function recursiveStaircaseDP(stairsNum) {
    if (stairsNum < 0) {
        // There is no way to go down - you climb the stairs only upwards.
        return 0;
    }
    // Init the steps vector that will hold all possible ways to get to the corresponding step.
    var steps = new Array(stairsNum + 1).fill(0);
    // Init the number of ways to get to the 0th, 1st and 2nd steps.
    steps[0] = 0;
    steps[1] = 1;
    steps[2] = 2;
    if (stairsNum <= 2) {
        // Return the number of ways to get to the 0th or 1st or 2nd steps.
        return steps[stairsNum];
    }
    // Calculate every next step based on two previous ones.
    for (var currentStep = 3; currentStep <= stairsNum; currentStep += 1) {
        steps[currentStep] = steps[currentStep - 1] + steps[currentStep - 2];
    }
    // Return possible ways to get to the requested step.
    return steps[stairsNum];
}

/**
 * Recursive Staircase Problem (Iterative Solution).
 *
 * @param stairsNum Number of stairs to climb on.
 * @return Number of ways to climb a staircase.
 */
function recursiveStaircaseIT(stairsNum) {
    var _a;
    if (stairsNum <= 0) {
        // There is no way to go down - you climb the stairs only upwards.
        // Also you don't need to do anything to stay on the 0th step.
        return 0;
    }
    // Init the number of ways to get to the 0th, 1st and 2nd steps.
    var steps = [1, 2];
    if (stairsNum <= 2) {
        // Return the number of possible ways of how to get to the 1st or 2nd steps.
        return steps[stairsNum - 1];
    }
    // Calculate the number of ways to get to the n'th step based on previous ones.
    // Comparing to Dynamic Programming solution we don't store info for all the steps but
    // rather for two previous ones only.
    for (var currentStep = 3; currentStep <= stairsNum; currentStep += 1) {
        _a = __read([steps[1], steps[0] + steps[1]], 2), steps[0] = _a[0], steps[1] = _a[1];
    }
    // Return possible ways to get to the requested step.
    return steps[1];
}

/**
 * Recursive Staircase Problem (Recursive Solution With Memoization).
 *
 * @param totalStairs Number of stairs to climb on.
 * @return Number of ways to climb a staircase.
 */
function recursiveStaircaseMEM(totalStairs) {
    // Memo table that will hold all recursively calculated results to avoid calculating them
    // over and over again.
    var memo = [];
    // Recursive closure.
    var getSteps = function (stairsNum) {
        if (stairsNum <= 0) {
            // There is no way to go down - you climb the stairs only upwards.
            // Also if you're standing on the ground floor that you don't need to do any further steps.
            return 0;
        }
        if (stairsNum === 1) {
            // There is only one way to go to the first step.
            return 1;
        }
        if (stairsNum === 2) {
            // There are two ways to get to the second steps: (1 + 1) or (2).
            return 2;
        }
        // Avoid recursion for the steps that we've calculated recently.
        if (memo[stairsNum]) {
            return memo[stairsNum];
        }
        // Sum up how many steps we need to take after doing one step up with the number of
        // steps we need to take after doing two steps up.
        memo[stairsNum] = getSteps(stairsNum - 1) + getSteps(stairsNum - 2);
        return memo[stairsNum];
    };
    // Return possible ways to get to the requested step.
    return getSteps(totalStairs);
}

/**
 * @param originalMatrix
 */
function squareMatrixRotation(originalMatrix) {
    var _a, _b;
    var matrix = originalMatrix.slice();
    // Do top-right/bottom-left diagonal reflection of the matrix.
    for (var rowIndex = 0; rowIndex < matrix.length; rowIndex += 1) {
        for (var columnIndex = rowIndex + 1; columnIndex < matrix.length; columnIndex += 1) {
            // Swap elements.
            _a = __read([
                matrix[rowIndex][columnIndex],
                matrix[columnIndex][rowIndex] ], 2), matrix[columnIndex][rowIndex] = _a[0], matrix[rowIndex][columnIndex] = _a[1];
        }
    }
    // Do horizontal reflection of the matrix.
    for (var rowIndex = 0; rowIndex < matrix.length; rowIndex += 1) {
        for (var columnIndex = 0; columnIndex < matrix.length / 2; columnIndex += 1) {
            // Swap elements.
            _b = __read([
                matrix[rowIndex][columnIndex],
                matrix[rowIndex][matrix.length - columnIndex - 1] ], 2), matrix[rowIndex][matrix.length - columnIndex - 1] = _b[0], matrix[rowIndex][columnIndex] = _b[1];
        }
    }
    return matrix;
}

/**
 * BACKTRACKING approach of solving Unique Paths problem.
 *
 * @param width Width of the board.
 * @param height Height of the board.
 * @param steps The steps that have been already made.
 * @param uniqueSteps - Total number of unique steps.
 * @return Number of unique paths.
 */
function btUniquePaths(width, height, steps, uniqueSteps) {
    if (steps === void 0) { steps = [[0, 0]]; }
    if (uniqueSteps === void 0) { uniqueSteps = 0; }
    // Fetch current position on board.
    var currentPos = steps[steps.length - 1];
    // Check if we've reached the end.
    if (currentPos[0] === width - 1 && currentPos[1] === height - 1) {
        // In case if we've reached the end let's increase total
        // number of unique steps.
        return uniqueSteps + 1;
    }
    // Let's calculate how many unique path we will have
    // by going right and by going down.
    var rightUniqueSteps = 0;
    var downUniqueSteps = 0;
    // Do right step if possible.
    if (currentPos[0] < width - 1) {
        steps.push([
            currentPos[0] + 1,
            currentPos[1] ]);
        // Calculate how many unique paths we'll get by moving right.
        rightUniqueSteps = btUniquePaths(width, height, steps, uniqueSteps);
        // BACKTRACK and try another move.
        steps.pop();
    }
    // Do down step if possible.
    if (currentPos[1] < height - 1) {
        steps.push([
            currentPos[0],
            currentPos[1] + 1 ]);
        // Calculate how many unique paths we'll get by moving down.
        downUniqueSteps = btUniquePaths(width, height, steps, uniqueSteps);
        // BACKTRACK and try another move.
        steps.pop();
    }
    // Total amount of unique steps will be equal to total amount of
    // unique steps by going right plus total amount of unique steps
    // by going down.
    return rightUniqueSteps + downUniqueSteps;
}

/**
 * DYNAMIC PROGRAMMING approach of solving Unique Paths problem.
 *
 * @param width Width of the board.
 * @param height Height of the board.
 * @return Number of unique paths.
 */
function dpUniquePaths(width, height) {
    // Init board.
    var board = Array(height).fill(null).map(function () { return Array(width).fill(0); });
    // Base case.
    // There is only one way of getting to board[0][any] and
    // there is also only one way of getting to board[any][0].
    // This is because we have a restriction of moving right
    // and down only.
    for (var rowIndex = 0; rowIndex < height; rowIndex += 1) {
        for (var columnIndex = 0; columnIndex < width; columnIndex += 1) {
            if (rowIndex === 0 || columnIndex === 0) {
                board[rowIndex][columnIndex] = 1;
            }
        }
    }
    // Now, since we have this restriction of moving only to the right
    // and down we might say that number of unique paths to the current
    // cell is a sum of numbers of unique paths to the cell above the
    // current one and to the cell to the left of current one.
    for (var rowIndex = 1; rowIndex < height; rowIndex += 1) {
        for (var columnIndex = 1; columnIndex < width; columnIndex += 1) {
            var uniquesFromTop = board[rowIndex - 1][columnIndex];
            var uniquesFromLeft = board[rowIndex][columnIndex - 1];
            board[rowIndex][columnIndex] = uniquesFromTop + uniquesFromLeft;
        }
    }
    return board[height - 1][width - 1];
}

/**
 * @param width
 * @param height
 */
function uniquePaths(width, height) {
    var pascalLine = width + height - 2;
    var pascalLinePosition = Math.min(width, height) - 1;
    return pascalTriangle(pascalLine)[pascalLinePosition];
}

/**
 * 布隆过滤器
 */
var BloomFilter = /** @class */ (function () {
    /**
     * @param size the size of the storage.
     */
    function BloomFilter(size) {
        if (size === void 0) { size = 100; }
        this.size = 100;
        // Bloom filter size directly affects the likelihood of false positives.
        // The bigger the size the lower the likelihood of false positives.
        this.size = size;
        this.storage = this.createStore(size);
    }
    /**
     * 插入
     *
     * @param item 元素
     */
    BloomFilter.prototype.insert = function (item) {
        var _this = this;
        var hashValues = this.getHashValues(item);
        // Set each hashValue index to true.
        hashValues.forEach(function (val) { return _this.storage.setValue(val); });
    };
    /**
     * 可能包含
     *
     * @param item 元素
     * @return
     */
    BloomFilter.prototype.mayContain = function (item) {
        var hashValues = this.getHashValues(item);
        for (var hashIndex = 0; hashIndex < hashValues.length; hashIndex += 1) {
            if (!this.storage.getValue(hashValues[hashIndex])) {
                // We know that the item was definitely not inserted.
                return false;
            }
        }
        // The item may or may not have been inserted.
        return true;
    };
    /**
     * Creates the data store for our filter.
     * We use this method to generate the store in order to
     * encapsulate the data itself and only provide access
     * to the necessary methods.
     *
     * @param size 尺寸
     * @return
     */
    BloomFilter.prototype.createStore = function (size) {
        var storage = [];
        // Initialize all indexes to false
        for (var storageCellIndex = 0; storageCellIndex < size; storageCellIndex += 1) {
            storage.push(false);
        }
        var storageInterface = {
            getValue: function (index) {
                return storage[index];
            },
            setValue: function (index) {
                storage[index] = true;
            },
        };
        return storageInterface;
    };
    /**
     * 计算哈希值1
     *
     * @param item 元素
     * @return 哈希值1
     */
    BloomFilter.prototype.hash1 = function (item) {
        var hash = 0;
        for (var charIndex = 0; charIndex < item.length; charIndex += 1) {
            var char = item.charCodeAt(charIndex);
            hash = (hash << 5) + hash + char;
            hash &= hash; // Convert to 32bit integer
            hash = Math.abs(hash);
        }
        return hash % this.size;
    };
    /**
     * 计算哈希值2
     *
     * @param item 元素
     * @return 哈希值2
     */
    BloomFilter.prototype.hash2 = function (item) {
        var hash = 5381;
        for (var charIndex = 0; charIndex < item.length; charIndex += 1) {
            var char = item.charCodeAt(charIndex);
            hash = (hash << 5) + hash + char; /* hash * 33 + c */
        }
        return Math.abs(hash % this.size);
    };
    /**
     * 计算哈希值3
     *
     * @param item 元素
     * @return 哈希值3
     */
    BloomFilter.prototype.hash3 = function (item) {
        var hash = 0;
        for (var charIndex = 0; charIndex < item.length; charIndex += 1) {
            var char = item.charCodeAt(charIndex);
            hash = (hash << 5) - hash;
            hash += char;
            hash &= hash; // Convert to 32bit integer
        }
        return Math.abs(hash % this.size);
    };
    /**
     * Runs all 3 hash functions on the input and returns an array of results.
     * 获取3个哈希值组成的数组
     *
     * @param item 元素
     * @return 3个哈希值组成的数组
     */
    BloomFilter.prototype.getHashValues = function (item) {
        return [
            this.hash1(item),
            this.hash2(item),
            this.hash3(item) ];
    };
    return BloomFilter;
}());

/**
 * 双向链接结点
 */
var DoublyLinkedListNode = /** @class */ (function () {
    function DoublyLinkedListNode(value, next, previous) {
        if (next === void 0) { next = null; }
        if (previous === void 0) { previous = null; }
        this.value = value;
        this.next = next;
        this.previous = previous;
    }
    DoublyLinkedListNode.prototype.toString = function (callback) {
        return callback ? callback(this.value) : "" + this.value;
    };
    return DoublyLinkedListNode;
}());

/**
 * 双向链表
 */
var DoublyLinkedList = /** @class */ (function () {
    /**
     * 构建双向链表
     *
     * @param comparatorFunction 比较函数
     */
    function DoublyLinkedList(comparatorFunction) {
        this.head = null;
        this.tail = null;
        this.compare = new Comparator(comparatorFunction);
    }
    /**
     * 在头部新增结点
     *
     * @param value 新增结点值
     * @return 新增的结点
     */
    DoublyLinkedList.prototype.prepend = function (value) {
        // Make new node to be a head.
        var newNode = new DoublyLinkedListNode(value, this.head);
        // If there is head, then it won't be head anymore.
        // Therefore, make its previous reference to be new node (new head).
        // Then mark the new node as head.
        if (this.head) {
            this.head.previous = newNode;
        }
        this.head = newNode;
        // If there is no tail yet let's make new node a tail.
        if (!this.tail) {
            this.tail = newNode;
        }
        return this;
    };
    /**
     * 在尾部新增结点
     *
     * @param value 新增结点值
     * @return 新增的结点
     */
    DoublyLinkedList.prototype.append = function (value) {
        var newNode = new DoublyLinkedListNode(value);
        // If there is no head yet let's make new node a head.
        if (!this.head) {
            this.head = newNode;
            this.tail = newNode;
            return this;
        }
        // Attach new node to the end of linked list.
        this.tail.next = newNode;
        // Attach current tail to the new node's previous reference.
        newNode.previous = this.tail;
        // Set new node to be the tail of linked list.
        this.tail = newNode;
        return this;
    };
    /**
     * 删除链表中第一个与指定值相等的结点
     *
     * @param value 结点值
     * @return 删除的结点
     */
    DoublyLinkedList.prototype.delete = function (value) {
        if (!this.head) {
            return null;
        }
        var deletedNode = null;
        var currentNode = this.head;
        while (currentNode) {
            if (this.compare.equal(currentNode.value, value)) {
                deletedNode = currentNode;
                if (deletedNode === this.head) {
                    // If HEAD is going to be deleted...
                    // Set head to second node, which will become new head.
                    this.head = deletedNode.next;
                    // Set new head's previous to null.
                    if (this.head) {
                        this.head.previous = null;
                    }
                    // If all the nodes in list has same value that is passed as argument
                    // then all nodes will get deleted, therefore tail needs to be updated.
                    if (deletedNode === this.tail) {
                        this.tail = null;
                    }
                }
                else if (deletedNode === this.tail) {
                    // If TAIL is going to be deleted...
                    // Set tail to second last node, which will become new tail.
                    this.tail = deletedNode.previous;
                    this.tail.next = null;
                }
                else {
                    // If MIDDLE node is going to be deleted...
                    var previousNode = deletedNode.previous;
                    var nextNode = deletedNode.next;
                    previousNode.next = nextNode;
                    nextNode.previous = previousNode;
                }
            }
            currentNode = currentNode.next;
        }
        return deletedNode;
    };
    /**
     * 查找与结点值相等的结点
     *
     * @return 找到的第一个结点
     */
    DoublyLinkedList.prototype.find = function (_a) {
        var _b = _a.value, value = _b === void 0 ? undefined : _b, _c = _a.callback, callback = _c === void 0 ? undefined : _c;
        if (!this.head) {
            return null;
        }
        var currentNode = this.head;
        while (currentNode) {
            // If callback is specified then try to find node by callback.
            if (callback && callback(currentNode.value)) {
                return currentNode;
            }
            // If value is specified then try to compare by value..
            if (value !== undefined && this.compare.equal(currentNode.value, value)) {
                return currentNode;
            }
            currentNode = currentNode.next;
        }
        return null;
    };
    /**
     * 删除表尾
     *
     * @return 被删除的结点
     */
    DoublyLinkedList.prototype.deleteTail = function () {
        if (!this.tail) {
            // No tail to delete.
            return null;
        }
        if (this.head === this.tail) {
            // There is only one node in linked list.
            var deletedTail_1 = this.tail;
            this.head = null;
            this.tail = null;
            return deletedTail_1;
        }
        // If there are many nodes in linked list...
        var deletedTail = this.tail;
        this.tail = this.tail.previous;
        this.tail.next = null;
        return deletedTail;
    };
    /**
     * 删除表头
     *
     * @return 被删除的表头
     */
    DoublyLinkedList.prototype.deleteHead = function () {
        if (!this.head) {
            return null;
        }
        var deletedHead = this.head;
        if (this.head.next) {
            this.head = this.head.next;
            this.head.previous = null;
        }
        else {
            this.head = null;
            this.tail = null;
        }
        return deletedHead;
    };
    /**
     * 转换为数组
     *
     * @return 链表结点组成的函数
     */
    DoublyLinkedList.prototype.toArray = function () {
        var nodes = [];
        var currentNode = this.head;
        while (currentNode) {
            nodes.push(currentNode);
            currentNode = currentNode.next;
        }
        return nodes;
    };
    /**
     * 从数组中初始化链表
     *
     * @param values - Array of values that need to be converted to linked list.
     * @return 初始化后的链表
     */
    DoublyLinkedList.prototype.fromArray = function (values) {
        var _this = this;
        values.forEach(function (value) { return _this.append(value); });
        return this;
    };
    /**
     * 转换为字符串
     *
     * @param callback
     * @return 字符串
     */
    DoublyLinkedList.prototype.toString = function (callback) {
        return this.toArray().map(function (node) { return node.toString(callback); }).toString();
    };
    /**
     * Reverse a linked list.
     *
     * @returns 反转后的链表
     */
    DoublyLinkedList.prototype.reverse = function () {
        var currNode = this.head;
        var prevNode = null;
        var nextNode = null;
        while (currNode) {
            // Store next node.
            nextNode = currNode.next;
            prevNode = currNode.previous;
            // Change next node of the current node so it would link to previous node.
            currNode.next = prevNode;
            currNode.previous = nextNode;
            // Move prevNode and currNode nodes one step forward.
            prevNode = currNode;
            currNode = nextNode;
        }
        // Reset head and tail.
        this.tail = this.head;
        this.head = prevNode;
        return this;
    };
    return DoublyLinkedList;
}());

/**
 * 图边
 */
var GraphEdge = /** @class */ (function () {
    /**
     * 构建图边
     * @param startVertex 起始顶点
     * @param endVertex 结束顶点
     * @param weight 权重
     */
    function GraphEdge(startVertex, endVertex, weight) {
        if (weight === void 0) { weight = 0; }
        this.startVertex = startVertex;
        this.endVertex = endVertex;
        this.weight = weight;
    }
    /**
     * 获取键值
     */
    GraphEdge.prototype.getKey = function () {
        var startVertexKey = this.startVertex.getKey();
        var endVertexKey = this.endVertex.getKey();
        return startVertexKey + "_" + endVertexKey;
    };
    /**
     * 反转
     */
    GraphEdge.prototype.reverse = function () {
        var tmp = this.startVertex;
        this.startVertex = this.endVertex;
        this.endVertex = tmp;
        return this;
    };
    /**
     * 转换为字符串
     */
    GraphEdge.prototype.toString = function () {
        return this.getKey();
    };
    return GraphEdge;
}());

/**
 * Hash table size directly affects on the number of collisions.
 * The bigger the hash table size the less collisions you'll get.
 * For demonstrating purposes hash table size is small to show how collisions
 * are being handled.
 */
var defaultHashTableSize = 32;
/**
 * 哈希表（散列表）
 */
var HashTable = /** @class */ (function () {
    /**
     * 构建哈希表
     *
     * @param hashTableSize 哈希表尺寸
     */
    function HashTable(hashTableSize) {
        if (hashTableSize === void 0) { hashTableSize = defaultHashTableSize; }
        // Create hash table of certain size and fill each bucket with empty linked list.
        this.buckets = Array(hashTableSize).fill(null).map(function () { return new LinkedList(); });
        // Just to keep track of all actual keys in a fast way.
        this.keys = {};
    }
    /**
     * Converts key string to hash number.
     * 将字符串键转换为哈希数。
     *
     * @param key 字符串键
     * @return 哈希数
     */
    HashTable.prototype.hash = function (key) {
        // For simplicity reasons we will just use character codes sum of all characters of the key
        // to calculate the hash.
        //
        // But you may also use more sophisticated approaches like polynomial string hash to reduce the
        // number of collisions:
        //
        // hash = charCodeAt(0) * PRIME^(n-1) + charCodeAt(1) * PRIME^(n-2) + ... + charCodeAt(n-1)
        //
        // where charCodeAt(i) is the i-th character code of the key, n is the length of the key and
        // PRIME is just any prime number like 31.
        var hash = Array.from(key).reduce(function (hashAccumulator, keySymbol) { return (hashAccumulator + keySymbol.charCodeAt(0)); }, 0);
        // Reduce hash number so it would fit hash table size.
        return hash % this.buckets.length;
    };
    /**
     * 设置值
     *
     * @param key 键
     * @param value 值
     */
    HashTable.prototype.set = function (key, value) {
        var keyHash = this.hash(key);
        this.keys[key] = keyHash;
        var bucketLinkedList = this.buckets[keyHash];
        var node = bucketLinkedList.find({ callback: function (nodeValue) { return nodeValue.key === key; } });
        if (!node) {
            // Insert new node.
            bucketLinkedList.append({ key: key, value: value });
        }
        else {
            // Update value of existing node.
            node.value.value = value;
        }
    };
    /**
     * 删除指定键以及对于值
     *
     * @param key 需要删除的键
     * @return 被删除的结点
     */
    HashTable.prototype.delete = function (key) {
        var keyHash = this.hash(key);
        delete this.keys[key];
        var bucketLinkedList = this.buckets[keyHash];
        var node = bucketLinkedList.find({ callback: function (nodeValue) { return nodeValue.key === key; } });
        if (node) {
            return bucketLinkedList.delete(node.value);
        }
        return null;
    };
    /**
     * 获取与键对应的值
     *
     * @param key 键
     * @return
     */
    HashTable.prototype.get = function (key) {
        var bucketLinkedList = this.buckets[this.hash(key)];
        var node = bucketLinkedList.find({ callback: function (nodeValue) { return nodeValue.key === key; } });
        return node ? node.value.value : undefined;
    };
    /**
     * 是否拥有键
     *
     * @param key 键
     * @return 是否拥有键
     */
    HashTable.prototype.has = function (key) {
        return Object.hasOwnProperty.call(this.keys, key);
    };
    /**
     * 获取键列表
     *
     * @return 键列表
     */
    HashTable.prototype.getKeys = function () {
        return Object.keys(this.keys);
    };
    /**
     * Gets the list of all the stored values in the hash table.
     *
     * @return 值列表
     */
    HashTable.prototype.getValues = function () {
        return this.buckets.reduce(function (values, bucket) {
            var bucketValues = bucket.toArray()
                .map(function (linkedListNode) { return linkedListNode.value.value; });
            return values.concat(bucketValues);
        }, []);
    };
    return HashTable;
}());

/**
 * 最大堆
 *
 * 所有父结点都大于子结点
 */
var MaxHeap = /** @class */ (function (_super) {
    __extends(MaxHeap, _super);
    function MaxHeap() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    /**
     * Checks if pair of heap elements is in correct order.
     * For MinHeap the first element must be always smaller or equal.
     * For MaxHeap the first element must be always bigger or equal.
     *
     * 检查堆元素对的顺序是否正确。
     * 对于MinHeap，第一个元素必须总是小于等于。
     * 对于MaxHeap，第一个元素必须总是大于或等于。
     *
     * @param firstElement 第一个元素
     * @param secondElement 第二个元素
     * @return 堆元素对的顺序是否正确
     */
    MaxHeap.prototype.pairIsInCorrectOrder = function (firstElement, secondElement) {
        return this.compare.greaterThanOrEqual(firstElement, secondElement);
    };
    return MaxHeap;
}(Heap));

var BinaryTreeNode = /** @class */ (function () {
    /**
     * 构建二叉树结点
     *
     * @param value node value.
     */
    function BinaryTreeNode(value) {
        if (value === void 0) { value = null; }
        this.left = null;
        this.right = null;
        this.parent = null;
        this.value = value;
        // Any node related meta information may be stored here.
        this.meta = new HashTable();
        // This comparator is used to compare binary tree nodes with each other.
        this.nodeComparator = new Comparator();
    }
    Object.defineProperty(BinaryTreeNode.prototype, "leftHeight", {
        /**
         * 左结点高度
         */
        get: function () {
            if (!this.left) {
                return 0;
            }
            return this.left.height + 1;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(BinaryTreeNode.prototype, "rightHeight", {
        /**
         * 右结点高度
         */
        get: function () {
            if (!this.right) {
                return 0;
            }
            return this.right.height + 1;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(BinaryTreeNode.prototype, "height", {
        /**
         * 高度
         */
        get: function () {
            return Math.max(this.leftHeight, this.rightHeight);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(BinaryTreeNode.prototype, "balanceFactor", {
        /**
         * 平衡系数
         */
        get: function () {
            return this.leftHeight - this.rightHeight;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(BinaryTreeNode.prototype, "uncle", {
        /**
         * Get parent's sibling if it exists.
         *
         * @return 叔伯结点
         */
        get: function () {
            // Check if current node has parent.
            if (!this.parent) {
                return undefined;
            }
            // Check if current node has grand-parent.
            if (!this.parent.parent) {
                return undefined;
            }
            // Check if grand-parent has two children.
            if (!this.parent.parent.left || !this.parent.parent.right) {
                return undefined;
            }
            // So for now we know that current node has grand-parent and this
            // grand-parent has two children. Let's find out who is the uncle.
            if (this.nodeComparator.equal(this.parent, this.parent.parent.left)) {
                // Right one is an uncle.
                return this.parent.parent.right;
            }
            // Left one is an uncle.
            return this.parent.parent.left;
        },
        enumerable: false,
        configurable: true
    });
    /**
     * 设置结点值
     *
     * @param value 结点新值
     */
    BinaryTreeNode.prototype.setValue = function (value) {
        this.value = value;
        return this;
    };
    /**
     * 设置左结点
     *
     * @param node 结点
     */
    BinaryTreeNode.prototype.setLeft = function (node) {
        // Reset parent for left node since it is going to be detached.
        if (this.left) {
            this.left.parent = null;
        }
        // Attach new node to the left.
        this.left = node;
        // Make current node to be a parent for new left one.
        if (this.left) {
            this.left.parent = this;
        }
        return this;
    };
    /**
     * 设置右结点
     *
     * @param node 结点
     */
    BinaryTreeNode.prototype.setRight = function (node) {
        // Reset parent for right node since it is going to be detached.
        if (this.right) {
            this.right.parent = null;
        }
        // Attach new node to the right.
        this.right = node;
        // Make current node to be a parent for new right one.
        if (node) {
            this.right.parent = this;
        }
        return this;
    };
    /**
     * 移除子结点
     *
     * @param nodeToRemove 需要移除的子结点
     * @return 是否成功移除
     */
    BinaryTreeNode.prototype.removeChild = function (nodeToRemove) {
        if (this.left && this.nodeComparator.equal(this.left, nodeToRemove)) {
            this.left = null;
            return true;
        }
        if (this.right && this.nodeComparator.equal(this.right, nodeToRemove)) {
            this.right = null;
            return true;
        }
        return false;
    };
    /**
     * 替换节点
     *
     * @param nodeToReplace 被替换的节点
     * @param replacementNode 替换后的节点
     * @return 是否替换成功
     */
    BinaryTreeNode.prototype.replaceChild = function (nodeToReplace, replacementNode) {
        if (!nodeToReplace || !replacementNode) {
            return false;
        }
        if (this.left && this.nodeComparator.equal(this.left, nodeToReplace)) {
            this.left = replacementNode;
            return true;
        }
        if (this.right && this.nodeComparator.equal(this.right, nodeToReplace)) {
            this.right = replacementNode;
            return true;
        }
        return false;
    };
    /**
     * 拷贝节点
     *
     * @param sourceNode 源节点
     * @param targetNode 目标节点
     */
    BinaryTreeNode.copyNode = function (sourceNode, targetNode) {
        targetNode.setValue(sourceNode.value);
        targetNode.setLeft(sourceNode.left);
        targetNode.setRight(sourceNode.right);
    };
    /**
     * 左序深度遍历
     */
    BinaryTreeNode.prototype.traverseInOrder = function () {
        var traverse = [];
        // Add left node.
        if (this.left) {
            traverse = traverse.concat(this.left.traverseInOrder());
        }
        // Add root.
        traverse.push(this.value);
        // Add right node.
        if (this.right) {
            traverse = traverse.concat(this.right.traverseInOrder());
        }
        return traverse;
    };
    /**
     * 转换为字符串
     */
    BinaryTreeNode.prototype.toString = function () {
        return this.traverseInOrder().toString();
    };
    return BinaryTreeNode;
}());

/**
 * 二叉查找树结点
 */
var BinarySearchTreeNode = /** @class */ (function (_super) {
    __extends(BinarySearchTreeNode, _super);
    /**
     * 构建二叉查找树结点
     *
     * @param value node value.
     * @param compareFunction comparator function for node values.
     */
    function BinarySearchTreeNode(value, compareFunction) {
        if (value === void 0) { value = null; }
        if (compareFunction === void 0) { compareFunction = undefined; }
        var _this = _super.call(this, value) || this;
        /**
         * 左结点
         */
        _this.left = null;
        /**
         * 右结点
         */
        _this.right = null;
        /**
         * 父结点
         */
        _this.parent = null;
        // This comparator is used to compare node values with each other.
        _this.compareFunction = compareFunction;
        _this.nodeValueComparator = new Comparator(compareFunction);
        return _this;
    }
    /**
     * 插入值
     *
     * @param value 值
     */
    BinarySearchTreeNode.prototype.insert = function (value) {
        if (this.nodeValueComparator.equal(this.value, null)) {
            this.value = value;
            return this;
        }
        if (this.nodeValueComparator.lessThan(value, this.value)) {
            // Insert to the left.
            if (this.left) {
                return this.left.insert(value);
            }
            var newNode = new BinarySearchTreeNode(value, this.compareFunction);
            this.setLeft(newNode);
            return newNode;
        }
        if (this.nodeValueComparator.greaterThan(value, this.value)) {
            // Insert to the right.
            if (this.right) {
                return this.right.insert(value);
            }
            var newNode = new BinarySearchTreeNode(value, this.compareFunction);
            this.setRight(newNode);
            return newNode;
        }
        return this;
    };
    /**
     * 查找结点
     *
     * @param value 值
     * @return {BinarySearchTreeNode}
     */
    BinarySearchTreeNode.prototype.find = function (value) {
        // Check the root.
        if (this.nodeValueComparator.equal(this.value, value)) {
            return this;
        }
        if (this.nodeValueComparator.lessThan(value, this.value) && this.left) {
            // Check left nodes.
            return this.left.find(value);
        }
        if (this.nodeValueComparator.greaterThan(value, this.value) && this.right) {
            // Check right nodes.
            return this.right.find(value);
        }
        return null;
    };
    /**
     * 是否包含指定值
     *
     * @param value 指定值
     */
    BinarySearchTreeNode.prototype.contains = function (value) {
        return !!this.find(value);
    };
    /**
     * 移除指定值
     *
     * @param  value 指定值
     * @return 是否移除成功
     */
    BinarySearchTreeNode.prototype.remove = function (value) {
        var nodeToRemove = this.find(value);
        if (!nodeToRemove) {
            throw new Error('Item not found in the tree');
        }
        var parent = nodeToRemove.parent;
        if (!nodeToRemove.left && !nodeToRemove.right) {
            // Node is a leaf and thus has no children.
            if (parent) {
                // Node has a parent. Just remove the pointer to this node from the parent.
                parent.removeChild(nodeToRemove);
            }
            else {
                // Node has no parent. Just erase current node value.
                nodeToRemove.setValue(undefined);
            }
        }
        else if (nodeToRemove.left && nodeToRemove.right) {
            // Node has two children.
            // Find the next biggest value (minimum value in the right branch)
            // and replace current value node with that next biggest value.
            var nextBiggerNode = nodeToRemove.right.findMin();
            if (!this.nodeComparator.equal(nextBiggerNode, nodeToRemove.right)) {
                this.remove(nextBiggerNode.value);
                nodeToRemove.setValue(nextBiggerNode.value);
            }
            else {
                // In case if next right value is the next bigger one and it doesn't have left child
                // then just replace node that is going to be deleted with the right node.
                nodeToRemove.setValue(nodeToRemove.right.value);
                nodeToRemove.setRight(nodeToRemove.right.right);
            }
        }
        else {
            // Node has only one child.
            // Make this child to be a direct child of current node's parent.
            /** @var BinarySearchTreeNode */
            var childNode = nodeToRemove.left || nodeToRemove.right;
            if (parent) {
                parent.replaceChild(nodeToRemove, childNode);
            }
            else {
                BinaryTreeNode.copyNode(childNode, nodeToRemove);
            }
        }
        // Clear the parent of removed node.
        nodeToRemove.parent = null;
        return true;
    };
    /**
     * 查找最小值
     */
    BinarySearchTreeNode.prototype.findMin = function () {
        if (!this.left) {
            return this;
        }
        return this.left.findMin();
    };
    return BinarySearchTreeNode;
}(BinaryTreeNode));

/**
 * 二叉查找树
 *
 * 二叉查找树（英语：Binary Search Tree），也称为二叉搜索树、有序二叉树（ordered binary tree）或排序二叉树（sorted binary tree），是指一棵空树或者具有下列性质的二叉树：
 *
 * 1. 若任意节点的左子树不空，则左子树上所有节点的值均小于它的根节点的值；
 * 2. 若任意节点的右子树不空，则右子树上所有节点的值均大于它的根节点的值；
 * 3. 任意节点的左、右子树也分别为二叉查找树；
 * 4. 没有键值相等的节点。
 */
var BinarySearchTree = /** @class */ (function () {
    /**
     * 构建 二叉查找树
     *
     * @param nodeValueCompareFunction 结点值比较器
     */
    function BinarySearchTree(nodeValueCompareFunction) {
        this.root = new BinarySearchTreeNode(null, nodeValueCompareFunction);
        // Steal node comparator from the root.
        this.nodeComparator = this.root.nodeComparator;
    }
    /**
     * 插入值
     *
     * @param value 值
     */
    BinarySearchTree.prototype.insert = function (value) {
        return this.root.insert(value);
    };
    /**
     * 是否包含指定值
     *
     * @param value 值
     */
    BinarySearchTree.prototype.contains = function (value) {
        return this.root.contains(value);
    };
    /**
     * 移除指定值
     *
     * @param value 值
     */
    BinarySearchTree.prototype.remove = function (value) {
        return this.root.remove(value);
    };
    /**
     * 转换为字符串
     */
    BinarySearchTree.prototype.toString = function () {
        return this.root.toString();
    };
    return BinarySearchTree;
}());

var AvlTree = /** @class */ (function (_super) {
    __extends(AvlTree, _super);
    function AvlTree() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    /**
     * @param value
     */
    AvlTree.prototype.insert = function (value) {
        // Do the normal BST insert.
        var newNode = _super.prototype.insert.call(this, value);
        // Let's move up to the root and check balance factors along the way.
        var currentNode = this.root.find(value);
        while (currentNode) {
            this.balance(currentNode);
            currentNode = currentNode.parent;
        }
        return newNode;
    };
    /**
     * @param value
     */
    AvlTree.prototype.remove = function (value) {
        // Do standard BST removal.
        var result = _super.prototype.remove.call(this, value);
        // Balance the tree starting from the root node.
        this.balance(this.root);
        return result;
    };
    /**
     * 平衡结点
     *
     * @param node 结点
     */
    AvlTree.prototype.balance = function (node) {
        // If balance factor is not OK then try to balance the node.
        if (node.balanceFactor > 1) {
            // Left rotation.
            if (node.left.balanceFactor > 0) {
                // Left-Left rotation
                this.rotateLeftLeft(node);
            }
            else if (node.left.balanceFactor < 0) {
                // Left-Right rotation.
                this.rotateLeftRight(node);
            }
        }
        else if (node.balanceFactor < -1) {
            // Right rotation.
            if (node.right.balanceFactor < 0) {
                // Right-Right rotation
                this.rotateRightRight(node);
            }
            else if (node.right.balanceFactor > 0) {
                // Right-Left rotation.
                this.rotateRightLeft(node);
            }
        }
    };
    /**
     * @param  rootNode
     */
    AvlTree.prototype.rotateLeftLeft = function (rootNode) {
        // Detach left node from root node.
        var leftNode = rootNode.left;
        rootNode.setLeft(null);
        // Make left node to be a child of rootNode's parent.
        if (rootNode.parent) {
            rootNode.parent.setLeft(leftNode);
        }
        else if (rootNode === this.root) {
            // If root node is root then make left node to be a new root.
            this.root = leftNode;
        }
        // If left node has a right child then detach it and
        // attach it as a left child for rootNode.
        if (leftNode.right) {
            rootNode.setLeft(leftNode.right);
        }
        // Attach rootNode to the right of leftNode.
        leftNode.setRight(rootNode);
    };
    /**
     * @param  rootNode
     */
    AvlTree.prototype.rotateLeftRight = function (rootNode) {
        // Detach left node from rootNode since it is going to be replaced.
        var leftNode = rootNode.left;
        rootNode.setLeft(null);
        // Detach right node from leftNode.
        var leftRightNode = leftNode.right;
        leftNode.setRight(null);
        // Preserve leftRightNode's left subtree.
        if (leftRightNode.left) {
            leftNode.setRight(leftRightNode.left);
            leftRightNode.setLeft(null);
        }
        // Attach leftRightNode to the rootNode.
        rootNode.setLeft(leftRightNode);
        // Attach leftNode as left node for leftRight node.
        leftRightNode.setLeft(leftNode);
        // Do left-left rotation.
        this.rotateLeftLeft(rootNode);
    };
    /**
     * @param  rootNode
     */
    AvlTree.prototype.rotateRightLeft = function (rootNode) {
        // Detach right node from rootNode since it is going to be replaced.
        var rightNode = rootNode.right;
        rootNode.setRight(null);
        // Detach left node from rightNode.
        var rightLeftNode = rightNode.left;
        rightNode.setLeft(null);
        if (rightLeftNode.right) {
            rightNode.setLeft(rightLeftNode.right);
            rightLeftNode.setRight(null);
        }
        // Attach rightLeftNode to the rootNode.
        rootNode.setRight(rightLeftNode);
        // Attach rightNode as right node for rightLeft node.
        rightLeftNode.setRight(rightNode);
        // Do right-right rotation.
        this.rotateRightRight(rootNode);
    };
    /**
     * @param rootNode
     */
    AvlTree.prototype.rotateRightRight = function (rootNode) {
        // Detach right node from root node.
        var rightNode = rootNode.right;
        rootNode.setRight(null);
        // Make right node to be a child of rootNode's parent.
        if (rootNode.parent) {
            rootNode.parent.setRight(rightNode);
        }
        else if (rootNode === this.root) {
            // If root node is root then make right node to be a new root.
            this.root = rightNode;
        }
        // If right node has a left child then detach it and
        // attach it as a right child for rootNode.
        if (rightNode.left) {
            rootNode.setRight(rightNode.left);
        }
        // Attach rootNode to the left of rightNode.
        rightNode.setLeft(rootNode);
    };
    return AvlTree;
}(BinarySearchTree));

/**
 * 树状数组（Fenwick树、Binary Indexed Tree）
 *
 * @see https://zh.wikipedia.org/wiki/%E6%A0%91%E7%8A%B6%E6%95%B0%E7%BB%84
 */
var FenwickTree = /** @class */ (function () {
    /**
     * Constructor creates empty fenwick tree of size 'arraySize',
     * however, array size is size+1, because index is 1-based.
     *
     * @param arraySize
     */
    function FenwickTree(arraySize) {
        this.arraySize = arraySize;
        // Fill tree array with zeros.
        this.treeArray = Array(this.arraySize + 1).fill(0);
    }
    /**
     * Adds value to existing value at position.
     *
     * @param position
     * @param value
     */
    FenwickTree.prototype.increase = function (position, value) {
        if (position < 1 || position > this.arraySize) {
            throw new Error('Position is out of allowed range');
        }
        for (var i = position; i <= this.arraySize; i += (i & -i)) {
            this.treeArray[i] += value;
        }
        return this;
    };
    /**
     * Query sum from index 1 to position.
     *
     * @param position
     */
    FenwickTree.prototype.query = function (position) {
        if (position < 1 || position > this.arraySize) {
            throw new Error('Position is out of allowed range');
        }
        var sum = 0;
        for (var i = position; i > 0; i -= (i & -i)) {
            sum += this.treeArray[i];
        }
        return sum;
    };
    /**
     * Query sum from index leftIndex to rightIndex.
     *
     * @param leftIndex
     * @param rightIndex
     */
    FenwickTree.prototype.queryRange = function (leftIndex, rightIndex) {
        if (leftIndex > rightIndex) {
            throw new Error('Left index can not be greater than right one');
        }
        if (leftIndex === 1) {
            return this.query(rightIndex);
        }
        return this.query(rightIndex) - this.query(leftIndex - 1);
    };
    return FenwickTree;
}());

// Possible colors of red-black tree nodes.
var RED_BLACK_TREE_COLORS = {
    red: 'red',
    black: 'black',
};
// Color property name in meta information of the nodes.
var COLOR_PROP_NAME = 'color';
/**
 * 红黑树
 */
var RedBlackTree = /** @class */ (function (_super) {
    __extends(RedBlackTree, _super);
    function RedBlackTree() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    /**
     * @param value
     */
    RedBlackTree.prototype.insert = function (value) {
        var insertedNode = _super.prototype.insert.call(this, value);
        // if (!this.root.left && !this.root.right) {
        if (this.nodeComparator.equal(insertedNode, this.root)) {
            // Make root to always be black.
            this.makeNodeBlack(insertedNode);
        }
        else {
            // Make all newly inserted nodes to be red.
            this.makeNodeRed(insertedNode);
        }
        // Check all conditions and balance the node.
        this.balance(insertedNode);
        return insertedNode;
    };
    /**
     * @param value
     */
    RedBlackTree.prototype.remove = function (value) {
        throw new Error("Can't remove " + value + ". Remove method is not implemented yet");
    };
    /**
     * @param node
     */
    RedBlackTree.prototype.balance = function (node) {
        // If it is a root node then nothing to balance here.
        if (this.nodeComparator.equal(node, this.root)) {
            return;
        }
        // If the parent is black then done. Nothing to balance here.
        if (this.isNodeBlack(node.parent)) {
            return;
        }
        var grandParent = node.parent.parent;
        if (node.uncle && this.isNodeRed(node.uncle)) {
            // If node has red uncle then we need to do RECOLORING.
            // Recolor parent and uncle to black.
            this.makeNodeBlack(node.uncle);
            this.makeNodeBlack(node.parent);
            if (!this.nodeComparator.equal(grandParent, this.root)) {
                // Recolor grand-parent to red if it is not root.
                this.makeNodeRed(grandParent);
            }
            else {
                // If grand-parent is black root don't do anything.
                // Since root already has two black sibling that we've just recolored.
                return;
            }
            // Now do further checking for recolored grand-parent.
            this.balance(grandParent);
        }
        else if (!node.uncle || this.isNodeBlack(node.uncle)) {
            // If node uncle is black or absent then we need to do ROTATIONS.
            if (grandParent) {
                // Grand parent that we will receive after rotations.
                var newGrandParent = void 0;
                if (this.nodeComparator.equal(grandParent.left, node.parent)) {
                    // Left case.
                    if (this.nodeComparator.equal(node.parent.left, node)) {
                        // Left-left case.
                        newGrandParent = this.leftLeftRotation(grandParent);
                    }
                    else {
                        // Left-right case.
                        newGrandParent = this.leftRightRotation(grandParent);
                    }
                }
                else {
                    // Right case.
                    if (this.nodeComparator.equal(node.parent.right, node)) {
                        // Right-right case.
                        newGrandParent = this.rightRightRotation(grandParent);
                    }
                    else {
                        // Right-left case.
                        newGrandParent = this.rightLeftRotation(grandParent);
                    }
                }
                // Set newGrandParent as a root if it doesn't have parent.
                if (newGrandParent && newGrandParent.parent === null) {
                    this.root = newGrandParent;
                    // Recolor root into black.
                    this.makeNodeBlack(this.root);
                }
                // Check if new grand parent don't violate red-black-tree rules.
                this.balance(newGrandParent);
            }
        }
    };
    /**
     * Left Left Case (p is left child of g and x is left child of p)
     * @param grandParentNode
     */
    RedBlackTree.prototype.leftLeftRotation = function (grandParentNode) {
        // Memorize the parent of grand-parent node.
        var grandGrandParent = grandParentNode.parent;
        // Check what type of sibling is our grandParentNode is (left or right).
        var grandParentNodeIsLeft;
        if (grandGrandParent) {
            grandParentNodeIsLeft = this.nodeComparator.equal(grandGrandParent.left, grandParentNode);
        }
        // Memorize grandParentNode's left node.
        var parentNode = grandParentNode.left;
        // Memorize parent's right node since we're going to transfer it to
        // grand parent's left subtree.
        var parentRightNode = parentNode.right;
        // Make grandParentNode to be right child of parentNode.
        parentNode.setRight(grandParentNode);
        // Move child's right subtree to grandParentNode's left subtree.
        grandParentNode.setLeft(parentRightNode);
        // Put parentNode node in place of grandParentNode.
        if (grandGrandParent) {
            if (grandParentNodeIsLeft) {
                grandGrandParent.setLeft(parentNode);
            }
            else {
                grandGrandParent.setRight(parentNode);
            }
        }
        else {
            // Make parent node a root
            parentNode.parent = null;
        }
        // Swap colors of grandParentNode and parentNode.
        this.swapNodeColors(parentNode, grandParentNode);
        // Return new root node.
        return parentNode;
    };
    /**
     * Left Right Case (p is left child of g and x is right child of p)
     *
     * @param grandParentNode
     */
    RedBlackTree.prototype.leftRightRotation = function (grandParentNode) {
        // Memorize left and left-right nodes.
        var parentNode = grandParentNode.left;
        var childNode = parentNode.right;
        // We need to memorize child left node to prevent losing
        // left child subtree. Later it will be re-assigned to
        // parent's right sub-tree.
        var childLeftNode = childNode.left;
        // Make parentNode to be a left child of childNode node.
        childNode.setLeft(parentNode);
        // Move child's left subtree to parent's right subtree.
        parentNode.setRight(childLeftNode);
        // Put left-right node in place of left node.
        grandParentNode.setLeft(childNode);
        // Now we're ready to do left-left rotation.
        return this.leftLeftRotation(grandParentNode);
    };
    /**
     * Right Right Case (p is right child of g and x is right child of p)
     *
     * @param grandParentNode
     */
    RedBlackTree.prototype.rightRightRotation = function (grandParentNode) {
        // Memorize the parent of grand-parent node.
        var grandGrandParent = grandParentNode.parent;
        // Check what type of sibling is our grandParentNode is (left or right).
        var grandParentNodeIsLeft;
        if (grandGrandParent) {
            grandParentNodeIsLeft = this.nodeComparator.equal(grandGrandParent.left, grandParentNode);
        }
        // Memorize grandParentNode's right node.
        var parentNode = grandParentNode.right;
        // Memorize parent's left node since we're going to transfer it to
        // grand parent's right subtree.
        var parentLeftNode = parentNode.left;
        // Make grandParentNode to be left child of parentNode.
        parentNode.setLeft(grandParentNode);
        // Transfer all left nodes from parent to right sub-tree of grandparent.
        grandParentNode.setRight(parentLeftNode);
        // Put parentNode node in place of grandParentNode.
        if (grandGrandParent) {
            if (grandParentNodeIsLeft) {
                grandGrandParent.setLeft(parentNode);
            }
            else {
                grandGrandParent.setRight(parentNode);
            }
        }
        else {
            // Make parent node a root.
            parentNode.parent = null;
        }
        // Swap colors of granParent and parent nodes.
        this.swapNodeColors(parentNode, grandParentNode);
        // Return new root node.
        return parentNode;
    };
    /**
     * Right Left Case (p is right child of g and x is left child of p)
     *
     * @param grandParentNode
     */
    RedBlackTree.prototype.rightLeftRotation = function (grandParentNode) {
        // Memorize right and right-left nodes.
        var parentNode = grandParentNode.right;
        var childNode = parentNode.left;
        // We need to memorize child right node to prevent losing
        // right child subtree. Later it will be re-assigned to
        // parent's left sub-tree.
        var childRightNode = childNode.right;
        // Make parentNode to be a right child of childNode.
        childNode.setRight(parentNode);
        // Move child's right subtree to parent's left subtree.
        parentNode.setLeft(childRightNode);
        // Put childNode node in place of parentNode.
        grandParentNode.setRight(childNode);
        // Now we're ready to do right-right rotation.
        return this.rightRightRotation(grandParentNode);
    };
    /**
     * @param node
     */
    RedBlackTree.prototype.makeNodeRed = function (node) {
        node.meta.set(COLOR_PROP_NAME, RED_BLACK_TREE_COLORS.red);
        return node;
    };
    /**
     * @param node
     */
    RedBlackTree.prototype.makeNodeBlack = function (node) {
        node.meta.set(COLOR_PROP_NAME, RED_BLACK_TREE_COLORS.black);
        return node;
    };
    /**
     * @param node
     */
    RedBlackTree.prototype.isNodeRed = function (node) {
        return node.meta.get(COLOR_PROP_NAME) === RED_BLACK_TREE_COLORS.red;
    };
    /**
     * @param node
     */
    RedBlackTree.prototype.isNodeBlack = function (node) {
        return node.meta.get(COLOR_PROP_NAME) === RED_BLACK_TREE_COLORS.black;
    };
    /**
     * @param node
     */
    RedBlackTree.prototype.isNodeColored = function (node) {
        return this.isNodeRed(node) || this.isNodeBlack(node);
    };
    /**
     * @param firstNode
     * @param secondNode
     */
    RedBlackTree.prototype.swapNodeColors = function (firstNode, secondNode) {
        var firstColor = firstNode.meta.get(COLOR_PROP_NAME);
        var secondColor = secondNode.meta.get(COLOR_PROP_NAME);
        firstNode.meta.set(COLOR_PROP_NAME, secondColor);
        secondNode.meta.set(COLOR_PROP_NAME, firstColor);
    };
    return RedBlackTree;
}(BinarySearchTree));

/**
 * 线段树
 */
var SegmentTree = /** @class */ (function () {
    /**
     * @param inputArray
     * @param operation binary function (i.e. sum, min)
     * @param operationFallback operation fallback value (i.e. 0 for sum, Infinity for min)
     */
    function SegmentTree(inputArray, operation, operationFallback) {
        this.inputArray = inputArray;
        this.operation = operation;
        this.operationFallback = operationFallback;
        // Init array representation of segment tree.
        this.segmentTree = this.initSegmentTree(this.inputArray);
        this.buildSegmentTree();
    }
    /**
     *
     * @param inputArray
     */
    SegmentTree.prototype.initSegmentTree = function (inputArray) {
        var segmentTreeArrayLength;
        var inputArrayLength = inputArray.length;
        if (isPowerOfTwo1(inputArrayLength)) {
            // If original array length is a power of two.
            segmentTreeArrayLength = (2 * inputArrayLength) - 1;
        }
        else {
            // If original array length is not a power of two then we need to find
            // next number that is a power of two and use it to calculate
            // tree array size. This is happens because we need to fill empty children
            // in perfect binary tree with nulls.And those nulls need extra space.
            var currentPower = Math.floor(Math.log2(inputArrayLength));
            var nextPower = currentPower + 1;
            var nextPowerOfTwoNumber = Math.pow(2, nextPower);
            segmentTreeArrayLength = (2 * nextPowerOfTwoNumber) - 1;
        }
        return new Array(segmentTreeArrayLength).fill(null);
    };
    /**
     * Build segment tree.
     */
    SegmentTree.prototype.buildSegmentTree = function () {
        var leftIndex = 0;
        var rightIndex = this.inputArray.length - 1;
        var position = 0;
        this.buildTreeRecursively(leftIndex, rightIndex, position);
    };
    /**
     * Build segment tree recursively.
     *
     * @param leftInputIndex
     * @param rightInputIndex
     * @param position
     */
    SegmentTree.prototype.buildTreeRecursively = function (leftInputIndex, rightInputIndex, position) {
        // If low input index and high input index are equal that would mean
        // the we have finished splitting and we are already came to the leaf
        // of the segment tree. We need to copy this leaf value from input
        // array to segment tree.
        if (leftInputIndex === rightInputIndex) {
            this.segmentTree[position] = this.inputArray[leftInputIndex];
            return;
        }
        // Split input array on two halves and process them recursively.
        var middleIndex = Math.floor((leftInputIndex + rightInputIndex) / 2);
        // Process left half of the input array.
        this.buildTreeRecursively(leftInputIndex, middleIndex, this.getLeftChildIndex(position));
        // Process right half of the input array.
        this.buildTreeRecursively(middleIndex + 1, rightInputIndex, this.getRightChildIndex(position));
        // Once every tree leaf is not empty we're able to build tree bottom up using
        // provided operation function.
        this.segmentTree[position] = this.operation(this.segmentTree[this.getLeftChildIndex(position)], this.segmentTree[this.getRightChildIndex(position)]);
    };
    /**
     * Do range query on segment tree in context of this.operation function.
     *
     * @param queryLeftIndex
     * @param queryRightIndex
     */
    SegmentTree.prototype.rangeQuery = function (queryLeftIndex, queryRightIndex) {
        var leftIndex = 0;
        var rightIndex = this.inputArray.length - 1;
        var position = 0;
        return this.rangeQueryRecursive(queryLeftIndex, queryRightIndex, leftIndex, rightIndex, position);
    };
    /**
     * Do range query on segment tree recursively in context of this.operation function.
     *
     * @param queryLeftIndex left index of the query
     * @param queryRightIndex right index of the query
     * @param leftIndex left index of input array segment
     * @param rightIndex right index of input array segment
     * @param position root position in binary tree
     */
    SegmentTree.prototype.rangeQueryRecursive = function (queryLeftIndex, queryRightIndex, leftIndex, rightIndex, position) {
        if (queryLeftIndex <= leftIndex && queryRightIndex >= rightIndex) {
            // Total overlap.
            return this.segmentTree[position];
        }
        if (queryLeftIndex > rightIndex || queryRightIndex < leftIndex) {
            // No overlap.
            return this.operationFallback;
        }
        // Partial overlap.
        var middleIndex = Math.floor((leftIndex + rightIndex) / 2);
        var leftOperationResult = this.rangeQueryRecursive(queryLeftIndex, queryRightIndex, leftIndex, middleIndex, this.getLeftChildIndex(position));
        var rightOperationResult = this.rangeQueryRecursive(queryLeftIndex, queryRightIndex, middleIndex + 1, rightIndex, this.getRightChildIndex(position));
        return this.operation(leftOperationResult, rightOperationResult);
    };
    /**
     * Left child index.
     *
     * @param parentIndex
     */
    SegmentTree.prototype.getLeftChildIndex = function (parentIndex) {
        return (2 * parentIndex) + 1;
    };
    /**
     * Right child index.
     *
     * @param parentIndex
     */
    SegmentTree.prototype.getRightChildIndex = function (parentIndex) {
        return (2 * parentIndex) + 2;
    };
    return SegmentTree;
}());

var TrieNode = /** @class */ (function () {
    /**
     * @param character 字符
     * @param isCompleteWord 是否完整单词
     */
    function TrieNode(character, isCompleteWord) {
        if (isCompleteWord === void 0) { isCompleteWord = false; }
        this.character = character;
        this.isCompleteWord = isCompleteWord;
        this.children = new HashTable();
    }
    /**
     * 获取子结点
     *
     * @param character 字符
     */
    TrieNode.prototype.getChild = function (character) {
        return this.children.get(character);
    };
    /**
     * 添加子结点
     *
     * @param character 字符
     * @param isCompleteWord 字符完整单词
     * @return 自身
     */
    TrieNode.prototype.addChild = function (character, isCompleteWord) {
        if (isCompleteWord === void 0) { isCompleteWord = false; }
        if (!this.children.has(character)) {
            this.children.set(character, new TrieNode(character, isCompleteWord));
        }
        var childNode = this.children.get(character);
        // In cases similar to adding "car" after "carpet" we need to mark "r" character as complete.
        childNode.isCompleteWord = childNode.isCompleteWord || isCompleteWord;
        return childNode;
    };
    /**
     * 移除结点
     *
     * @param character 字符
     * @return 自身
     */
    TrieNode.prototype.removeChild = function (character) {
        var childNode = this.getChild(character);
        // Delete childNode only if:
        // - childNode has NO children,
        // - childNode.isCompleteWord === false.
        if (childNode
            && !childNode.isCompleteWord
            && !childNode.hasChildren()) {
            this.children.delete(character);
        }
        return this;
    };
    /**
     * 是否有指定子结点
     *
     * @param character 字符
     */
    TrieNode.prototype.hasChild = function (character) {
        return this.children.has(character);
    };
    /**
     * Check whether current TrieNode has children or not.
     */
    TrieNode.prototype.hasChildren = function () {
        return this.children.getKeys().length !== 0;
    };
    /**
     *
     */
    TrieNode.prototype.suggestChildren = function () {
        return __spreadArray([], __read(this.children.getKeys()));
    };
    /**
     * 转换为字符串
     */
    TrieNode.prototype.toString = function () {
        var childrenAsString = this.suggestChildren().toString();
        childrenAsString = childrenAsString ? ":" + childrenAsString : '';
        var isCompleteString = this.isCompleteWord ? '*' : '';
        return "" + this.character + isCompleteString + childrenAsString;
    };
    return TrieNode;
}());

// Character that we will use for trie tree root.
var HEAD_CHARACTER = '*';
/**
 * 字典树
 */
var Trie = /** @class */ (function () {
    function Trie() {
        this.head = new TrieNode(HEAD_CHARACTER);
    }
    /**
     * 添加单词
     *
     * @param word 单词
     * @return 自身
     */
    Trie.prototype.addWord = function (word) {
        var characters = Array.from(word);
        var currentNode = this.head;
        for (var charIndex = 0; charIndex < characters.length; charIndex += 1) {
            var isComplete = charIndex === characters.length - 1;
            currentNode = currentNode.addChild(characters[charIndex], isComplete);
        }
        return this;
    };
    /**
     * 删除单词
     *
     * @param word 单词
     * @return 自身
     */
    Trie.prototype.deleteWord = function (word) {
        var depthFirstDelete = function (currentNode, charIndex) {
            if (charIndex === void 0) { charIndex = 0; }
            if (charIndex >= word.length) {
                // Return if we're trying to delete the character that is out of word's scope.
                return;
            }
            var character = word[charIndex];
            var nextNode = currentNode.getChild(character);
            if (!nextNode) {
                // Return if we're trying to delete a word that has not been added to the Trie.
                return;
            }
            // Go deeper.
            depthFirstDelete(nextNode, charIndex + 1);
            // Since we're going to delete a word let's un-mark its last character isCompleteWord flag.
            if (charIndex === (word.length - 1)) {
                nextNode.isCompleteWord = false;
            }
            // childNode is deleted only if:
            // - childNode has NO children
            // - childNode.isCompleteWord === false
            currentNode.removeChild(character);
        };
        // Start depth-first deletion from the head node.
        depthFirstDelete(this.head);
        return this;
    };
    /**
     *
     * @param word 单词
     */
    Trie.prototype.suggestNextCharacters = function (word) {
        var lastCharacter = this.getLastCharacterNode(word);
        if (!lastCharacter) {
            return null;
        }
        return lastCharacter.suggestChildren();
    };
    /**
     * Check if complete word exists in Trie.
     *
     * @param word 单词
     */
    Trie.prototype.doesWordExist = function (word) {
        var lastCharacter = this.getLastCharacterNode(word);
        return !!lastCharacter && lastCharacter.isCompleteWord;
    };
    /**
     *
     * @param word 单词
     */
    Trie.prototype.getLastCharacterNode = function (word) {
        var characters = Array.from(word);
        var currentNode = this.head;
        for (var charIndex = 0; charIndex < characters.length; charIndex += 1) {
            if (!currentNode.hasChild(characters[charIndex])) {
                return null;
            }
            currentNode = currentNode.getChild(characters[charIndex]);
        }
        return currentNode;
    };
    return Trie;
}());

export { AvlTree, BinarySearchTree, BinarySearchTreeNode, BinaryTreeNode, BloomFilter, BubbleSort, Comparator, ComplexNumber, CountingSort, DisjointSet, DisjointSetItem, DoublyLinkedList, DoublyLinkedListNode, FenwickTree, Graph, GraphEdge, GraphVertex, HashTable, Heap, InsertionSort, KMeans, Knapsack, KnapsackItem, LinkedList, LinkedListNode, MaxHeap, MergeSort, MinHeap, PolynomialHash, PriorityQueue, Queue, QuickSort, QuickSortInPlace, RadixSort, RedBlackTree, SegmentTree, SelectionSort, ShellSort, SimplePolynomialHash, Sort, Stack, Trie, TrieNode, accumulatorBestTimeToBuySellStocks, articulationPoints, backtrackingJumpGame, bellmanFord, bfMaximumSubarray, bfRainTerraces, bfTravellingSalesman, binarySearch, bitLength, bitsDiff, bitsToFloat16, bitsToFloat32, bitsToFloat64, breadthFirstSearch, breadthFirstSearch1, btPowerSet, btUniquePaths, bwPowerSet, caesarCipherDecrypt, caesarCipherEncrypt, cartesianProduct, classicPolynome, clearBit, combinationSum, combineWithRepetitions, combineWithoutRepetitions, countSetBits, dcMaximumSubarraySum, decodeRailFenceCipher, degreeToRadian, depthFirstSearch, depthFirstSearch1, depthFirstSearchRecursive, detectDirectedCycle, detectUndirectedCycle, detectUndirectedCycleUsingDisjointSet, dijkstra, discreteFourierTransform, divideByTwo, dpBestTimeToBuySellStocks, dpBottomUpJumpGame, dpLongestIncreasingSubsequence, dpMaximumSubarray, dpRainTerraces, dpTopDownJumpGame, dpUniquePaths, dqBestTimeToBuySellStocks, encodeRailFenceCipher, euclideanAlgorithm, euclideanAlgorithmIterative, euclideanDistance, eulerianPath, factorial, factorialRecursive, fastFourierTransform, fastPowering, fibonacci, fibonacciNth, fibonacciNthClosedForm, fisherYates, floatAs32BinaryString, floatAs64BinaryString, fullAdder, getBit, getPixel, graphBridges, greedyJumpGame, hammingDistance, hanoiTower, hardyRamanujan, hillCipherDecrypt, hillCipherEncrypt, hornerMethod, integerPartition, interpolationSearch, inverseDiscreteFourierTransform, isEven, isPalindrome, isPositive, isPowerOfTwo, isPowerOfTwo1, isPowerOfTwoBitwise, jumpSearch, kNN, knightTour, knuthMorrisPratt, kruskal, leastCommonMultiple, linearSearch, liuHui, longestCommonSubsequence, longestCommonSubstring, Matrix as matrix, multiply, multiplyByTwo, multiplyUnsigned, nQueens, pascalTriangle, pascalTriangleRecursive, peakvalleyBestTimeToBuySellStocks, permutateWithRepetitions, permutateWithoutRepetitions, prim, primeFactors, rabinKarp, radianToDegree, recursiveStaircaseBF, recursiveStaircaseDP, recursiveStaircaseIT, recursiveStaircaseMEM, regularExpressionMatching, resizeImageWidth, reverseTraversal, setBit, setPixel, shortestCommonSupersequence, sieveOfEratosthenes, squareMatrixRotation, squareRoot, stronglyConnectedComponents, switchSign, topologicalSort, traversal, trialDivision, uniquePaths, weightedRandom, zAlgorithm };
//# sourceMappingURL=index.es.js.map
